/*
	Rollup.js v0.50.0
	Wed Apr 07 2021 16:28:06 GMT+0000 (Coordinated Universal Time) - commit unknown

	https://github.com/rollup/rollup

	Released under the MIT License.
*/

'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var path = require('path');
var path__default = _interopDefault(path);
var fs = require('fs');
var sourcemapCodec = require('sourcemap-codec');
var MagicString = require('magic-string');
var MagicString__default = _interopDefault(MagicString);
var acorn = require('acorn');
var locateCharacter = require('locate-character');
var isReference = _interopDefault(require('is-reference'));
var EventEmitter = _interopDefault(require('events'));
var createFilter = _interopDefault(require('rollup-pluginutils/src/createFilter.js'));
var relative$1 = _interopDefault(require('require-relative'));

var DEBUG = false;
var map = new Map();

var timeStartHelper = void 0;
var timeEndHelper = void 0;

if (typeof process === 'undefined' || typeof process.hrtime === 'undefined') {
	timeStartHelper = function timeStartHelper() {
		return window.performance.now();
	};

	timeEndHelper = function timeEndHelper(previous) {
		return window.performance.now() - previous;
	};
} else {
	timeStartHelper = function timeStartHelper() {
		return process.hrtime();
	};

	timeEndHelper = function timeEndHelper(previous) {
		var hrtime = process.hrtime(previous);
		return hrtime[0] * 1e3 + Math.floor(hrtime[1] / 1e6);
	};
}

function timeStart(label) {
	if (!map.has(label)) {
		map.set(label, {
			time: 0
		});
	}
	map.get(label).start = timeStartHelper();
}

function timeEnd(label) {
	if (map.has(label)) {
		var item = map.get(label);
		item.time += timeEndHelper(item.start);
	}
}

function flushTime() {
	var log = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultLog;
	var _iteratorNormalCompletion = true;
	var _didIteratorError = false;
	var _iteratorError = undefined;

	try {
		for (var _iterator = map.entries()[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
			var item = _step.value;

			log(item[0], item[1].time);
		}
	} catch (err) {
		_didIteratorError = true;
		_iteratorError = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion && _iterator.return) {
				_iterator.return();
			}
		} finally {
			if (_didIteratorError) {
				throw _iteratorError;
			}
		}
	}

	map.clear();
}

function defaultLog(label, time) {
	if (DEBUG) {
		/* eslint-disable no-console */
		console.info('%dms: %s', time, label);
		/* eslint-enable no-console */
	}
}

var absolutePath = /^(?:\/|(?:[A-Za-z]:)?[\\|/])/;
var relativePath = /^\.?\.\//;

function isAbsolute(path$$1) {
	return absolutePath.test(path$$1);
}

function isRelative(path$$1) {
	return relativePath.test(path$$1);
}

function normalize(path$$1) {
	return path$$1.replace(/\\/g, '/');
}

function mkdirpath(path$$1) {
	var dir = path.dirname(path$$1);
	try {
		fs.readdirSync(dir);
	} catch (err) {
		mkdirpath(dir);
		try {
			fs.mkdirSync(dir);
		} catch (err2) {
			if (err2.code !== 'EEXIST') {
				throw err2;
			}
		}
	}
}

function writeFile$1(dest, data) {
	return new Promise(function (fulfil, reject) {
		mkdirpath(dest);

		fs.writeFile(dest, data, function (err) {
			if (err) {
				reject(err);
			} else {
				fulfil();
			}
		});
	});
}

var keys = Object.keys;
function blank() {
	return Object.create(null);
}

function forOwn(object, func) {
	Object.keys(object).forEach(function (key) {
		return func(object[key], key);
	});
}

function assign(target) {
	for (var _len = arguments.length, sources = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
		sources[_key - 1] = arguments[_key];
	}

	sources.forEach(function (source) {
		for (var key in source) {
			if (source.hasOwnProperty(key)) target[key] = source[key];
		}
	});

	return target;
}

function mapSequence(array, fn) {
	var results = [];
	var promise = Promise.resolve();

	function next(member, i) {
		return Promise.resolve(fn(member)).then(function (value) {
			return results[i] = value;
		});
	}

	var _loop = function _loop(i) {
		promise = promise.then(function () {
			return next(array[i], i);
		});
	};

	for (var i = 0; i < array.length; i += 1) {
		_loop(i);
	}

	return promise.then(function () {
		return results;
	});
}

function validateKeys(actualKeys, allowedKeys) {
	var i = actualKeys.length;

	while (i--) {
		var key = actualKeys[i];

		if (allowedKeys.indexOf(key) === -1) {
			return new Error('Unexpected key \'' + key + '\' found, expected one of: ' + allowedKeys.join(', '));
		}
	}
}

function error(props) {
	// use the same constructor as props (if it's an error object)
	// so that err.name is preserved etc
	// (Object.keys below does not update these values because they
	// are properties on the prototype chain)
	// basically if props is a SyntaxError it will not be overriden as a generic Error
	var constructor = props instanceof Error ? props.constructor : Error;
	var err = new constructor(props.message);

	Object.keys(props).forEach(function (key) {
		err[key] = props[key];
	});

	throw err;
}

// this looks ridiculous, but it prevents sourcemap tooling from mistaking
// this for an actual sourceMappingURL
var SOURCEMAPPING_URL = 'sourceMa';
SOURCEMAPPING_URL += 'ppingURL';

var SOURCEMAPPING_URL_RE = new RegExp('^#\\s+' + SOURCEMAPPING_URL + '=.+\\n?');

// Return the first non-falsy result from an array of
// maybe-sync, maybe-promise-returning functions
function first(candidates) {
	return function () {
		for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
			args[_key] = arguments[_key];
		}

		return candidates.reduce(function (promise, candidate) {
			return promise.then(function (result) {
				return result != null ? result : Promise.resolve(candidate.apply(undefined, args));
			});
		}, Promise.resolve());
	};
}

function find(array, fn) {
	for (var i = 0; i < array.length; i += 1) {
		if (fn(array[i], i)) return array[i];
	}

	return null;
}

var reservedWords = 'break case class catch const continue debugger default delete do else export extends finally for function if import in instanceof let new return super switch this throw try typeof var void while with yield enum await implements package protected static interface private public'.split(' ');
var builtins = 'Infinity NaN undefined null true false eval uneval isFinite isNaN parseFloat parseInt decodeURI decodeURIComponent encodeURI encodeURIComponent escape unescape Object Function Boolean Symbol Error EvalError InternalError RangeError ReferenceError SyntaxError TypeError URIError Number Math Date String RegExp Array Int8Array Uint8Array Uint8ClampedArray Int16Array Uint16Array Int32Array Uint32Array Float32Array Float64Array Map Set WeakMap WeakSet SIMD ArrayBuffer DataView JSON Promise Generator GeneratorFunction Reflect Proxy Intl'.split(' ');

var blacklisted = blank();
reservedWords.concat(builtins).forEach(function (word) {
	return blacklisted[word] = true;
});

var illegalCharacters = /[^$_a-zA-Z0-9]/g;

var startsWithDigit = function startsWithDigit(str) {
	return (/\d/.test(str[0])
	);
};

function isLegal(str) {
	if (startsWithDigit(str) || blacklisted[str]) {
		return false;
	}
	if (illegalCharacters.test(str)) {
		return false;
	}
	return true;
}

function makeLegal(str) {
	str = str.replace(/-(\w)/g, function (_, letter) {
		return letter.toUpperCase();
	}).replace(illegalCharacters, '_');

	if (startsWithDigit(str) || blacklisted[str]) str = '_' + str;

	return str;
}

function spaces(i) {
	var result = '';
	while (i--) {
		result += ' ';
	}return result;
}

function tabsToSpaces(str) {
	return str.replace(/^\t+/, function (match) {
		return match.split('\t').join('  ');
	});
}

function getCodeFrame(source, line, column) {
	var lines = source.split('\n');

	var frameStart = Math.max(0, line - 3);
	var frameEnd = Math.min(line + 2, lines.length);

	lines = lines.slice(frameStart, frameEnd);
	while (!/\S/.test(lines[lines.length - 1])) {
		lines.pop();
		frameEnd -= 1;
	}

	var digits = String(frameEnd).length;

	return lines.map(function (str, i) {
		var isErrorLine = frameStart + i + 1 === line;

		var lineNum = String(i + frameStart + 1);
		while (lineNum.length < digits) {
			lineNum = ' ' + lineNum;
		}if (isErrorLine) {
			var indicator = spaces(digits + 2 + tabsToSpaces(str.slice(0, column)).length) + '^';
			return lineNum + ': ' + tabsToSpaces(str) + '\n' + indicator;
		}

		return lineNum + ': ' + tabsToSpaces(str);
	}).join('\n');
}

function relativeId(id) {
	if (typeof process === 'undefined' || !isAbsolute(id)) return id;
	return path.relative(process.cwd(), id);
}

var _createClass$3 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$3(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Variable = function () {
	function Variable(name) {
		_classCallCheck$3(this, Variable);

		this.name = name;
	}

	_createClass$3(Variable, [{
		key: "addCall",
		value: function addCall() {}
	}, {
		key: "getName",
		value: function getName() {
			return this.name;
		}
	}, {
		key: "hasEffectsWhenCalled",
		value: function hasEffectsWhenCalled() {
			return true;
		}
	}, {
		key: "hasEffectsWhenMutated",
		value: function hasEffectsWhenMutated() {
			return true;
		}
	}, {
		key: "includeVariable",
		value: function includeVariable() {
			if (this.included) {
				return false;
			}
			this.included = true;
			return true;
		}
	}]);

	return Variable;
}();

var _createClass$2 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$2(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var NamespaceVariable = function (_Variable) {
	_inherits(NamespaceVariable, _Variable);

	function NamespaceVariable(module) {
		_classCallCheck$2(this, NamespaceVariable);

		var _this = _possibleConstructorReturn(this, (NamespaceVariable.__proto__ || Object.getPrototypeOf(NamespaceVariable)).call(this, module.basename()));

		_this.isNamespace = true;
		_this.module = module;
		_this.needsNamespaceBlock = false;

		_this.originals = blank();
		module.getExports().concat(module.getReexports()).forEach(function (name) {
			_this.originals[name] = module.traceExport(name);
		});
		return _this;
	}

	_createClass$2(NamespaceVariable, [{
		key: 'addReference',
		value: function addReference(node) {
			this.name = node.name;
		}
	}, {
		key: 'assignExpression',
		value: function assignExpression() {}
	}, {
		key: 'includeVariable',
		value: function includeVariable() {
			var hasBeenIncluded = _get(NamespaceVariable.prototype.__proto__ || Object.getPrototypeOf(NamespaceVariable.prototype), 'includeVariable', this).call(this);
			if (hasBeenIncluded) {
				this.needsNamespaceBlock = true;
				forOwn(this.originals, function (original) {
					return original.includeVariable();
				});
			}
			return hasBeenIncluded;
		}
	}, {
		key: 'renderBlock',
		value: function renderBlock(es, legacy, indentString) {
			var _this2 = this;

			var members = keys(this.originals).map(function (name) {
				var original = _this2.originals[name];

				if (original.isReassigned && !legacy) {
					return indentString + 'get ' + name + ' () { return ' + original.getName(es) + '; }';
				}

				if (legacy && ~reservedWords.indexOf(name)) name = '\'' + name + '\'';
				return '' + indentString + name + ': ' + original.getName(es);
			});

			var callee = legacy ? '(Object.freeze || Object)' : 'Object.freeze';
			return this.module.bundle.varOrConst + ' ' + this.getName(es) + ' = ' + callee + '({\n' + members.join(',\n') + '\n});\n\n';
		}
	}]);

	return NamespaceVariable;
}(Variable);

function extractNames(param) {
	var names = [];
	extractors[param.type](names, param);
	return names;
}

var extractors = {
	Identifier: function Identifier(names, param) {
		names.push(param.name);
	},
	ObjectPattern: function ObjectPattern(names, param) {
		param.properties.forEach(function (prop) {
			extractors[prop.value.type](names, prop.value);
		});
	},
	ArrayPattern: function ArrayPattern(names, param) {
		param.elements.forEach(function (element) {
			if (element) extractors[element.type](names, element);
		});
	},
	RestElement: function RestElement(names, param) {
		extractors[param.argument.type](names, param.argument);
	},
	AssignmentPattern: function AssignmentPattern(names, param) {
		extractors[param.left.type](names, param.left);
	}
};

var UNKNOWN_VALUE = { toString: function toString() {
		return '[[UNKNOWN]]';
	} };

var UNKNOWN_ASSIGNMENT = {
	type: 'UNKNOWN',
	bindCall: function bindCall() {},
	hasEffectsWhenCalled: function hasEffectsWhenCalled() {
		return true;
	},
	hasEffectsWhenMutated: function hasEffectsWhenMutated() {
		return true;
	}
};

var UNDEFINED_ASSIGNMENT = {
	type: 'UNDEFINED',
	bindCall: function bindCall() {},
	hasEffectsWhenCalled: function hasEffectsWhenCalled() {
		return true;
	},
	hasEffectsWhenMutated: function hasEffectsWhenMutated() {
		return true;
	}
};

var UNKNOWN_OBJECT_LITERAL = {
	type: 'UNKNOWN_OBJECT_LITERAL',
	bindCall: function bindCall() {},
	hasEffectsWhenCalled: function hasEffectsWhenCalled() {
		return true;
	},
	hasEffectsWhenMutated: function hasEffectsWhenMutated() {
		return false;
	}
};

var _createClass$6 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck$6(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var OPTION_IGNORE_BREAK_STATEMENTS = 'IGNORE_BREAK_STATEMENTS';
var OPTION_IGNORE_RETURN_AWAIT_YIELD = 'IGNORE_RETURN_AWAIT_YIELD';
var OPTION_IGNORE_SAFE_THIS_MUTATIONS = 'IGNORE_SAFE_THIS_MUTATIONS';
var OPTION_CALLED_NODES = 'CALLED_NODES';
var OPTION_MUTATED_NODES = 'MUTATED_NODES';
var IGNORED_LABELS = 'IGNORED_LABELS';

/** Wrapper to ensure immutability */

var ExecutionPathOptions = function () {
	_createClass$6(ExecutionPathOptions, null, [{
		key: 'create',

		/**
   * @returns {ExecutionPathOptions}
   */
		value: function create() {
			return new this({});
		}
	}]);

	function ExecutionPathOptions(optionValues) {
		_classCallCheck$6(this, ExecutionPathOptions);

		this._optionValues = optionValues;
	}

	/**
  * Returns a new ExecutionPathOptions instance with the given option set to a new value.
  * Does not mutate the current instance. Also works in sub-classes.
  * @param {string} option - The name of an option
  * @param {*} value - The new value of the option
  * @returns {ExecutionPathOptions} A new options instance
  */


	_createClass$6(ExecutionPathOptions, [{
		key: 'set',
		value: function set(option, value) {
			return new this.constructor(Object.assign({}, this._optionValues, _defineProperty({}, option, value)));
		}

		/**
   * @param {string} option - The name of an option
   * @returns {*} Its value
   */

	}, {
		key: 'get',
		value: function get(option) {
			return this._optionValues[option];
		}

		/**
   * @return {boolean}
   */

	}, {
		key: 'ignoreBreakStatements',
		value: function ignoreBreakStatements() {
			return this.get(OPTION_IGNORE_BREAK_STATEMENTS);
		}

		/**
   * @param {boolean} [value=true]
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'setIgnoreBreakStatements',
		value: function setIgnoreBreakStatements() {
			var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

			return this.set(OPTION_IGNORE_BREAK_STATEMENTS, value);
		}

		/**
   * @param {string} labelName
   * @return {boolean}
   */

	}, {
		key: 'ignoreLabel',
		value: function ignoreLabel(labelName) {
			var ignoredLabels = this.get(IGNORED_LABELS);
			return ignoredLabels && ignoredLabels.has(labelName);
		}

		/**
   * @param {string} labelName
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'setIgnoreLabel',
		value: function setIgnoreLabel(labelName) {
			return this.set(IGNORED_LABELS, new Set(this.get(IGNORED_LABELS)).add(labelName));
		}

		/**
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'setIgnoreNoLabels',
		value: function setIgnoreNoLabels() {
			return this.set(IGNORED_LABELS, null);
		}

		/**
   * @return {boolean}
   */

	}, {
		key: 'ignoreReturnAwaitYield',
		value: function ignoreReturnAwaitYield() {
			return this.get(OPTION_IGNORE_RETURN_AWAIT_YIELD);
		}

		/**
   * @param {boolean} [value=true]
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'setIgnoreReturnAwaitYield',
		value: function setIgnoreReturnAwaitYield() {
			var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

			return this.set(OPTION_IGNORE_RETURN_AWAIT_YIELD, value);
		}

		/**
   * @return {boolean}
   */

	}, {
		key: 'ignoreSafeThisMutations',
		value: function ignoreSafeThisMutations() {
			return this.get(OPTION_IGNORE_SAFE_THIS_MUTATIONS);
		}

		/**
   * @param {boolean} [value=true]
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'setIgnoreSafeThisMutations',
		value: function setIgnoreSafeThisMutations() {
			var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

			return this.set(OPTION_IGNORE_SAFE_THIS_MUTATIONS, value);
		}

		/**
   * @param {Node} node
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'addMutatedNode',
		value: function addMutatedNode(node) {
			return this.set(OPTION_MUTATED_NODES, new Set(this.get(OPTION_MUTATED_NODES)).add(node));
		}

		/**
   * @param {Node} node
   * @return {boolean}
   */

	}, {
		key: 'hasNodeBeenMutated',
		value: function hasNodeBeenMutated(node) {
			var mutatedNodes = this.get(OPTION_MUTATED_NODES);
			return mutatedNodes && mutatedNodes.has(node);
		}

		/**
   * @param {Node} node
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'addCalledNode',
		value: function addCalledNode(node) {
			return this.set(OPTION_CALLED_NODES, new Set(this.get(OPTION_CALLED_NODES)).add(node));
		}

		/**
   * @param {Node} node
   * @return {boolean}
   */

	}, {
		key: 'hasNodeBeenCalled',
		value: function hasNodeBeenCalled(node) {
			var calledNodes = this.get(OPTION_CALLED_NODES);
			return calledNodes && calledNodes.has(node);
		}

		/**
   * @param {Node} calledNode
   * @return {ExecutionPathOptions}
   */

	}, {
		key: 'getHasEffectsWhenCalledOptions',
		value: function getHasEffectsWhenCalledOptions(calledNode) {
			return this.addCalledNode(calledNode).setIgnoreReturnAwaitYield().setIgnoreBreakStatements(false).setIgnoreNoLabels();
		}
	}]);

	return ExecutionPathOptions;
}();

var _createClass$5 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$5(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Node = function () {
	function Node() {
		_classCallCheck$5(this, Node);
	}

	_createClass$5(Node, [{
		key: 'bind',

		/**
   * Called once all nodes have been initialised and the scopes have been populated.
   * Use this to bind assignments and calls to variables.
   */
		value: function bind() {
			this.eachChild(function (child) {
				return child.bind();
			});
		}

		/**
   * Bind an expression as an assignment to a node.
   * The default noop implementation is ok as long as hasEffectsWhenAssigned
   * always returns true for this node. Otherwise it should be overridden.
   * @param {Node} expression
   */

	}, {
		key: 'bindAssignment',
		value: function bindAssignment() {}

		/**
   * Binds ways a node is called to a node. Current options are:
   * - withNew: boolean - Did this call use the "new" operator
   * The default noop implementation is ok as long as hasEffectsWhenCalled
   * always returns true for this node. Otherwise it should be overridden.
   * @param callOptions
   */

	}, {
		key: 'bindCall',
		value: function bindCall() {}
	}, {
		key: 'eachChild',
		value: function eachChild(callback) {
			var _this = this;

			this.keys.forEach(function (key) {
				var value = _this[key];
				if (!value) return;

				if (Array.isArray(value)) {
					value.forEach(function (child) {
						return child && callback(child);
					});
				} else {
					callback(value);
				}
			});
		}
	}, {
		key: 'getValue',
		value: function getValue() {
			return UNKNOWN_VALUE;
		}

		/**
   * Determine if this Node would have an effect on the bundle.
   * This is usually true for already included nodes. Exceptions are e.g. break statements
   * which only have an effect if their surrounding loop or switch statement is included.
   * The options pass on information like this about the current execution path.
   * @param {ExecutionPathOptions} options
   * @return {boolean}
   */

	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.included || this.someChild(function (child) {
				return child.hasEffects(options);
			});
		}

		/**
   * Special make-shift logic to treat cases where apparently side-effect free statements
   * are executed for side-effects. The most important case are getters with side-effects.
   * Once we can reliably handle this case in member expressions, this function should
   * probably be removed again.
   * @param {ExecutionPathOptions} options
   * @return {boolean}
   */

	}, {
		key: 'hasEffectsAsExpressionStatement',
		value: function hasEffectsAsExpressionStatement() {
			return true;
		}

		/**
   * @param {ExecutionPathOptions} options
   * @return {boolean}
   */

	}, {
		key: 'hasEffectsWhenAssigned',
		value: function hasEffectsWhenAssigned() {
			return true;
		}

		/**
   * @param {ExecutionPathOptions} options
   * @return {boolean}
   */

	}, {
		key: 'hasEffectsWhenCalled',
		value: function hasEffectsWhenCalled() {
			return true;
		}

		/**
   * @param {ExecutionPathOptions} options
   * @return {boolean}
   */

	}, {
		key: 'hasEffectsWhenMutated',
		value: function hasEffectsWhenMutated() {
			return true;
		}

		/**
   * Includes the node in the bundle. Children are usually included if they are
   * necessary for this node (e.g. a function body) or if they have effects.
   * Necessary variables should be included as well. Should return true if any
   * nodes or variables have been added that were missing before.
   * @return {boolean}
   */

	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			if (this.isFullyIncluded()) return false;
			var addedNewNodes = false;
			this.eachChild(function (childNode) {
				if (childNode.includeInBundle()) {
					addedNewNodes = true;
				}
			});
			if (this.included && !addedNewNodes) {
				return false;
			}
			this.included = true;
			return true;
		}

		/**
   * Alternative version of includeInBundle to override the default behaviour of
   * declarations to only include nodes for declarators that have an effect. Necessary
   * for for-loops that do not use a declared loop variable.
   * @return {boolean}
   */

	}, {
		key: 'includeWithAllDeclarations',
		value: function includeWithAllDeclarations() {
			return this.includeInBundle();
		}

		/**
   * Assign a scope to this node and make sure all children have the right scopes.
   * Perform any additional initialisation that does not depend on the scope being
   * populated with variables.
   * Usually one should not override this function but override initialiseScope,
   * initialiseNode and/or initialiseChildren instead. BlockScopes have a special
   * alternative initialisation initialiseAndReplaceScope.
   * @param {Scope} parentScope
   */

	}, {
		key: 'initialise',
		value: function initialise(parentScope) {
			this.initialiseScope(parentScope);
			this.initialiseNode(parentScope);
			this.initialiseChildren(parentScope);
		}

		/**
   * Override to change how and with what scopes children are initialised
   * @param {Scope} parentScope
   */

	}, {
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			var _this2 = this;

			this.eachChild(function (child) {
				return child.initialise(_this2.scope);
			});
		}

		/**
   * Override to perform special initialisation steps after the scope is initialised
   * @param {Scope} parentScope
   */

	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {}

		/**
   * Override if this scope should receive a different scope than the parent scope.
   * @param {Scope} parentScope
   */

	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = parentScope;
		}
	}, {
		key: 'insertSemicolon',
		value: function insertSemicolon(code) {
			if (code.original[this.end - 1] !== ';') {
				code.appendLeft(this.end, ';');
			}
		}

		/**
   * Shortcut to skip checking this node for effects when all children have already
   * been included.
   * @param {Scope} parentScope
   */

	}, {
		key: 'isFullyIncluded',
		value: function isFullyIncluded() {
			if (this._fullyIncluded) {
				return true;
			}
			this._fullyIncluded = this.included && !this.someChild(function (child) {
				return !child.isFullyIncluded();
			});
		}
	}, {
		key: 'locate',
		value: function locate$$1() {
			// useful for debugging
			var location = locateCharacter.locate(this.module.code, this.start, { offsetLine: 1 });
			location.file = this.module.id;
			location.toString = function () {
				return JSON.stringify(location);
			};

			return location;
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			this.eachChild(function (child) {
				return child.render(code, es);
			});
		}

		/**
   * Start a new execution path to determine if this node has an effect on the bundle and
   * should therefore be included. Unless they are fully included, included nodes should
   * always be included again in subsequent visits as the inclusion of additional variables
   * may require the inclusion of more child nodes in e.g. block statements.
   * @return {boolean}
   */

	}, {
		key: 'shouldBeIncluded',
		value: function shouldBeIncluded() {
			return this.hasEffects(ExecutionPathOptions.create());
		}
	}, {
		key: 'someChild',
		value: function someChild(callback) {
			var _this3 = this;

			return this.keys.some(function (key) {
				var value = _this3[key];
				if (!value) return false;

				if (Array.isArray(value)) {
					return value.some(function (child) {
						return child && callback(child);
					});
				}
				return callback(value);
			});
		}
	}, {
		key: 'toString',
		value: function toString() {
			return this.module.code.slice(this.start, this.end);
		}
	}]);

	return Node;
}();

var _createClass$4 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$4(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$1(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$1(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ArrayPattern = function (_Node) {
	_inherits$1(ArrayPattern, _Node);

	function ArrayPattern() {
		_classCallCheck$4(this, ArrayPattern);

		return _possibleConstructorReturn$1(this, (ArrayPattern.__proto__ || Object.getPrototypeOf(ArrayPattern)).apply(this, arguments));
	}

	_createClass$4(ArrayPattern, [{
		key: 'bindAssignment',
		value: function bindAssignment() {
			this.eachChild(function (child) {
				return child.bindAssignment(UNKNOWN_ASSIGNMENT);
			});
		}
	}, {
		key: 'hasEffectsWhenAssigned',
		value: function hasEffectsWhenAssigned(options) {
			return this.someChild(function (child) {
				return child.hasEffectsWhenAssigned(options);
			});
		}
	}, {
		key: 'initialiseAndDeclare',
		value: function initialiseAndDeclare(parentScope, kind) {
			this.initialiseScope(parentScope);
			this.eachChild(function (child) {
				return child.initialiseAndDeclare(parentScope, kind, UNKNOWN_ASSIGNMENT);
			});
		}
	}]);

	return ArrayPattern;
}(Node);

var _createClass$9 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$1 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$9(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$3(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$3(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var LocalVariable = function (_Variable) {
	_inherits$3(LocalVariable, _Variable);

	function LocalVariable(name, declarator, init) {
		_classCallCheck$9(this, LocalVariable);

		var _this = _possibleConstructorReturn$3(this, (LocalVariable.__proto__ || Object.getPrototypeOf(LocalVariable)).call(this, name));

		_this.isReassigned = false;
		_this.exportName = null;
		_this.declarations = new Set(declarator ? [declarator] : null);
		_this.assignedExpressions = new Set(init ? [init] : null);
		_this.calls = new Set();
		return _this;
	}

	_createClass$9(LocalVariable, [{
		key: 'addDeclaration',
		value: function addDeclaration(identifier) {
			this.declarations.add(identifier);
		}
	}, {
		key: 'addCall',
		value: function addCall(callOptions) {
			// To prevent infinite loops
			if (this.calls.has(callOptions)) return;
			this.calls.add(callOptions);
			Array.from(this.assignedExpressions).forEach(function (expression) {
				return expression.bindCall(callOptions);
			});
		}
	}, {
		key: 'addReference',
		value: function addReference() {}
	}, {
		key: 'assignExpression',
		value: function assignExpression(expression) {
			this.assignedExpressions.add(expression);
			this.isReassigned = true;
			Array.from(this.calls).forEach(function (callOptions) {
				return expression.bindCall(callOptions);
			});
		}
	}, {
		key: 'getName',
		value: function getName(es) {
			if (es) return this.name;
			if (!this.isReassigned || !this.exportName) return this.name;

			return 'exports.' + this.exportName;
		}
	}, {
		key: 'hasEffectsWhenCalled',
		value: function hasEffectsWhenCalled(options) {
			return Array.from(this.assignedExpressions).some(function (node) {
				return !options.hasNodeBeenCalled(node) && node.hasEffectsWhenCalled(options.getHasEffectsWhenCalledOptions(node));
			});
		}
	}, {
		key: 'hasEffectsWhenMutated',
		value: function hasEffectsWhenMutated(options) {
			return this.included || Array.from(this.assignedExpressions).some(function (node) {
				return !options.hasNodeBeenMutated(node) && node.hasEffectsWhenMutated(options.addMutatedNode(node));
			});
		}
	}, {
		key: 'includeVariable',
		value: function includeVariable() {
			var hasBeenIncluded = _get$1(LocalVariable.prototype.__proto__ || Object.getPrototypeOf(LocalVariable.prototype), 'includeVariable', this).call(this);
			if (hasBeenIncluded) {
				this.declarations.forEach(function (identifier) {
					return identifier.includeInBundle();
				});
			}
			return hasBeenIncluded;
		}
	}, {
		key: 'toString',
		value: function toString() {
			return this.name;
		}
	}]);

	return LocalVariable;
}(Variable);

var _createClass$10 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$10(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$4(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$4(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ParameterVariable = function (_LocalVariable) {
	_inherits$4(ParameterVariable, _LocalVariable);

	function ParameterVariable(name, declarator) {
		_classCallCheck$10(this, ParameterVariable);

		return _possibleConstructorReturn$4(this, (ParameterVariable.__proto__ || Object.getPrototypeOf(ParameterVariable)).call(this, name, declarator, UNKNOWN_ASSIGNMENT));
	}

	_createClass$10(ParameterVariable, [{
		key: 'getName',
		value: function getName() {
			return this.name;
		}
	}]);

	return ParameterVariable;
}(LocalVariable);

var _createClass$8 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$8(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Scope = function () {
	function Scope() {
		var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

		_classCallCheck$8(this, Scope);

		this.parent = options.parent;
		this.isModuleScope = !!options.isModuleScope;

		this.children = [];
		if (this.parent) this.parent.children.push(this);

		this.variables = blank();
	}

	_createClass$8(Scope, [{
		key: 'addDeclaration',
		value: function addDeclaration(identifier, isHoisted, init) {
			var name = identifier.name;
			if (this.variables[name]) {
				var variable = this.variables[name];
				variable.addDeclaration(identifier);
				init && variable.assignExpression(init);
			} else {
				this.variables[name] = new LocalVariable(identifier.name, identifier, init);
			}
		}
	}, {
		key: 'addParameterDeclaration',
		value: function addParameterDeclaration(identifier) {
			var name = identifier.name;
			this.variables[name] = new ParameterVariable(name, identifier);
		}
	}, {
		key: 'contains',
		value: function contains(name) {
			return !!this.variables[name] || (this.parent ? this.parent.contains(name) : false);
		}
	}, {
		key: 'deshadow',
		value: function deshadow(names) {
			var _this = this;

			keys(this.variables).forEach(function (key) {
				var declaration = _this.variables[key];

				// we can disregard exports.foo etc
				if (declaration.exportName && declaration.isReassigned) return;

				var name = declaration.getName(true);
				var deshadowed = name;

				var i = 1;

				while (names.has(deshadowed)) {
					deshadowed = name + '$$' + i++;
				}

				declaration.name = deshadowed;
			});

			this.children.forEach(function (scope) {
				return scope.deshadow(names);
			});
		}
	}, {
		key: 'findLexicalBoundary',
		value: function findLexicalBoundary() {
			return this.parent.findLexicalBoundary();
		}
	}, {
		key: 'findVariable',
		value: function findVariable(name) {
			return this.variables[name] || this.parent && this.parent.findVariable(name);
		}
	}]);

	return Scope;
}();

var _createClass$7 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$7(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$2(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$2(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ArrowFunctionExpression = function (_Node) {
	_inherits$2(ArrowFunctionExpression, _Node);

	function ArrowFunctionExpression() {
		_classCallCheck$7(this, ArrowFunctionExpression);

		return _possibleConstructorReturn$2(this, (ArrowFunctionExpression.__proto__ || Object.getPrototypeOf(ArrowFunctionExpression)).apply(this, arguments));
	}

	_createClass$7(ArrowFunctionExpression, [{
		key: 'bindCall',

		// Should receive an implementation once we start tracking parameter values
		value: function bindCall() {}
	}, {
		key: 'hasEffects',
		value: function hasEffects() {
			return this.included;
		}
	}, {
		key: 'hasEffectsWhenCalled',
		value: function hasEffectsWhenCalled(options) {
			return this.params.some(function (param) {
				return param.hasEffects(options);
			}) || this.body.hasEffects(options);
		}
	}, {
		key: 'hasEffectsWhenMutated',
		value: function hasEffectsWhenMutated() {
			return this.included;
		}
	}, {
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			var _this2 = this;

			this.params.forEach(function (param) {
				return param.initialiseAndDeclare(_this2.scope, 'parameter');
			});
			if (this.body.initialiseAndReplaceScope) {
				this.body.initialiseAndReplaceScope(new Scope({ parent: this.scope }));
			} else {
				this.body.initialise(this.scope);
			}
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new Scope({ parent: parentScope });
		}
	}]);

	return ArrowFunctionExpression;
}(Node);

// TODO tidy this up a bit (e.g. they can both use node.module.imports)
function disallowIllegalReassignment(scope, node) {
	if (node.type === 'MemberExpression' && node.object.type === 'Identifier') {
		var variable = scope.findVariable(node.object.name);
		if (variable.isNamespace) {
			node.module.error({
				code: 'ILLEGAL_NAMESPACE_REASSIGNMENT',
				message: 'Illegal reassignment to import \'' + node.object.name + '\''
			}, node.start);
		}
	} else if (node.type === 'Identifier') {
		if (node.module.imports[node.name] && !scope.contains(node.name)) {
			node.module.error({
				code: 'ILLEGAL_REASSIGNMENT',
				message: 'Illegal reassignment to import \'' + node.name + '\''
			}, node.start);
		}
	}
}

var _createClass$11 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$2 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$11(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$5(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$5(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var AssignmentExpression = function (_Node) {
	_inherits$5(AssignmentExpression, _Node);

	function AssignmentExpression() {
		_classCallCheck$11(this, AssignmentExpression);

		return _possibleConstructorReturn$5(this, (AssignmentExpression.__proto__ || Object.getPrototypeOf(AssignmentExpression)).apply(this, arguments));
	}

	_createClass$11(AssignmentExpression, [{
		key: 'bind',
		value: function bind() {
			_get$2(AssignmentExpression.prototype.__proto__ || Object.getPrototypeOf(AssignmentExpression.prototype), 'bind', this).call(this);
			disallowIllegalReassignment(this.scope, this.left);
			this.left.bindAssignment(this.right);
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return _get$2(AssignmentExpression.prototype.__proto__ || Object.getPrototypeOf(AssignmentExpression.prototype), 'hasEffects', this).call(this, options) || this.left.hasEffectsWhenAssigned(options);
		}
	}, {
		key: 'hasEffectsAsExpressionStatement',
		value: function hasEffectsAsExpressionStatement(options) {
			return this.hasEffects(options);
		}
	}]);

	return AssignmentExpression;
}(Node);

var _createClass$12 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$3 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$12(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$6(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$6(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var AssignmentPattern = function (_Node) {
	_inherits$6(AssignmentPattern, _Node);

	function AssignmentPattern() {
		_classCallCheck$12(this, AssignmentPattern);

		return _possibleConstructorReturn$6(this, (AssignmentPattern.__proto__ || Object.getPrototypeOf(AssignmentPattern)).apply(this, arguments));
	}

	_createClass$12(AssignmentPattern, [{
		key: 'bind',
		value: function bind() {
			_get$3(AssignmentPattern.prototype.__proto__ || Object.getPrototypeOf(AssignmentPattern.prototype), 'bind', this).call(this);
			this.left.bindAssignment(this.right);
		}
	}, {
		key: 'bindAssignment',
		value: function bindAssignment(expression) {
			this.left.bindAssignment(expression);
		}
	}, {
		key: 'hasEffectsWhenAssigned',
		value: function hasEffectsWhenAssigned(options) {
			return this.left.hasEffectsWhenAssigned(options);
		}
	}, {
		key: 'initialiseAndDeclare',
		value: function initialiseAndDeclare(parentScope, kind, init) {
			this.initialiseScope(parentScope);
			this.right.initialise(parentScope);
			this.left.initialiseAndDeclare(parentScope, kind, init);
		}
	}]);

	return AssignmentPattern;
}(Node);

var _createClass$13 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$4 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$13(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$7(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$7(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var AwaitExpression = function (_Node) {
	_inherits$7(AwaitExpression, _Node);

	function AwaitExpression() {
		_classCallCheck$13(this, AwaitExpression);

		return _possibleConstructorReturn$7(this, (AwaitExpression.__proto__ || Object.getPrototypeOf(AwaitExpression)).apply(this, arguments));
	}

	_createClass$13(AwaitExpression, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return _get$4(AwaitExpression.prototype.__proto__ || Object.getPrototypeOf(AwaitExpression.prototype), 'hasEffects', this).call(this, options) || !options.ignoreReturnAwaitYield();
		}
	}, {
		key: 'hasEffectsAsExpressionStatement',
		value: function hasEffectsAsExpressionStatement(options) {
			return this.hasEffects(options);
		}
	}]);

	return AwaitExpression;
}(Node);

var _createClass$14 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$14(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$8(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$8(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var operators = {
	'==': function _(left, right) {
		return left == right;
	},
	'!=': function _(left, right) {
		return left != right;
	},
	'===': function _(left, right) {
		return left === right;
	},
	'!==': function _(left, right) {
		return left !== right;
	},
	'<': function _(left, right) {
		return left < right;
	},
	'<=': function _(left, right) {
		return left <= right;
	},
	'>': function _(left, right) {
		return left > right;
	},
	'>=': function _(left, right) {
		return left >= right;
	},
	'<<': function _(left, right) {
		return left << right;
	},
	'>>': function _(left, right) {
		return left >> right;
	},
	'>>>': function _(left, right) {
		return left >>> right;
	},
	'+': function _(left, right) {
		return left + right;
	},
	'-': function _(left, right) {
		return left - right;
	},
	'*': function _(left, right) {
		return left * right;
	},
	'/': function _(left, right) {
		return left / right;
	},
	'%': function _(left, right) {
		return left % right;
	},
	'|': function _(left, right) {
		return left | right;
	},
	'^': function _(left, right) {
		return left ^ right;
	},
	'&': function _(left, right) {
		return left & right;
	},
	'**': function _(left, right) {
		return Math.pow(left, right);
	},
	in: function _in(left, right) {
		return left in right;
	},
	instanceof: function _instanceof(left, right) {
		return left instanceof right;
	}
};

var BinaryExpression = function (_Node) {
	_inherits$8(BinaryExpression, _Node);

	function BinaryExpression() {
		_classCallCheck$14(this, BinaryExpression);

		return _possibleConstructorReturn$8(this, (BinaryExpression.__proto__ || Object.getPrototypeOf(BinaryExpression)).apply(this, arguments));
	}

	_createClass$14(BinaryExpression, [{
		key: 'getValue',
		value: function getValue() {
			var leftValue = this.left.getValue();
			if (leftValue === UNKNOWN_VALUE) return UNKNOWN_VALUE;

			var rightValue = this.right.getValue();
			if (rightValue === UNKNOWN_VALUE) return UNKNOWN_VALUE;

			if (!operators[this.operator]) return UNKNOWN_VALUE;

			return operators[this.operator](leftValue, rightValue);
		}
	}]);

	return BinaryExpression;
}(Node);

var _createClass$16 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$5 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$16(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$10(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$10(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Statement = function (_Node) {
	_inherits$10(Statement, _Node);

	function Statement() {
		_classCallCheck$16(this, Statement);

		return _possibleConstructorReturn$10(this, (Statement.__proto__ || Object.getPrototypeOf(Statement)).apply(this, arguments));
	}

	_createClass$16(Statement, [{
		key: 'render',
		value: function render(code, es) {
			if (!this.module.bundle.treeshake || this.included) {
				_get$5(Statement.prototype.__proto__ || Object.getPrototypeOf(Statement.prototype), 'render', this).call(this, code, es);
			} else {
				code.remove(this.leadingCommentStart || this.start, this.next || this.end);
			}
		}
	}]);

	return Statement;
}(Node);

var _createClass$17 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$6 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$17(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$11(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$11(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BlockScope = function (_Scope) {
	_inherits$11(BlockScope, _Scope);

	function BlockScope() {
		_classCallCheck$17(this, BlockScope);

		return _possibleConstructorReturn$11(this, (BlockScope.__proto__ || Object.getPrototypeOf(BlockScope)).apply(this, arguments));
	}

	_createClass$17(BlockScope, [{
		key: 'addDeclaration',
		value: function addDeclaration(identifier, isHoisted, init) {
			if (isHoisted) {
				this.parent.addDeclaration(identifier, isHoisted, init);
			} else {
				_get$6(BlockScope.prototype.__proto__ || Object.getPrototypeOf(BlockScope.prototype), 'addDeclaration', this).call(this, identifier, false, init);
			}
		}
	}]);

	return BlockScope;
}(Scope);

var _createClass$15 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$15(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$9(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$9(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BlockStatement = function (_Statement) {
	_inherits$9(BlockStatement, _Statement);

	function BlockStatement() {
		_classCallCheck$15(this, BlockStatement);

		return _possibleConstructorReturn$9(this, (BlockStatement.__proto__ || Object.getPrototypeOf(BlockStatement)).apply(this, arguments));
	}

	_createClass$15(BlockStatement, [{
		key: 'bind',
		value: function bind() {
			this.body.forEach(function (node) {
				return node.bind();
			});
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			// Empty block statements do not have effects even though they may be included as e.g. function body
			return this.body.some(function (child) {
				return child.hasEffects(options);
			});
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			if (this.isFullyIncluded()) return false;
			var addedNewNodes = false;
			this.body.forEach(function (node) {
				if (node.shouldBeIncluded()) {
					if (node.includeInBundle()) {
						addedNewNodes = true;
					}
				}
			});
			if (!this.included || addedNewNodes) {
				this.included = true;
				return true;
			}
			return false;
		}
	}, {
		key: 'initialiseAndReplaceScope',
		value: function initialiseAndReplaceScope(scope) {
			this.scope = scope;
			this.initialiseNode();
			this.initialiseChildren(scope);
		}
	}, {
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			var lastNode = void 0;
			var _iteratorNormalCompletion = true;
			var _didIteratorError = false;
			var _iteratorError = undefined;

			try {
				for (var _iterator = this.body[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
					var node = _step.value;

					node.initialise(this.scope);

					if (lastNode) lastNode.next = node.start;
					lastNode = node;
				}
			} catch (err) {
				_didIteratorError = true;
				_iteratorError = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion && _iterator.return) {
						_iterator.return();
					}
				} finally {
					if (_didIteratorError) {
						throw _iteratorError;
					}
				}
			}
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new BlockScope({ parent: parentScope });
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			if (this.body.length) {
				var _iteratorNormalCompletion2 = true;
				var _didIteratorError2 = false;
				var _iteratorError2 = undefined;

				try {
					for (var _iterator2 = this.body[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
						var node = _step2.value;

						node.render(code, es);
					}
				} catch (err) {
					_didIteratorError2 = true;
					_iteratorError2 = err;
				} finally {
					try {
						if (!_iteratorNormalCompletion2 && _iterator2.return) {
							_iterator2.return();
						}
					} finally {
						if (_didIteratorError2) {
							throw _iteratorError2;
						}
					}
				}
			} else {
				Statement.prototype.render.call(this, code, es);
			}
		}
	}]);

	return BlockStatement;
}(Statement);

var _createClass$18 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$7 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$18(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$12(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$12(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BreakStatement = function (_Node) {
	_inherits$12(BreakStatement, _Node);

	function BreakStatement() {
		_classCallCheck$18(this, BreakStatement);

		return _possibleConstructorReturn$12(this, (BreakStatement.__proto__ || Object.getPrototypeOf(BreakStatement)).apply(this, arguments));
	}

	_createClass$18(BreakStatement, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return _get$7(BreakStatement.prototype.__proto__ || Object.getPrototypeOf(BreakStatement.prototype), 'hasEffects', this).call(this, options) || !options.ignoreBreakStatements() || this.label && !options.ignoreLabel(this.label.name);
		}
	}]);

	return BreakStatement;
}(Node);

var _createClass$19 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$8 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$19(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$13(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$13(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var CallExpression = function (_Node) {
	_inherits$13(CallExpression, _Node);

	function CallExpression() {
		_classCallCheck$19(this, CallExpression);

		return _possibleConstructorReturn$13(this, (CallExpression.__proto__ || Object.getPrototypeOf(CallExpression)).apply(this, arguments));
	}

	_createClass$19(CallExpression, [{
		key: 'bind',
		value: function bind() {
			if (this.callee.type === 'Identifier') {
				var variable = this.scope.findVariable(this.callee.name);

				if (variable.isNamespace) {
					this.module.error({
						code: 'CANNOT_CALL_NAMESPACE',
						message: 'Cannot call a namespace (\'' + this.callee.name + '\')'
					}, this.start);
				}

				if (this.callee.name === 'eval' && variable.isGlobal) {
					this.module.warn({
						code: 'EVAL',
						message: 'Use of eval is strongly discouraged, as it poses security risks and may cause issues with minification',
						url: 'https://github.com/rollup/rollup/wiki/Troubleshooting#avoiding-eval'
					}, this.start);
				}
			}

			_get$8(CallExpression.prototype.__proto__ || Object.getPrototypeOf(CallExpression.prototype), 'bind', this).call(this);
			this.callee.bindCall({ withNew: false });
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.included || this.arguments.some(function (child) {
				return child.hasEffects(options);
			}) || this.callee.hasEffectsWhenCalled(options.getHasEffectsWhenCalledOptions(this.callee));
		}
	}, {
		key: 'hasEffectsAsExpressionStatement',
		value: function hasEffectsAsExpressionStatement(options) {
			return this.hasEffects(options);
		}
	}]);

	return CallExpression;
}(Node);

var _createClass$20 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$20(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$14(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$14(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var CatchClause = function (_Node) {
	_inherits$14(CatchClause, _Node);

	function CatchClause() {
		_classCallCheck$20(this, CatchClause);

		return _possibleConstructorReturn$14(this, (CatchClause.__proto__ || Object.getPrototypeOf(CatchClause)).apply(this, arguments));
	}

	_createClass$20(CatchClause, [{
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			this.param && this.param.initialiseAndDeclare(this.scope, 'parameter');
			this.body.initialiseAndReplaceScope(this.scope);
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new BlockScope({ parent: parentScope });
		}
	}]);

	return CatchClause;
}(Node);

var _createClass$21 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$21(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$15(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$15(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ClassBody = function (_Node) {
	_inherits$15(ClassBody, _Node);

	function ClassBody() {
		_classCallCheck$21(this, ClassBody);

		return _possibleConstructorReturn$15(this, (ClassBody.__proto__ || Object.getPrototypeOf(ClassBody)).apply(this, arguments));
	}

	_createClass$21(ClassBody, [{
		key: 'bindCall',
		value: function bindCall(callOptions) {
			if (this.classConstructor) {
				this.classConstructor.bindCall(callOptions);
			}
		}
	}, {
		key: 'hasEffectsWhenCalled',
		value: function hasEffectsWhenCalled(options) {
			if (this.classConstructor) {
				return this.classConstructor.hasEffectsWhenCalled(options);
			}
			return false;
		}
	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {
			this.classConstructor = this.body.find(function (method) {
				return method.kind === 'constructor';
			});
		}
	}]);

	return ClassBody;
}(Node);

var _createClass$23 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$23(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$17(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$17(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ClassNode = function (_Node) {
	_inherits$17(ClassNode, _Node);

	function ClassNode() {
		_classCallCheck$23(this, ClassNode);

		return _possibleConstructorReturn$17(this, (ClassNode.__proto__ || Object.getPrototypeOf(ClassNode)).apply(this, arguments));
	}

	_createClass$23(ClassNode, [{
		key: 'bindCall',
		value: function bindCall(callOptions) {
			if (this.superClass) {
				this.superClass.bindCall(callOptions);
			}
			this.body.bindCall(callOptions);
		}
	}, {
		key: 'hasEffectsAsExpressionStatement',
		value: function hasEffectsAsExpressionStatement(options) {
			return this.hasEffects(options);
		}
	}, {
		key: 'hasEffectsWhenCalled',
		value: function hasEffectsWhenCalled(options) {
			return this.body.hasEffectsWhenCalled(options) || this.superClass && this.superClass.hasEffectsWhenCalled(options);
		}
	}, {
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			if (this.superClass) {
				this.superClass.initialise(this.scope);
			}
			this.body.initialise(this.scope);
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new Scope({ parent: parentScope });
		}
	}]);

	return ClassNode;
}(Node);

var _createClass$22 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$9 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$22(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$16(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$16(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ClassDeclaration = function (_ClassNode) {
	_inherits$16(ClassDeclaration, _ClassNode);

	function ClassDeclaration() {
		_classCallCheck$22(this, ClassDeclaration);

		return _possibleConstructorReturn$16(this, (ClassDeclaration.__proto__ || Object.getPrototypeOf(ClassDeclaration)).apply(this, arguments));
	}

	_createClass$22(ClassDeclaration, [{
		key: 'initialiseChildren',
		value: function initialiseChildren(parentScope) {
			// Class declarations are like let declarations: Not hoisted, can be reassigned, cannot be redeclared
			this.id && this.id.initialiseAndDeclare(parentScope, 'class', this);
			_get$9(ClassDeclaration.prototype.__proto__ || Object.getPrototypeOf(ClassDeclaration.prototype), 'initialiseChildren', this).call(this, parentScope);
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			if (!this.module.bundle.treeshake || this.included) {
				_get$9(ClassDeclaration.prototype.__proto__ || Object.getPrototypeOf(ClassDeclaration.prototype), 'render', this).call(this, code, es);
			} else {
				code.remove(this.leadingCommentStart || this.start, this.next || this.end);
			}
		}
	}]);

	return ClassDeclaration;
}(ClassNode);

var _createClass$24 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$10 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$24(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$18(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$18(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ClassExpression = function (_ClassNode) {
	_inherits$18(ClassExpression, _ClassNode);

	function ClassExpression() {
		_classCallCheck$24(this, ClassExpression);

		return _possibleConstructorReturn$18(this, (ClassExpression.__proto__ || Object.getPrototypeOf(ClassExpression)).apply(this, arguments));
	}

	_createClass$24(ClassExpression, [{
		key: 'initialiseChildren',
		value: function initialiseChildren(parentScope) {
			this.id && this.id.initialiseAndDeclare(this.scope, 'class', this);
			_get$10(ClassExpression.prototype.__proto__ || Object.getPrototypeOf(ClassExpression.prototype), 'initialiseChildren', this).call(this, parentScope);
		}
	}]);

	return ClassExpression;
}(ClassNode);

var _createClass$25 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$11 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$25(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$19(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$19(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ConditionalExpression = function (_Node) {
	_inherits$19(ConditionalExpression, _Node);

	function ConditionalExpression() {
		_classCallCheck$25(this, ConditionalExpression);

		return _possibleConstructorReturn$19(this, (ConditionalExpression.__proto__ || Object.getPrototypeOf(ConditionalExpression)).apply(this, arguments));
	}

	_createClass$25(ConditionalExpression, [{
		key: 'initialiseChildren',
		value: function initialiseChildren(parentScope) {
			if (this.module.bundle.treeshake) {
				this.testValue = this.test.getValue();

				if (this.testValue === UNKNOWN_VALUE) {
					_get$11(ConditionalExpression.prototype.__proto__ || Object.getPrototypeOf(ConditionalExpression.prototype), 'initialiseChildren', this).call(this, parentScope);
				} else if (this.testValue) {
					this.consequent.initialise(this.scope);
					this.alternate = null;
				} else if (this.alternate) {
					this.alternate.initialise(this.scope);
					this.consequent = null;
				}
			} else {
				_get$11(ConditionalExpression.prototype.__proto__ || Object.getPrototypeOf(ConditionalExpression.prototype), 'initialiseChildren', this).call(this, parentScope);
			}
		}
	}, {
		key: 'getValue',
		value: function getValue() {
			var testValue = this.test.getValue();
			if (testValue === UNKNOWN_VALUE) return UNKNOWN_VALUE;

			return testValue ? this.consequent.getValue() : this.alternate.getValue();
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			if (!this.module.bundle.treeshake) {
				_get$11(ConditionalExpression.prototype.__proto__ || Object.getPrototypeOf(ConditionalExpression.prototype), 'render', this).call(this, code, es);
			} else {
				if (this.testValue === UNKNOWN_VALUE) {
					_get$11(ConditionalExpression.prototype.__proto__ || Object.getPrototypeOf(ConditionalExpression.prototype), 'render', this).call(this, code, es);
				} else if (this.testValue) {
					code.remove(this.start, this.consequent.start);
					code.remove(this.consequent.end, this.end);
					if (this.consequent.type === 'SequenceExpression') {
						code.prependRight(this.consequent.start, '(');
						code.appendLeft(this.consequent.end, ')');
					}
					this.consequent.render(code, es);
				} else {
					code.remove(this.start, this.alternate.start);
					code.remove(this.alternate.end, this.end);
					if (this.alternate.type === 'SequenceExpression') {
						code.prependRight(this.alternate.start, '(');
						code.appendLeft(this.alternate.end, ')');
					}
					this.alternate.render(code, es);
				}
			}
		}
	}]);

	return ConditionalExpression;
}(Node);

var _createClass$26 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$26(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$20(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$20(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var DoWhileStatement = function (_Statement) {
	_inherits$20(DoWhileStatement, _Statement);

	function DoWhileStatement() {
		_classCallCheck$26(this, DoWhileStatement);

		return _possibleConstructorReturn$20(this, (DoWhileStatement.__proto__ || Object.getPrototypeOf(DoWhileStatement)).apply(this, arguments));
	}

	_createClass$26(DoWhileStatement, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.included || this.test.hasEffects(options) || this.body.hasEffects(options.setIgnoreBreakStatements());
		}
	}]);

	return DoWhileStatement;
}(Statement);

var _createClass$27 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$27(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$21(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$21(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var EmptyStatement = function (_Statement) {
	_inherits$21(EmptyStatement, _Statement);

	function EmptyStatement() {
		_classCallCheck$27(this, EmptyStatement);

		return _possibleConstructorReturn$21(this, (EmptyStatement.__proto__ || Object.getPrototypeOf(EmptyStatement)).apply(this, arguments));
	}

	_createClass$27(EmptyStatement, [{
		key: 'render',
		value: function render(code) {
			if (this.parent.type === 'BlockStatement' || this.parent.type === 'Program') {
				code.remove(this.start, this.end);
			}
		}
	}]);

	return EmptyStatement;
}(Statement);

var _createClass$28 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$28(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$22(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$22(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ExportAllDeclaration = function (_Node) {
	_inherits$22(ExportAllDeclaration, _Node);

	function ExportAllDeclaration() {
		_classCallCheck$28(this, ExportAllDeclaration);

		return _possibleConstructorReturn$22(this, (ExportAllDeclaration.__proto__ || Object.getPrototypeOf(ExportAllDeclaration)).apply(this, arguments));
	}

	_createClass$28(ExportAllDeclaration, [{
		key: 'initialiseNode',
		value: function initialiseNode() {
			this.isExportDeclaration = true;
		}
	}, {
		key: 'render',
		value: function render(code) {
			code.remove(this.leadingCommentStart || this.start, this.next || this.end);
		}
	}]);

	return ExportAllDeclaration;
}(Node);

var _createClass$29 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$12 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$29(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$23(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$23(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var functionOrClassDeclaration = /^(?:Function|Class)Declaration/;

var ExportDefaultDeclaration = function (_Node) {
	_inherits$23(ExportDefaultDeclaration, _Node);

	function ExportDefaultDeclaration() {
		_classCallCheck$29(this, ExportDefaultDeclaration);

		return _possibleConstructorReturn$23(this, (ExportDefaultDeclaration.__proto__ || Object.getPrototypeOf(ExportDefaultDeclaration)).apply(this, arguments));
	}

	_createClass$29(ExportDefaultDeclaration, [{
		key: 'addCall',
		value: function addCall(options) {
			this.declaration.bindCall(options);
		}
	}, {
		key: 'addReference',
		value: function addReference(reference) {
			this.name = reference.name;
			if (this.original) this.original.addReference(reference);
		}
	}, {
		key: 'bind',
		value: function bind() {
			var name = this.declaration.id && this.declaration.id.name || this.declaration.name;
			if (name) this.original = this.scope.findVariable(name);

			this.declaration.bind();
		}
	}, {
		key: 'getName',
		value: function getName(es) {
			if (this.original && !this.original.isReassigned) {
				return this.original.getName(es);
			}

			return this.name;
		}
	}, {
		key: 'hasEffectsWhenCalled',
		value: function hasEffectsWhenCalled(options) {
			return this.declaration.hasEffectsWhenCalled(options);
		}
	}, {
		key: 'includeVariable',
		value: function includeVariable() {
			if (this.included) {
				return false;
			}
			this.included = true;
			this.declaration.includeInBundle();
			return true;
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			if (this.declaration.shouldBeIncluded()) {
				return this.declaration.includeInBundle();
			}
			return false;
		}
	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {
			this.isExportDeclaration = true;
			this.isDefault = true;

			this.name = this.declaration.id && this.declaration.id.name || this.declaration.name || this.module.basename();
			this.scope.variables.default = this;
		}

		// TODO this is total chaos, tidy it up

	}, {
		key: 'render',
		value: function render(code, es) {
			var treeshake = this.module.bundle.treeshake;
			var name = this.getName(es);

			// paren workaround: find first non-whitespace character position after `export default`
			var declaration_start = void 0;
			if (this.declaration) {
				var statementStr = code.original.slice(this.start, this.end);
				declaration_start = this.start + statementStr.match(/^\s*export\s+default\s*/)[0].length;
			}

			if (this.included || this.declaration.included) {
				if (this.included) {
					if (functionOrClassDeclaration.test(this.declaration.type)) {
						if (this.declaration.id) {
							code.remove(this.start, declaration_start);
						} else {
							code.overwrite(this.start, declaration_start, 'var ' + this.name + ' = ');
							if (code.original[this.end - 1] !== ';') code.appendLeft(this.end, ';');
						}
					} else {
						if (this.original && this.original.getName(es) === name) {
							// prevent `var foo = foo`
							code.remove(this.leadingCommentStart || this.start, this.next || this.end);
							return; // don't render children. TODO this seems like a bit of a hack
						} else {
							code.overwrite(this.start, declaration_start, this.module.bundle.varOrConst + ' ' + name + ' = ');
						}

						this.insertSemicolon(code);
					}
				} else {
					// remove `var foo` from `var foo = bar()`, if `foo` is unused
					code.remove(this.start, declaration_start);
				}

				_get$12(ExportDefaultDeclaration.prototype.__proto__ || Object.getPrototypeOf(ExportDefaultDeclaration.prototype), 'render', this).call(this, code, es);
			} else {
				if (treeshake) {
					if (functionOrClassDeclaration.test(this.declaration.type)) {
						code.remove(this.leadingCommentStart || this.start, this.next || this.end);
					} else {
						var hasEffects = this.declaration.hasEffects(ExecutionPathOptions.create());
						code.remove(this.start, hasEffects ? declaration_start : this.next || this.end);
					}
				} else if (name === this.declaration.name) {
					code.remove(this.start, this.next || this.end);
				} else {
					code.overwrite(this.start, declaration_start, this.module.bundle.varOrConst + ' ' + name + ' = ');
				}
				// code.remove( this.start, this.next || this.end );
			}
		}
	}]);

	return ExportDefaultDeclaration;
}(Node);

var _createClass$30 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$30(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$24(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$24(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ExportNamedDeclaration = function (_Node) {
	_inherits$24(ExportNamedDeclaration, _Node);

	function ExportNamedDeclaration() {
		_classCallCheck$30(this, ExportNamedDeclaration);

		return _possibleConstructorReturn$24(this, (ExportNamedDeclaration.__proto__ || Object.getPrototypeOf(ExportNamedDeclaration)).apply(this, arguments));
	}

	_createClass$30(ExportNamedDeclaration, [{
		key: 'bind',
		value: function bind() {
			// Do not bind specifiers
			if (this.declaration) this.declaration.bind();
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.included || this.declaration && this.declaration.hasEffects(options);
		}
	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {
			this.isExportDeclaration = true;
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			if (this.declaration) {
				code.remove(this.start, this.declaration.start);
				this.declaration.render(code, es);
			} else {
				var start = this.leadingCommentStart || this.start;
				var end = this.next || this.end;

				if (this.defaultExport) {
					var name = this.defaultExport.getName(es);
					var originalName = this.defaultExport.original.getName(es);

					if (name !== originalName) {
						code.overwrite(start, end, 'var ' + name + ' = ' + originalName + ';');
						return;
					}
				}

				code.remove(start, end);
			}
		}
	}]);

	return ExportNamedDeclaration;
}(Node);

var _createClass$31 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$13 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$31(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$25(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$25(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ExpressionStatement = function (_Statement) {
	_inherits$25(ExpressionStatement, _Statement);

	function ExpressionStatement() {
		_classCallCheck$31(this, ExpressionStatement);

		return _possibleConstructorReturn$25(this, (ExpressionStatement.__proto__ || Object.getPrototypeOf(ExpressionStatement)).apply(this, arguments));
	}

	_createClass$31(ExpressionStatement, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return _get$13(ExpressionStatement.prototype.__proto__ || Object.getPrototypeOf(ExpressionStatement.prototype), 'hasEffects', this).call(this, options) || this.expression.hasEffectsAsExpressionStatement(options);
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			_get$13(ExpressionStatement.prototype.__proto__ || Object.getPrototypeOf(ExpressionStatement.prototype), 'render', this).call(this, code, es);
			if (this.included) this.insertSemicolon(code);
		}
	}]);

	return ExpressionStatement;
}(Statement);

var _createClass$32 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$32(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$26(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$26(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ForStatement = function (_Statement) {
	_inherits$26(ForStatement, _Statement);

	function ForStatement() {
		_classCallCheck$32(this, ForStatement);

		return _possibleConstructorReturn$26(this, (ForStatement.__proto__ || Object.getPrototypeOf(ForStatement)).apply(this, arguments));
	}

	_createClass$32(ForStatement, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.included || this.init && this.init.hasEffects(options) || this.test && this.test.hasEffects(options) || this.update && this.update.hasEffects(options) || this.body.hasEffects(options.setIgnoreBreakStatements());
		}
	}, {
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			if (this.init) this.init.initialise(this.scope);
			if (this.test) this.test.initialise(this.scope);
			if (this.update) this.update.initialise(this.scope);

			if (this.body.type === 'BlockStatement') {
				this.body.initialiseScope(this.scope);
				this.body.initialiseChildren();
			} else {
				this.body.initialise(this.scope);
			}
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new BlockScope({ parent: parentScope });
		}
	}]);

	return ForStatement;
}(Statement);

var _createClass$33 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$14 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$33(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$27(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$27(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ForInStatement = function (_Statement) {
	_inherits$27(ForInStatement, _Statement);

	function ForInStatement() {
		_classCallCheck$33(this, ForInStatement);

		return _possibleConstructorReturn$27(this, (ForInStatement.__proto__ || Object.getPrototypeOf(ForInStatement)).apply(this, arguments));
	}

	_createClass$33(ForInStatement, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.included || this.left && (this.left.hasEffects(options) || this.left.hasEffectsWhenAssigned(options)) || this.right && this.right.hasEffects(options) || this.body.hasEffects(options.setIgnoreBreakStatements());
		}
	}, {
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			this.left.initialise(this.scope);
			this.right.initialise(this.scope.parent);
			this.body.initialiseAndReplaceScope ? this.body.initialiseAndReplaceScope(this.scope) : this.body.initialise(this.scope);
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			var addedNewNodes = _get$14(ForInStatement.prototype.__proto__ || Object.getPrototypeOf(ForInStatement.prototype), 'includeInBundle', this).call(this);
			if (this.left.includeWithAllDeclarations()) {
				addedNewNodes = true;
			}
			return addedNewNodes;
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new BlockScope({ parent: parentScope });
		}
	}]);

	return ForInStatement;
}(Statement);

var _createClass$34 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$15 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$34(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$28(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$28(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ForOfStatement = function (_Statement) {
	_inherits$28(ForOfStatement, _Statement);

	function ForOfStatement() {
		_classCallCheck$34(this, ForOfStatement);

		return _possibleConstructorReturn$28(this, (ForOfStatement.__proto__ || Object.getPrototypeOf(ForOfStatement)).apply(this, arguments));
	}

	_createClass$34(ForOfStatement, [{
		key: 'bind',
		value: function bind() {
			_get$15(ForOfStatement.prototype.__proto__ || Object.getPrototypeOf(ForOfStatement.prototype), 'bind', this).call(this);
			this.left.bindAssignment(UNKNOWN_ASSIGNMENT);
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.included || this.left && (this.left.hasEffects(options) || this.left.hasEffectsWhenAssigned(options)) || this.right && this.right.hasEffects(options) || this.body.hasEffects(options.setIgnoreBreakStatements());
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			var addedNewNodes = _get$15(ForOfStatement.prototype.__proto__ || Object.getPrototypeOf(ForOfStatement.prototype), 'includeInBundle', this).call(this);
			if (this.left.includeWithAllDeclarations()) {
				addedNewNodes = true;
			}
			return addedNewNodes;
		}
	}, {
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			this.left.initialise(this.scope);
			this.right.initialise(this.scope.parent);
			this.body.initialiseAndReplaceScope ? this.body.initialiseAndReplaceScope(this.scope) : this.body.initialise(this.scope);
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new BlockScope({ parent: parentScope });
		}
	}]);

	return ForOfStatement;
}(Statement);

var _createClass$37 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$37(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$31(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$31(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var FunctionScope = function (_Scope) {
	_inherits$31(FunctionScope, _Scope);

	function FunctionScope() {
		var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

		_classCallCheck$37(this, FunctionScope);

		var _this = _possibleConstructorReturn$31(this, (FunctionScope.__proto__ || Object.getPrototypeOf(FunctionScope)).call(this, options));

		_this.variables.arguments = new ParameterVariable('arguments');
		_this.variables.this = new LocalVariable('this', null, null);
		return _this;
	}

	_createClass$37(FunctionScope, [{
		key: 'findLexicalBoundary',
		value: function findLexicalBoundary() {
			return this;
		}
	}]);

	return FunctionScope;
}(Scope);

var _createClass$36 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$36(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$30(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$30(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var FunctionNode = function (_Node) {
	_inherits$30(FunctionNode, _Node);

	function FunctionNode() {
		_classCallCheck$36(this, FunctionNode);

		return _possibleConstructorReturn$30(this, (FunctionNode.__proto__ || Object.getPrototypeOf(FunctionNode)).apply(this, arguments));
	}

	_createClass$36(FunctionNode, [{
		key: 'bindCall',
		value: function bindCall(_ref) {
			var withNew = _ref.withNew;

			var thisVariable = this.scope.findVariable('this');

			if (withNew) {
				thisVariable.assignExpression(UNKNOWN_OBJECT_LITERAL);
			} else {
				thisVariable.assignExpression(UNKNOWN_ASSIGNMENT);
			}
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.included || this.id && this.id.hasEffects(options);
		}
	}, {
		key: 'hasEffectsAsExpressionStatement',
		value: function hasEffectsAsExpressionStatement(options) {
			return this.hasEffects(options);
		}
	}, {
		key: 'hasEffectsWhenCalled',
		value: function hasEffectsWhenCalled(options) {
			var innerOptions = options.setIgnoreSafeThisMutations();
			return this.params.some(function (param) {
				return param.hasEffects(innerOptions);
			}) || this.body.hasEffects(innerOptions);
		}
	}, {
		key: 'hasEffectsWhenMutated',
		value: function hasEffectsWhenMutated() {
			return this.included;
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new FunctionScope({ parent: parentScope });
		}
	}]);

	return FunctionNode;
}(Node);

var _createClass$35 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$16 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$35(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$29(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$29(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var FunctionDeclaration = function (_FunctionNode) {
	_inherits$29(FunctionDeclaration, _FunctionNode);

	function FunctionDeclaration() {
		_classCallCheck$35(this, FunctionDeclaration);

		return _possibleConstructorReturn$29(this, (FunctionDeclaration.__proto__ || Object.getPrototypeOf(FunctionDeclaration)).apply(this, arguments));
	}

	_createClass$35(FunctionDeclaration, [{
		key: 'initialiseChildren',
		value: function initialiseChildren(parentScope) {
			var _this2 = this;

			this.id && this.id.initialiseAndDeclare(parentScope, 'function', this);
			this.params.forEach(function (param) {
				return param.initialiseAndDeclare(_this2.scope, 'parameter');
			});
			this.body.initialiseAndReplaceScope(new Scope({ parent: this.scope }));
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			if (!this.module.bundle.treeshake || this.included) {
				_get$16(FunctionDeclaration.prototype.__proto__ || Object.getPrototypeOf(FunctionDeclaration.prototype), 'render', this).call(this, code, es);
			} else {
				code.remove(this.leadingCommentStart || this.start, this.next || this.end);
			}
		}
	}]);

	return FunctionDeclaration;
}(FunctionNode);

var _createClass$38 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$38(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$32(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$32(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var FunctionExpression = function (_FunctionNode) {
	_inherits$32(FunctionExpression, _FunctionNode);

	function FunctionExpression() {
		_classCallCheck$38(this, FunctionExpression);

		return _possibleConstructorReturn$32(this, (FunctionExpression.__proto__ || Object.getPrototypeOf(FunctionExpression)).apply(this, arguments));
	}

	_createClass$38(FunctionExpression, [{
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			var _this2 = this;

			this.id && this.id.initialiseAndDeclare(this.scope, 'function', this);
			this.params.forEach(function (param) {
				return param.initialiseAndDeclare(_this2.scope, 'parameter');
			});
			this.body.initialiseAndReplaceScope(new Scope({ parent: this.scope }));
		}
	}]);

	return FunctionExpression;
}(FunctionNode);

var _createClass$39 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$39(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$33(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$33(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function isAssignmentPatternLhs(node, parent) {
	// special case: `({ foo = 42 }) => {...}`
	// `foo` actually has two different parents, the Property of the
	// ObjectPattern, and the AssignmentPattern. In one case it's a
	// reference, in one case it's not, because it's shorthand for
	// `({ foo: foo = 42 }) => {...}`. But unlike a regular shorthand
	// property, the `foo` node appears at different levels of the tree
	return parent.type === 'Property' && parent.shorthand && parent.value.type === 'AssignmentPattern' && parent.value.left === node;
}

var Identifier = function (_Node) {
	_inherits$33(Identifier, _Node);

	function Identifier() {
		_classCallCheck$39(this, Identifier);

		return _possibleConstructorReturn$33(this, (Identifier.__proto__ || Object.getPrototypeOf(Identifier)).apply(this, arguments));
	}

	_createClass$39(Identifier, [{
		key: 'bind',
		value: function bind() {
			if (isReference(this, this.parent) || isAssignmentPatternLhs(this, this.parent)) {
				this.variable = this.scope.findVariable(this.name);
				this.variable.addReference(this);
			}
		}
	}, {
		key: 'bindAssignment',
		value: function bindAssignment(expression) {
			if (this.variable) {
				this.variable.assignExpression(expression);
			}
		}
	}, {
		key: 'bindCall',
		value: function bindCall(callOptions) {
			if (this.variable) {
				this.variable.addCall(callOptions);
			}
		}
	}, {
		key: 'hasEffectsAsExpressionStatement',
		value: function hasEffectsAsExpressionStatement(options) {
			return this.hasEffects(options) || this.variable.isGlobal;
		}
	}, {
		key: 'hasEffectsWhenAssigned',
		value: function hasEffectsWhenAssigned() {
			return this.variable && this.variable.included;
		}
	}, {
		key: 'hasEffectsWhenCalled',
		value: function hasEffectsWhenCalled(options) {
			if (!this.variable) {
				return true;
			}
			return this.variable.hasEffectsWhenCalled(options);
		}
	}, {
		key: 'hasEffectsWhenMutated',
		value: function hasEffectsWhenMutated(options) {
			return this.variable && this.variable.hasEffectsWhenMutated(options);
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			if (this.included) return false;
			this.included = true;
			this.variable && this.variable.includeVariable();
			return true;
		}
	}, {
		key: 'initialiseAndDeclare',
		value: function initialiseAndDeclare(parentScope, kind, init) {
			this.initialiseScope(parentScope);
			switch (kind) {
				case 'var':
				case 'function':
					this.scope.addDeclaration(this, true, init);
					break;
				case 'let':
				case 'const':
				case 'class':
					this.scope.addDeclaration(this, false, init);
					break;
				case 'parameter':
					this.scope.addParameterDeclaration(this);
					break;
				default:
					throw new Error('Unexpected identifier kind', kind);
			}
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			if (this.variable) {
				var name = this.variable.getName(es);
				if (name !== this.name) {
					code.overwrite(this.start, this.end, name, { storeName: true, contentOnly: false });

					// special case
					if (this.parent.type === 'Property' && this.parent.shorthand) {
						code.appendLeft(this.start, this.name + ': ');
					}
				}
			}
		}
	}]);

	return Identifier;
}(Node);

var _createClass$40 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$17 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$40(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$34(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$34(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

// Statement types which may contain if-statements as direct children.
var statementsWithIfStatements = new Set(['DoWhileStatement', 'ForInStatement', 'ForOfStatement', 'ForStatement', 'IfStatement', 'WhileStatement']);

function getHoistedVars(node, scope) {
	var hoistedVars = [];

	function visit(node) {
		if (node.type === 'VariableDeclaration' && node.kind === 'var') {
			node.declarations.forEach(function (declarator) {
				declarator.init = null;
				declarator.initialise(scope);

				extractNames(declarator.id).forEach(function (name) {
					if (hoistedVars.indexOf(name) < 0) hoistedVars.push(name);
				});
			});
		} else if (!/Function/.test(node.type)) {
			node.eachChild(visit);
		}
	}

	visit(node);

	return hoistedVars;
}

var IfStatement = function (_Statement) {
	_inherits$34(IfStatement, _Statement);

	function IfStatement() {
		_classCallCheck$40(this, IfStatement);

		return _possibleConstructorReturn$34(this, (IfStatement.__proto__ || Object.getPrototypeOf(IfStatement)).apply(this, arguments));
	}

	_createClass$40(IfStatement, [{
		key: 'initialiseChildren',
		value: function initialiseChildren(parentScope) {
			_get$17(IfStatement.prototype.__proto__ || Object.getPrototypeOf(IfStatement.prototype), 'initialiseChildren', this).call(this, parentScope);
			if (this.module.bundle.treeshake) {
				this.testValue = this.test.getValue();

				if (this.testValue === UNKNOWN_VALUE) {
					return;
				}
				if (this.testValue) {
					if (this.alternate) {
						this.hoistedVars = getHoistedVars(this.alternate, this.scope);
						this.alternate = null;
					}
				} else {
					this.hoistedVars = getHoistedVars(this.consequent, this.scope);
					this.consequent = null;
				}
			}
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			var _this2 = this;

			if (this.module.bundle.treeshake) {
				if (this.testValue === UNKNOWN_VALUE) {
					_get$17(IfStatement.prototype.__proto__ || Object.getPrototypeOf(IfStatement.prototype), 'render', this).call(this, code, es);
				} else {
					code.overwrite(this.test.start, this.test.end, JSON.stringify(this.testValue));

					// TODO if no block-scoped declarations, remove enclosing
					// curlies and dedent block (if there is a block)

					if (this.hoistedVars) {
						var names = this.hoistedVars.map(function (name) {
							var variable = _this2.scope.findVariable(name);
							return variable.included ? variable.getName() : null;
						}).filter(Boolean);

						if (names.length > 0) {
							code.appendLeft(this.start, 'var ' + names.join(', ') + ';\n\n');
						}
					}

					if (this.testValue) {
						code.remove(this.start, this.consequent.start);
						code.remove(this.consequent.end, this.end);
						this.consequent.render(code, es);
					} else {
						code.remove(this.start, this.alternate ? this.alternate.start : this.next || this.end);

						if (this.alternate) {
							this.alternate.render(code, es);
						} else if (statementsWithIfStatements.has(this.parent.type)) {
							code.prependRight(this.start, '{}');
						}
					}
				}
			} else {
				_get$17(IfStatement.prototype.__proto__ || Object.getPrototypeOf(IfStatement.prototype), 'render', this).call(this, code, es);
			}
		}
	}]);

	return IfStatement;
}(Statement);

var _createClass$41 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$41(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$35(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$35(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ImportDeclaration = function (_Node) {
	_inherits$35(ImportDeclaration, _Node);

	function ImportDeclaration() {
		_classCallCheck$41(this, ImportDeclaration);

		return _possibleConstructorReturn$35(this, (ImportDeclaration.__proto__ || Object.getPrototypeOf(ImportDeclaration)).apply(this, arguments));
	}

	_createClass$41(ImportDeclaration, [{
		key: 'bind',
		value: function bind() {
			// noop
			// TODO do the inter-module binding setup here?
		}
	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {
			this.isImportDeclaration = true;
		}
	}, {
		key: 'render',
		value: function render(code) {
			code.remove(this.start, this.next || this.end);
		}
	}]);

	return ImportDeclaration;
}(Node);

var _createClass$42 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$42(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$36(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$36(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var LabeledStatement = function (_Statement) {
	_inherits$36(LabeledStatement, _Statement);

	function LabeledStatement() {
		_classCallCheck$42(this, LabeledStatement);

		return _possibleConstructorReturn$36(this, (LabeledStatement.__proto__ || Object.getPrototypeOf(LabeledStatement)).apply(this, arguments));
	}

	_createClass$42(LabeledStatement, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.body.hasEffects(options.setIgnoreLabel(this.label.name).setIgnoreBreakStatements());
		}
	}]);

	return LabeledStatement;
}(Statement);

var _createClass$43 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$43(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$37(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$37(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Literal = function (_Node) {
	_inherits$37(Literal, _Node);

	function Literal() {
		_classCallCheck$43(this, Literal);

		return _possibleConstructorReturn$37(this, (Literal.__proto__ || Object.getPrototypeOf(Literal)).apply(this, arguments));
	}

	_createClass$43(Literal, [{
		key: 'getValue',
		value: function getValue() {
			return this.value;
		}
	}, {
		key: 'hasEffectsWhenMutated',
		value: function hasEffectsWhenMutated() {
			return false;
		}
	}, {
		key: 'render',
		value: function render(code) {
			if (typeof this.value === 'string') {
				code.indentExclusionRanges.push([this.start + 1, this.end - 1]);
			}
		}
	}]);

	return Literal;
}(Node);

var _createClass$44 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$44(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$38(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$38(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var operators$1 = {
	'&&': function _(left, right) {
		return left && right;
	},
	'||': function _(left, right) {
		return left || right;
	}
};

var LogicalExpression = function (_Node) {
	_inherits$38(LogicalExpression, _Node);

	function LogicalExpression() {
		_classCallCheck$44(this, LogicalExpression);

		return _possibleConstructorReturn$38(this, (LogicalExpression.__proto__ || Object.getPrototypeOf(LogicalExpression)).apply(this, arguments));
	}

	_createClass$44(LogicalExpression, [{
		key: 'getValue',
		value: function getValue() {
			var leftValue = this.left.getValue();
			if (leftValue === UNKNOWN_VALUE) return UNKNOWN_VALUE;

			var rightValue = this.right.getValue();
			if (rightValue === UNKNOWN_VALUE) return UNKNOWN_VALUE;

			return operators$1[this.operator](leftValue, rightValue);
		}
	}, {
		key: 'hasEffectsWhenMutated',
		value: function hasEffectsWhenMutated(options) {
			var leftValue = this.left.getValue();
			if (leftValue === UNKNOWN_VALUE) {
				return this.left.hasEffectsWhenMutated(options) || this.right.hasEffectsWhenMutated(options);
			}
			if (leftValue && this.operator === '||' || !leftValue && this.operator === '&&') {
				return this.left.hasEffectsWhenMutated(options);
			}
			return this.right.hasEffectsWhenMutated(options);
		}
	}]);

	return LogicalExpression;
}(Node);

function flatten(node) {
	var parts = [];
	while (node.type === 'MemberExpression') {
		if (node.computed) return null;
		parts.unshift(node.property.name);

		node = node.object;
	}

	if (node.type !== 'Identifier') return null;

	var name = node.name;
	parts.unshift(name);

	return { name: name, keypath: parts.join('.') };
}

var pureFunctions = {};

var arrayTypes = 'Array Int8Array Uint8Array Uint8ClampedArray Int16Array Uint16Array Int32Array Uint32Array Float32Array Float64Array'.split(' ');
var simdTypes = 'Int8x16 Int16x8 Int32x4 Float32x4 Float64x2'.split(' ');
var simdMethods = 'abs add and bool check div equal extractLane fromFloat32x4 fromFloat32x4Bits fromFloat64x2 fromFloat64x2Bits fromInt16x8Bits fromInt32x4 fromInt32x4Bits fromInt8x16Bits greaterThan greaterThanOrEqual lessThan lessThanOrEqual load max maxNum min minNum mul neg not notEqual or reciprocalApproximation reciprocalSqrtApproximation replaceLane select selectBits shiftLeftByScalar shiftRightArithmeticByScalar shiftRightLogicalByScalar shuffle splat sqrt store sub swizzle xor'.split(' ');
var allSimdMethods = [];
simdTypes.forEach(function (t) {
	simdMethods.forEach(function (m) {
		allSimdMethods.push('SIMD.' + t + '.' + m);
	});
});

['Array.isArray', 'Error', 'EvalError', 'InternalError', 'RangeError', 'ReferenceError', 'SyntaxError', 'TypeError', 'URIError', 'isFinite', 'isNaN', 'parseFloat', 'parseInt', 'decodeURI', 'decodeURIComponent', 'encodeURI', 'encodeURIComponent', 'escape', 'unescape', 'Object', 'Object.create', 'Object.getNotifier', 'Object.getOwn', 'Object.getOwnPropertyDescriptor', 'Object.getOwnPropertyNames', 'Object.getOwnPropertySymbols', 'Object.getPrototypeOf', 'Object.is', 'Object.isExtensible', 'Object.isFrozen', 'Object.isSealed', 'Object.keys', 'Function', 'Boolean', 'Number', 'Number.isFinite', 'Number.isInteger', 'Number.isNaN', 'Number.isSafeInteger', 'Number.parseFloat', 'Number.parseInt', 'Symbol', 'Symbol.for', 'Symbol.keyFor', 'Math.abs', 'Math.acos', 'Math.acosh', 'Math.asin', 'Math.asinh', 'Math.atan', 'Math.atan2', 'Math.atanh', 'Math.cbrt', 'Math.ceil', 'Math.clz32', 'Math.cos', 'Math.cosh', 'Math.exp', 'Math.expm1', 'Math.floor', 'Math.fround', 'Math.hypot', 'Math.imul', 'Math.log', 'Math.log10', 'Math.log1p', 'Math.log2', 'Math.max', 'Math.min', 'Math.pow', 'Math.random', 'Math.round', 'Math.sign', 'Math.sin', 'Math.sinh', 'Math.sqrt', 'Math.tan', 'Math.tanh', 'Math.trunc', 'Date', 'Date.UTC', 'Date.now', 'Date.parse', 'String', 'String.fromCharCode', 'String.fromCodePoint', 'String.raw', 'RegExp', 'Map', 'Set', 'WeakMap', 'WeakSet', 'ArrayBuffer', 'ArrayBuffer.isView', 'DataView', 'JSON.parse', 'JSON.stringify', 'Promise.all', 'Promise.race', 'Promise.resolve', 'Intl.Collator', 'Intl.Collator.supportedLocalesOf', 'Intl.DateTimeFormat', 'Intl.DateTimeFormat.supportedLocalesOf', 'Intl.NumberFormat', 'Intl.NumberFormat.supportedLocalesOf'

// TODO properties of e.g. window...
].concat(arrayTypes, arrayTypes.map(function (t) {
	return t + '.from';
}), arrayTypes.map(function (t) {
	return t + '.of';
}), simdTypes.map(function (t) {
	return 'SIMD.' + t;
}), allSimdMethods).forEach(function (name) {
	return pureFunctions[name] = true;
});

var _createClass$45 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$18 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _possibleConstructorReturn$39(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$39(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck$45(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var validProp = /^[a-zA-Z_$][a-zA-Z_$0-9]*$/;

var Keypath = function Keypath(node) {
	_classCallCheck$45(this, Keypath);

	this.parts = [];

	while (node.type === 'MemberExpression') {
		var prop = node.property;

		if (node.computed) {
			if (prop.type !== 'Literal' || typeof prop.value !== 'string' || !validProp.test(prop.value)) {
				this.computed = true;
				return;
			}
		}

		this.parts.unshift(prop);
		node = node.object;
	}

	this.root = node;
};

var MemberExpression = function (_Node) {
	_inherits$39(MemberExpression, _Node);

	function MemberExpression() {
		_classCallCheck$45(this, MemberExpression);

		return _possibleConstructorReturn$39(this, (MemberExpression.__proto__ || Object.getPrototypeOf(MemberExpression)).apply(this, arguments));
	}

	_createClass$45(MemberExpression, [{
		key: 'bind',
		value: function bind() {
			// if this resolves to a namespaced declaration, prepare
			// to replace it
			// TODO this code is a bit inefficient
			var keypath = new Keypath(this);

			if (!keypath.computed && keypath.root.type === 'Identifier') {
				var variable = this.scope.findVariable(keypath.root.name);

				while (variable.isNamespace && keypath.parts.length) {
					var exporterId = variable.module.id;

					var part = keypath.parts[0];
					variable = variable.module.traceExport(part.name || part.value);

					if (!variable) {
						this.module.warn({
							code: 'MISSING_EXPORT',
							missing: part.name || part.value,
							importer: relativeId(this.module.id),
							exporter: relativeId(exporterId),
							message: '\'' + (part.name || part.value) + '\' is not exported by \'' + relativeId(exporterId) + '\'',
							url: 'https://github.com/rollup/rollup/wiki/Troubleshooting#name-is-not-exported-by-module'
						}, part.start);
						this.replacement = 'undefined';
						return;
					}

					keypath.parts.shift();
				}

				if (keypath.parts.length) {
					_get$18(MemberExpression.prototype.__proto__ || Object.getPrototypeOf(MemberExpression.prototype), 'bind', this).call(this);
					return; // not a namespaced declaration
				}

				this.variable = variable;

				if (variable.isExternal) {
					variable.module.suggestName(keypath.root.name);
				}
			} else {
				_get$18(MemberExpression.prototype.__proto__ || Object.getPrototypeOf(MemberExpression.prototype), 'bind', this).call(this);
			}
		}
	}, {
		key: 'bindCall',
		value: function bindCall(callOptions) {
			if (this.variable) {
				this.variable.addCall(callOptions);
			}
		}
	}, {
		key: 'hasEffectsWhenAssigned',
		value: function hasEffectsWhenAssigned(options) {
			return this.object.hasEffectsWhenMutated(options);
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			var addedNewNodes = _get$18(MemberExpression.prototype.__proto__ || Object.getPrototypeOf(MemberExpression.prototype), 'includeInBundle', this).call(this);
			if (this.variable && !this.variable.included) {
				this.variable.includeVariable();
				addedNewNodes = true;
			}
			return addedNewNodes;
		}
	}, {
		key: 'hasEffectsWhenCalled',
		value: function hasEffectsWhenCalled(options) {
			if (this.variable) {
				return this.variable.hasEffectsWhenCalled(options);
			}
			if (!isReference(this)) {
				return true;
			}
			var flattenedNode = flatten(this);
			return !(this.scope.findVariable(flattenedNode.name).isGlobal && pureFunctions[flattenedNode.keypath]);
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			if (this.variable) {
				var name = this.variable.getName(es);
				if (name !== this.name) code.overwrite(this.start, this.end, name, { storeName: true, contentOnly: false });
			} else if (this.replacement) {
				code.overwrite(this.start, this.end, this.replacement, { storeName: true, contentOnly: false });
			}

			_get$18(MemberExpression.prototype.__proto__ || Object.getPrototypeOf(MemberExpression.prototype), 'render', this).call(this, code, es);
		}
	}]);

	return MemberExpression;
}(Node);

var _createClass$46 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$46(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$40(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$40(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var MethodDefinition = function (_Node) {
	_inherits$40(MethodDefinition, _Node);

	function MethodDefinition() {
		_classCallCheck$46(this, MethodDefinition);

		return _possibleConstructorReturn$40(this, (MethodDefinition.__proto__ || Object.getPrototypeOf(MethodDefinition)).apply(this, arguments));
	}

	_createClass$46(MethodDefinition, [{
		key: 'bindCall',
		value: function bindCall(callOptions) {
			this.value.bindCall(callOptions);
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.key.hasEffects(options);
		}
	}, {
		key: 'hasEffectsWhenCalled',
		value: function hasEffectsWhenCalled(options) {
			return this.value.hasEffectsWhenCalled(options);
		}
	}]);

	return MethodDefinition;
}(Node);

var _createClass$47 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$19 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$47(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$41(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$41(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var NewExpression = function (_Node) {
	_inherits$41(NewExpression, _Node);

	function NewExpression() {
		_classCallCheck$47(this, NewExpression);

		return _possibleConstructorReturn$41(this, (NewExpression.__proto__ || Object.getPrototypeOf(NewExpression)).apply(this, arguments));
	}

	_createClass$47(NewExpression, [{
		key: 'bind',
		value: function bind() {
			_get$19(NewExpression.prototype.__proto__ || Object.getPrototypeOf(NewExpression.prototype), 'bind', this).call(this);
			this.callee.bindCall({ withNew: true });
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.included || this.arguments.some(function (child) {
				return child.hasEffects(options);
			}) || this.callee.hasEffectsWhenCalled(options.getHasEffectsWhenCalledOptions(this.callee));
		}
	}]);

	return NewExpression;
}(Node);

var _createClass$48 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$48(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$42(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$42(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ObjectExpression = function (_Node) {
	_inherits$42(ObjectExpression, _Node);

	function ObjectExpression() {
		_classCallCheck$48(this, ObjectExpression);

		return _possibleConstructorReturn$42(this, (ObjectExpression.__proto__ || Object.getPrototypeOf(ObjectExpression)).apply(this, arguments));
	}

	_createClass$48(ObjectExpression, [{
		key: 'hasEffectsWhenMutated',
		value: function hasEffectsWhenMutated() {
			return false;
		}
	}]);

	return ObjectExpression;
}(Node);

var _createClass$49 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$49(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$43(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$43(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ObjectPattern = function (_Node) {
	_inherits$43(ObjectPattern, _Node);

	function ObjectPattern() {
		_classCallCheck$49(this, ObjectPattern);

		return _possibleConstructorReturn$43(this, (ObjectPattern.__proto__ || Object.getPrototypeOf(ObjectPattern)).apply(this, arguments));
	}

	_createClass$49(ObjectPattern, [{
		key: 'bindAssignment',
		value: function bindAssignment(expression) {
			this.properties.forEach(function (child) {
				return child.bindAssignment(expression);
			});
		}
	}, {
		key: 'hasEffectsWhenAssigned',
		value: function hasEffectsWhenAssigned(options) {
			return this.someChild(function (child) {
				return child.hasEffectsWhenAssigned(options);
			});
		}
	}, {
		key: 'initialiseAndDeclare',
		value: function initialiseAndDeclare(parentScope, kind, init) {
			this.initialiseScope(parentScope);
			this.properties.forEach(function (child) {
				return child.initialiseAndDeclare(parentScope, kind, init);
			});
		}
	}]);

	return ObjectPattern;
}(Node);

var _createClass$50 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$50(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$44(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$44(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Property = function (_Node) {
	_inherits$44(Property, _Node);

	function Property() {
		_classCallCheck$50(this, Property);

		return _possibleConstructorReturn$44(this, (Property.__proto__ || Object.getPrototypeOf(Property)).apply(this, arguments));
	}

	_createClass$50(Property, [{
		key: 'bindAssignment',
		value: function bindAssignment() {
			this.value.bindAssignment(UNKNOWN_ASSIGNMENT);
		}
	}, {
		key: 'hasEffectsWhenAssigned',
		value: function hasEffectsWhenAssigned(options) {
			return this.value.hasEffectsWhenAssigned(options);
		}
	}, {
		key: 'initialiseAndDeclare',
		value: function initialiseAndDeclare(parentScope, kind, init) {
			this.initialiseScope(parentScope);
			this.key.initialise(parentScope);
			this.value.initialiseAndDeclare(parentScope, kind, init && UNKNOWN_ASSIGNMENT);
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			if (!this.shorthand) {
				this.key.render(code, es);
			}
			this.value.render(code, es);
		}
	}]);

	return Property;
}(Node);

var _createClass$51 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$51(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$45(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$45(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var RestElement = function (_Node) {
	_inherits$45(RestElement, _Node);

	function RestElement() {
		_classCallCheck$51(this, RestElement);

		return _possibleConstructorReturn$45(this, (RestElement.__proto__ || Object.getPrototypeOf(RestElement)).apply(this, arguments));
	}

	_createClass$51(RestElement, [{
		key: 'bindAssignment',
		value: function bindAssignment() {
			this.argument.bindAssignment(UNKNOWN_ASSIGNMENT);
		}
	}, {
		key: 'hasEffectsWhenAssigned',
		value: function hasEffectsWhenAssigned(options) {
			return this.argument.hasEffectsWhenAssigned(options);
		}
	}, {
		key: 'initialiseAndDeclare',
		value: function initialiseAndDeclare(parentScope, kind) {
			this.initialiseScope(parentScope);
			this.argument.initialiseAndDeclare(parentScope, kind, UNKNOWN_ASSIGNMENT);
		}
	}]);

	return RestElement;
}(Node);

var _createClass$52 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$20 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$52(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$46(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$46(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ReturnStatement = function (_Statement) {
	_inherits$46(ReturnStatement, _Statement);

	function ReturnStatement() {
		_classCallCheck$52(this, ReturnStatement);

		return _possibleConstructorReturn$46(this, (ReturnStatement.__proto__ || Object.getPrototypeOf(ReturnStatement)).apply(this, arguments));
	}

	_createClass$52(ReturnStatement, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return _get$20(ReturnStatement.prototype.__proto__ || Object.getPrototypeOf(ReturnStatement.prototype), 'hasEffects', this).call(this, options) || !options.ignoreReturnAwaitYield();
		}
	}]);

	return ReturnStatement;
}(Statement);

var _createClass$53 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$53(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$47(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$47(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var SwitchCase = function (_Node) {
	_inherits$47(SwitchCase, _Node);

	function SwitchCase() {
		_classCallCheck$53(this, SwitchCase);

		return _possibleConstructorReturn$47(this, (SwitchCase.__proto__ || Object.getPrototypeOf(SwitchCase)).apply(this, arguments));
	}

	_createClass$53(SwitchCase, [{
		key: 'includeInBundle',
		value: function includeInBundle() {
			if (this.isFullyIncluded()) return false;
			var addedNewNodes = false;
			if (this.test && this.test.includeInBundle()) {
				addedNewNodes = true;
			}
			this.consequent.forEach(function (node) {
				if (node.shouldBeIncluded()) {
					if (node.includeInBundle()) {
						addedNewNodes = true;
					}
				}
			});
			if (!this.included || addedNewNodes) {
				this.included = true;
				return true;
			}
			return false;
		}
	}]);

	return SwitchCase;
}(Node);

var _createClass$54 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$21 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$54(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$48(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$48(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var SwitchStatement = function (_Statement) {
	_inherits$48(SwitchStatement, _Statement);

	function SwitchStatement() {
		_classCallCheck$54(this, SwitchStatement);

		return _possibleConstructorReturn$48(this, (SwitchStatement.__proto__ || Object.getPrototypeOf(SwitchStatement)).apply(this, arguments));
	}

	_createClass$54(SwitchStatement, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return _get$21(SwitchStatement.prototype.__proto__ || Object.getPrototypeOf(SwitchStatement.prototype), 'hasEffects', this).call(this, options.setIgnoreBreakStatements());
		}
	}, {
		key: 'initialiseScope',
		value: function initialiseScope(parentScope) {
			this.scope = new BlockScope({ parent: parentScope });
		}
	}]);

	return SwitchStatement;
}(Statement);

var _createClass$55 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$22 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$55(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$49(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$49(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var TaggedTemplateExpression = function (_Node) {
	_inherits$49(TaggedTemplateExpression, _Node);

	function TaggedTemplateExpression() {
		_classCallCheck$55(this, TaggedTemplateExpression);

		return _possibleConstructorReturn$49(this, (TaggedTemplateExpression.__proto__ || Object.getPrototypeOf(TaggedTemplateExpression)).apply(this, arguments));
	}

	_createClass$55(TaggedTemplateExpression, [{
		key: 'bind',
		value: function bind() {
			if (this.tag.type === 'Identifier') {
				var variable = this.scope.findVariable(this.tag.name);

				if (variable.isNamespace) {
					this.module.error({
						code: 'CANNOT_CALL_NAMESPACE',
						message: 'Cannot call a namespace (\'' + this.tag.name + '\')'
					}, this.start);
				}

				if (this.tag.name === 'eval' && variable.isGlobal) {
					this.module.warn({
						code: 'EVAL',
						message: 'Use of eval is strongly discouraged, as it poses security risks and may cause issues with minification',
						url: 'https://github.com/rollup/rollup/wiki/Troubleshooting#avoiding-eval'
					}, this.start);
				}
			}

			_get$22(TaggedTemplateExpression.prototype.__proto__ || Object.getPrototypeOf(TaggedTemplateExpression.prototype), 'bind', this).call(this);
			this.tag.bindCall({ withNew: false });
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return _get$22(TaggedTemplateExpression.prototype.__proto__ || Object.getPrototypeOf(TaggedTemplateExpression.prototype), 'hasEffects', this).call(this, options) || this.tag.hasEffectsWhenCalled(options.getHasEffectsWhenCalledOptions(this.tag));
		}
	}]);

	return TaggedTemplateExpression;
}(Node);

var _createClass$56 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$56(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$50(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$50(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var TemplateElement = function (_Node) {
	_inherits$50(TemplateElement, _Node);

	function TemplateElement() {
		_classCallCheck$56(this, TemplateElement);

		return _possibleConstructorReturn$50(this, (TemplateElement.__proto__ || Object.getPrototypeOf(TemplateElement)).apply(this, arguments));
	}

	_createClass$56(TemplateElement, [{
		key: 'hasEffects',
		value: function hasEffects() {
			return false;
		}
	}]);

	return TemplateElement;
}(Node);

var _createClass$57 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$23 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$57(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$51(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$51(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var TemplateLiteral = function (_Node) {
	_inherits$51(TemplateLiteral, _Node);

	function TemplateLiteral() {
		_classCallCheck$57(this, TemplateLiteral);

		return _possibleConstructorReturn$51(this, (TemplateLiteral.__proto__ || Object.getPrototypeOf(TemplateLiteral)).apply(this, arguments));
	}

	_createClass$57(TemplateLiteral, [{
		key: 'render',
		value: function render(code, es) {
			code.indentExclusionRanges.push([this.start, this.end]);
			_get$23(TemplateLiteral.prototype.__proto__ || Object.getPrototypeOf(TemplateLiteral.prototype), 'render', this).call(this, code, es);
		}
	}]);

	return TemplateLiteral;
}(Node);

var _createClass$58 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$58(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$52(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$52(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ThisExpression = function (_Node) {
	_inherits$52(ThisExpression, _Node);

	function ThisExpression() {
		_classCallCheck$58(this, ThisExpression);

		return _possibleConstructorReturn$52(this, (ThisExpression.__proto__ || Object.getPrototypeOf(ThisExpression)).apply(this, arguments));
	}

	_createClass$58(ThisExpression, [{
		key: 'initialiseNode',
		value: function initialiseNode() {
			var lexicalBoundary = this.scope.findLexicalBoundary();

			if (lexicalBoundary.isModuleScope) {
				this.alias = this.module.context;
				if (this.alias === 'undefined') {
					this.module.warn({
						code: 'THIS_IS_UNDEFINED',
						message: 'The \'this\' keyword is equivalent to \'undefined\' at the top level of an ES module, and has been rewritten',
						url: 'https://github.com/rollup/rollup/wiki/Troubleshooting#this-is-undefined'
					}, this.start);
				}
			}
		}
	}, {
		key: 'bind',
		value: function bind() {
			this.variable = this.scope.findVariable('this');
		}
	}, {
		key: 'hasEffectsWhenMutated',
		value: function hasEffectsWhenMutated(options) {
			return !options.ignoreSafeThisMutations() || this.variable.hasEffectsWhenMutated(options);
		}
	}, {
		key: 'render',
		value: function render(code) {
			if (this.alias) {
				code.overwrite(this.start, this.end, this.alias, { storeName: true, contentOnly: false });
			}
		}
	}]);

	return ThisExpression;
}(Node);

var _createClass$59 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$59(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$53(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$53(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ThrowStatement = function (_Node) {
	_inherits$53(ThrowStatement, _Node);

	function ThrowStatement() {
		_classCallCheck$59(this, ThrowStatement);

		return _possibleConstructorReturn$53(this, (ThrowStatement.__proto__ || Object.getPrototypeOf(ThrowStatement)).apply(this, arguments));
	}

	_createClass$59(ThrowStatement, [{
		key: 'hasEffects',
		value: function hasEffects() {
			return true;
		}
	}]);

	return ThrowStatement;
}(Node);

var _createClass$60 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$24 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _typeof2 = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _classCallCheck$60(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$54(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$54(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var operators$2 = {
	'-': function _(value) {
		return -value;
	},
	'+': function _(value) {
		return +value;
	},
	'!': function _(value) {
		return !value;
	},
	'~': function _(value) {
		return ~value;
	},
	typeof: function _typeof(value) {
		return typeof value === 'undefined' ? 'undefined' : _typeof2(value);
	},
	void: function _void() {
		return undefined;
	},
	delete: function _delete() {
		return UNKNOWN_VALUE;
	}
};

var UnaryExpression = function (_Node) {
	_inherits$54(UnaryExpression, _Node);

	function UnaryExpression() {
		_classCallCheck$60(this, UnaryExpression);

		return _possibleConstructorReturn$54(this, (UnaryExpression.__proto__ || Object.getPrototypeOf(UnaryExpression)).apply(this, arguments));
	}

	_createClass$60(UnaryExpression, [{
		key: 'bind',
		value: function bind() {
			if (this.value === UNKNOWN_VALUE) _get$24(UnaryExpression.prototype.__proto__ || Object.getPrototypeOf(UnaryExpression.prototype), 'bind', this).call(this);
		}
	}, {
		key: 'getValue',
		value: function getValue() {
			var argumentValue = this.argument.getValue();
			if (argumentValue === UNKNOWN_VALUE) return UNKNOWN_VALUE;

			return operators$2[this.operator](argumentValue);
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.included || this.argument.hasEffects(options) || this.operator === 'delete' && (this.argument.type !== 'MemberExpression' || this.argument.object.hasEffectsWhenMutated(options));
		}
	}, {
		key: 'hasEffectsAsExpressionStatement',
		value: function hasEffectsAsExpressionStatement(options) {
			return this.hasEffects(options);
		}
	}, {
		key: 'initialiseNode',
		value: function initialiseNode() {
			this.value = this.getValue();
		}
	}]);

	return UnaryExpression;
}(Node);

var _createClass$61 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$25 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$61(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$55(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$55(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var UpdateExpression = function (_Node) {
	_inherits$55(UpdateExpression, _Node);

	function UpdateExpression() {
		_classCallCheck$61(this, UpdateExpression);

		return _possibleConstructorReturn$55(this, (UpdateExpression.__proto__ || Object.getPrototypeOf(UpdateExpression)).apply(this, arguments));
	}

	_createClass$61(UpdateExpression, [{
		key: 'bind',
		value: function bind() {
			disallowIllegalReassignment(this.scope, this.argument);
			if (this.argument.type === 'Identifier') {
				var variable = this.scope.findVariable(this.argument.name);
				variable.isReassigned = true;
			}
			_get$25(UpdateExpression.prototype.__proto__ || Object.getPrototypeOf(UpdateExpression.prototype), 'bind', this).call(this);
		}
	}, {
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.included || this.argument.hasEffectsWhenAssigned(options);
		}
	}, {
		key: 'hasEffectsAsExpressionStatement',
		value: function hasEffectsAsExpressionStatement(options) {
			return this.hasEffects(options);
		}
	}]);

	return UpdateExpression;
}(Node);

var _createClass$62 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$26 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$62(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$56(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$56(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var VariableDeclarator = function (_Node) {
	_inherits$56(VariableDeclarator, _Node);

	function VariableDeclarator() {
		_classCallCheck$62(this, VariableDeclarator);

		return _possibleConstructorReturn$56(this, (VariableDeclarator.__proto__ || Object.getPrototypeOf(VariableDeclarator)).apply(this, arguments));
	}

	_createClass$62(VariableDeclarator, [{
		key: 'bindAssignment',
		value: function bindAssignment(expression) {
			this.id.bindAssignment(expression);
		}
	}, {
		key: 'initialiseDeclarator',
		value: function initialiseDeclarator(parentScope, kind) {
			this.initialiseScope(parentScope);
			this.init && this.init.initialise(this.scope);
			this.id.initialiseAndDeclare(this.scope, kind, this.init);
		}

		// TODO Deleting this does not break any tests. Find meaningful test or delete.

	}, {
		key: 'render',
		value: function render(code, es) {
			var _this2 = this;

			extractNames(this.id).forEach(function (name) {
				var variable = _this2.scope.findVariable(name);

				if (!es && variable.exportName && variable.isReassigned) {
					if (_this2.init) {
						code.overwrite(_this2.start, _this2.id.end, variable.getName(es));
					} else if (_this2.module.bundle.treeshake) {
						code.remove(_this2.start, _this2.end);
					}
				}
			});

			_get$26(VariableDeclarator.prototype.__proto__ || Object.getPrototypeOf(VariableDeclarator.prototype), 'render', this).call(this, code, es);
		}
	}]);

	return VariableDeclarator;
}(Node);

var _createClass$63 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$63(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$57(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$57(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function getSeparator(code, start) {
	var c = start;

	while (c > 0 && code[c - 1] !== '\n') {
		c -= 1;
		if (code[c] === ';' || code[c] === '{') return '; ';
	}

	var lineStart = code.slice(c, start).match(/^\s*/)[0];

	return ';\n' + lineStart;
}

var forStatement = /^For(?:Of|In)?Statement/;

var VariableDeclaration = function (_Node) {
	_inherits$57(VariableDeclaration, _Node);

	function VariableDeclaration() {
		_classCallCheck$63(this, VariableDeclaration);

		return _possibleConstructorReturn$57(this, (VariableDeclaration.__proto__ || Object.getPrototypeOf(VariableDeclaration)).apply(this, arguments));
	}

	_createClass$63(VariableDeclaration, [{
		key: 'bindAssignment',
		value: function bindAssignment() {
			this.eachChild(function (child) {
				return child.bindAssignment(UNKNOWN_ASSIGNMENT);
			});
		}
	}, {
		key: 'hasEffectsWhenAssigned',
		value: function hasEffectsWhenAssigned() {
			return false;
		}
	}, {
		key: 'includeWithAllDeclarations',
		value: function includeWithAllDeclarations() {
			if (this.isFullyIncluded()) return false;
			var addedNewNodes = false;
			this.declarations.forEach(function (declarator) {
				if (declarator.includeInBundle()) {
					addedNewNodes = true;
				}
			});
			if (!this.included || addedNewNodes) {
				this.included = true;
				return true;
			}
			return false;
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			if (this.isFullyIncluded()) return false;
			var addedNewNodes = false;
			this.declarations.forEach(function (declarator) {
				if (declarator.shouldBeIncluded()) {
					if (declarator.includeInBundle()) {
						addedNewNodes = true;
					}
				}
			});
			if (!this.included || addedNewNodes) {
				this.included = true;
				return true;
			}
			return false;
		}
	}, {
		key: 'initialiseChildren',
		value: function initialiseChildren() {
			var _this2 = this;

			this.declarations.forEach(function (child) {
				return child.initialiseDeclarator(_this2.scope, _this2.kind);
			});
		}
	}, {
		key: 'render',
		value: function render(code, es) {
			var _this3 = this;

			var treeshake = this.module.bundle.treeshake;

			var shouldSeparate = false;
			var separator = void 0;

			if (this.scope.isModuleScope && !forStatement.test(this.parent.type)) {
				shouldSeparate = true;
				separator = getSeparator(this.module.code, this.start);
			}

			var c = this.start;
			var empty = true;

			var _loop = function _loop(i) {
				var declarator = _this3.declarations[i];

				var prefix = empty ? '' : separator; // TODO indentation

				if (declarator.id.type === 'Identifier') {
					var variable = _this3.scope.findVariable(declarator.id.name);
					var isExportedAndReassigned = !es && variable.exportName && variable.isReassigned;

					if (isExportedAndReassigned) {
						if (declarator.init) {
							if (shouldSeparate) code.overwrite(c, declarator.start, prefix);
							c = declarator.end;
							empty = false;
						}
					} else if (!treeshake || variable.included) {
						if (shouldSeparate) code.overwrite(c, declarator.start, '' + prefix + _this3.kind + ' '); // TODO indentation
						c = declarator.end;
						empty = false;
					}
				} else {
					var exportAssignments = [];
					var isIncluded = false;

					extractNames(declarator.id).forEach(function (name) {
						var variable = _this3.scope.findVariable(name);
						var isExportedAndReassigned = !es && variable.exportName && variable.isReassigned;

						if (isExportedAndReassigned) {
							// code.overwrite( c, declarator.start, prefix );
							// c = declarator.end;
							// empty = false;
							exportAssignments.push('TODO');
						} else if (declarator.included) {
							isIncluded = true;
						}
					});

					if (!treeshake || isIncluded) {
						if (shouldSeparate) code.overwrite(c, declarator.start, '' + prefix + _this3.kind + ' '); // TODO indentation
						c = declarator.end;
						empty = false;
					}

					if (exportAssignments.length) {
						throw new Error('TODO');
					}
				}

				declarator.render(code, es);
			};

			for (var i = 0; i < this.declarations.length; i += 1) {
				_loop(i);
			}

			if (treeshake && empty) {
				code.remove(this.leadingCommentStart || this.start, this.next || this.end);
			} else {
				// always include a semi-colon (https://github.com/rollup/rollup/pull/1013),
				// unless it's a var declaration in a loop head
				var needsSemicolon = !forStatement.test(this.parent.type) || this === this.parent.body;

				if (this.end > c) {
					code.overwrite(c, this.end, needsSemicolon ? ';' : '');
				} else if (needsSemicolon) {
					this.insertSemicolon(code);
				}
			}
		}
	}]);

	return VariableDeclaration;
}(Node);

var _createClass$64 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$64(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$58(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$58(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var WhileStatement = function (_Statement) {
	_inherits$58(WhileStatement, _Statement);

	function WhileStatement() {
		_classCallCheck$64(this, WhileStatement);

		return _possibleConstructorReturn$58(this, (WhileStatement.__proto__ || Object.getPrototypeOf(WhileStatement)).apply(this, arguments));
	}

	_createClass$64(WhileStatement, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return this.included || this.test.hasEffects(options) || this.body.hasEffects(options.setIgnoreBreakStatements());
		}
	}]);

	return WhileStatement;
}(Statement);

var _createClass$65 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$27 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$65(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$59(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$59(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var YieldExpression = function (_Node) {
	_inherits$59(YieldExpression, _Node);

	function YieldExpression() {
		_classCallCheck$65(this, YieldExpression);

		return _possibleConstructorReturn$59(this, (YieldExpression.__proto__ || Object.getPrototypeOf(YieldExpression)).apply(this, arguments));
	}

	_createClass$65(YieldExpression, [{
		key: 'hasEffects',
		value: function hasEffects(options) {
			return _get$27(YieldExpression.prototype.__proto__ || Object.getPrototypeOf(YieldExpression.prototype), 'hasEffects', this).call(this, options) || !options.ignoreReturnAwaitYield();
		}
	}, {
		key: 'hasEffectsAsExpressionStatement',
		value: function hasEffectsAsExpressionStatement(options) {
			return this.hasEffects(options);
		}
	}]);

	return YieldExpression;
}(Node);

var nodes = {
	ArrayExpression: Node,
	ArrayPattern: ArrayPattern,
	ArrowFunctionExpression: ArrowFunctionExpression,
	AssignmentExpression: AssignmentExpression,
	AssignmentPattern: AssignmentPattern,
	AwaitExpression: AwaitExpression,
	BinaryExpression: BinaryExpression,
	BlockStatement: BlockStatement,
	BreakStatement: BreakStatement,
	CallExpression: CallExpression,
	CatchClause: CatchClause,
	ClassBody: ClassBody,
	ClassDeclaration: ClassDeclaration,
	ClassExpression: ClassExpression,
	ConditionalExpression: ConditionalExpression,
	DoWhileStatement: DoWhileStatement,
	EmptyStatement: EmptyStatement,
	ExportAllDeclaration: ExportAllDeclaration,
	ExportDefaultDeclaration: ExportDefaultDeclaration,
	ExportNamedDeclaration: ExportNamedDeclaration,
	ExpressionStatement: ExpressionStatement,
	ForStatement: ForStatement,
	ForInStatement: ForInStatement,
	ForOfStatement: ForOfStatement,
	FunctionDeclaration: FunctionDeclaration,
	FunctionExpression: FunctionExpression,
	Identifier: Identifier,
	IfStatement: IfStatement,
	ImportDeclaration: ImportDeclaration,
	LabeledStatement: LabeledStatement,
	Literal: Literal,
	LogicalExpression: LogicalExpression,
	MemberExpression: MemberExpression,
	MethodDefinition: MethodDefinition,
	NewExpression: NewExpression,
	ObjectExpression: ObjectExpression,
	ObjectPattern: ObjectPattern,
	Property: Property,
	RestElement: RestElement,
	ReturnStatement: ReturnStatement,
	SwitchCase: SwitchCase,
	SwitchStatement: SwitchStatement,
	TaggedTemplateExpression: TaggedTemplateExpression,
	TemplateElement: TemplateElement,
	TemplateLiteral: TemplateLiteral,
	ThisExpression: ThisExpression,
	ThrowStatement: ThrowStatement,
	TryStatement: Statement,
	UnaryExpression: UnaryExpression,
	UpdateExpression: UpdateExpression,
	VariableDeclarator: VariableDeclarator,
	VariableDeclaration: VariableDeclaration,
	WhileStatement: WhileStatement,
	YieldExpression: YieldExpression
};

var _createClass$66 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$66(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$60(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$60(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var UnknownNode = function (_Node) {
	_inherits$60(UnknownNode, _Node);

	function UnknownNode() {
		_classCallCheck$66(this, UnknownNode);

		return _possibleConstructorReturn$60(this, (UnknownNode.__proto__ || Object.getPrototypeOf(UnknownNode)).apply(this, arguments));
	}

	_createClass$66(UnknownNode, [{
		key: 'hasEffects',
		value: function hasEffects() {
			return true;
		}
	}]);

	return UnknownNode;
}(Node);

var keys$1 = {
	Program: ['body'],
	Literal: []
};

var _typeof$1 = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var newline = /\n/;

function enhance(ast, module, comments) {
	enhanceNode(ast, module, module, module.magicString);

	var comment = comments.shift();

	var _iteratorNormalCompletion = true;
	var _didIteratorError = false;
	var _iteratorError = undefined;

	try {
		for (var _iterator = ast.body[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
			var node = _step.value;

			if (comment && comment.start < node.start) {
				node.leadingCommentStart = comment.start;
			}

			while (comment && comment.end < node.end) {
				comment = comments.shift();
			} // if the next comment is on the same line as the end of the node,
			// treat is as a trailing comment
			if (comment && !newline.test(module.code.slice(node.end, comment.start))) {
				node.trailingCommentEnd = comment.end; // TODO is node.trailingCommentEnd used anywhere?
				comment = comments.shift();
			}

			node.initialise(module.scope);
		}
	} catch (err) {
		_didIteratorError = true;
		_iteratorError = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion && _iterator.return) {
				_iterator.return();
			}
		} finally {
			if (_didIteratorError) {
				throw _iteratorError;
			}
		}
	}
}

function enhanceNode(raw, parent, module, code) {
	if (!raw) return;

	if ('length' in raw) {
		for (var i = 0; i < raw.length; i += 1) {
			enhanceNode(raw[i], parent, module, code);
		}

		return;
	}

	// with e.g. shorthand properties, key and value are
	// the same node. We don't want to enhance an object twice
	if (raw.__enhanced) return;
	raw.__enhanced = true;

	if (!keys$1[raw.type]) {
		keys$1[raw.type] = Object.keys(raw).filter(function (key) {
			return _typeof$1(raw[key]) === 'object';
		});
	}

	raw.parent = parent;
	raw.module = module;
	raw.keys = keys$1[raw.type];

	code.addSourcemapLocation(raw.start);
	code.addSourcemapLocation(raw.end);

	var _iteratorNormalCompletion2 = true;
	var _didIteratorError2 = false;
	var _iteratorError2 = undefined;

	try {
		for (var _iterator2 = keys$1[raw.type][Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
			var key = _step2.value;

			enhanceNode(raw[key], raw, module, code);
		}
	} catch (err) {
		_didIteratorError2 = true;
		_iteratorError2 = err;
	} finally {
		try {
			if (!_iteratorNormalCompletion2 && _iterator2.return) {
				_iterator2.return();
			}
		} finally {
			if (_didIteratorError2) {
				throw _iteratorError2;
			}
		}
	}

	var type = nodes[raw.type] || UnknownNode;
	raw.__proto__ = type.prototype;
}

var _typeof$2 = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function clone(node) {
	if (!node) return node;
	if ((typeof node === 'undefined' ? 'undefined' : _typeof$2(node)) !== 'object') return node;

	if (Array.isArray(node)) {
		var _cloned = new Array(node.length);
		for (var i = 0; i < node.length; i += 1) {
			_cloned[i] = clone(node[i]);
		}return _cloned;
	}

	var cloned = {};
	for (var key in node) {
		cloned[key] = clone(node[key]);
	}

	return cloned;
}

var _createClass$67 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get$28 = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

function _classCallCheck$67(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$61(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$61(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ModuleScope = function (_Scope) {
	_inherits$61(ModuleScope, _Scope);

	function ModuleScope(module) {
		_classCallCheck$67(this, ModuleScope);

		var _this = _possibleConstructorReturn$61(this, (ModuleScope.__proto__ || Object.getPrototypeOf(ModuleScope)).call(this, {
			isModuleScope: true,
			parent: module.bundle.scope
		}));

		_this.module = module;
		_this.variables.this = new LocalVariable('this', null, UNDEFINED_ASSIGNMENT);
		return _this;
	}

	_createClass$67(ModuleScope, [{
		key: 'deshadow',
		value: function deshadow(names) {
			var _this2 = this;

			names = new Set(names);

			forOwn(this.module.imports, function (specifier) {
				if (specifier.module.isExternal) return;

				var addDeclaration = function addDeclaration(declaration) {
					if (declaration.isNamespace && !declaration.isExternal) {
						declaration.module.getExports().forEach(function (name) {
							addDeclaration(declaration.module.traceExport(name));
						});
					}

					names.add(declaration.name);
				};

				specifier.module.getExports().forEach(function (name) {
					addDeclaration(specifier.module.traceExport(name));
				});

				if (specifier.name !== '*') {
					var declaration = specifier.module.traceExport(specifier.name);
					if (!declaration) {
						_this2.module.warn({
							code: 'NON_EXISTENT_EXPORT',
							name: specifier.name,
							source: specifier.module.id,
							message: 'Non-existent export \'' + specifier.name + '\' is imported from ' + relativeId(specifier.module.id)
						}, specifier.specifier.start);
						return;
					}

					var name = declaration.getName(true);
					if (name !== specifier.name) {
						names.add(declaration.getName(true));
					}

					if (specifier.name !== 'default' && specifier.specifier.imported.name !== specifier.specifier.local.name) {
						names.add(specifier.specifier.imported.name);
					}
				}
			});

			_get$28(ModuleScope.prototype.__proto__ || Object.getPrototypeOf(ModuleScope.prototype), 'deshadow', this).call(this, names);
		}
	}, {
		key: 'findLexicalBoundary',
		value: function findLexicalBoundary() {
			return this;
		}
	}, {
		key: 'findVariable',
		value: function findVariable(name) {
			if (this.variables[name]) {
				return this.variables[name];
			}

			return this.module.trace(name) || this.parent.findVariable(name);
		}
	}]);

	return ModuleScope;
}(Scope);

var _createClass$1 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$1(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function tryParse(module, acornOptions) {
	try {
		return acorn.parse(module.code, assign({
			ecmaVersion: 8,
			sourceType: 'module',
			onComment: function onComment(block, text, start, end) {
				return module.comments.push({ block: block, text: text, start: start, end: end });
			},
			preserveParens: false
		}, acornOptions));
	} catch (err) {
		module.error({
			code: 'PARSE_ERROR',
			message: err.message.replace(/ \(\d+:\d+\)$/, '')
		}, err.pos);
	}
}

function includeFully(node) {
	node.includeInBundle();
	node.eachChild(includeFully);
}

var Module = function () {
	function Module(_ref) {
		var _this = this;

		var id = _ref.id,
		    code = _ref.code,
		    originalCode = _ref.originalCode,
		    originalSourcemap = _ref.originalSourcemap,
		    ast = _ref.ast,
		    sourcemapChain = _ref.sourcemapChain,
		    resolvedIds = _ref.resolvedIds,
		    resolvedExternalIds = _ref.resolvedExternalIds,
		    bundle = _ref.bundle;

		_classCallCheck$1(this, Module);

		this.code = code;
		this.id = id;
		this.bundle = bundle;
		this.originalCode = originalCode;
		this.originalSourcemap = originalSourcemap;
		this.sourcemapChain = sourcemapChain;

		this.comments = [];

		timeStart('ast');

		if (ast) {
			// prevent mutating the provided AST, as it may be reused on
			// subsequent incremental rebuilds
			this.ast = clone(ast);
			this.astClone = ast;
		} else {
			this.ast = tryParse(this, bundle.acornOptions); // TODO what happens to comments if AST is provided?
			this.astClone = clone(this.ast);
		}

		timeEnd('ast');

		this.excludeFromSourcemap = /\0/.test(id);
		this.context = bundle.getModuleContext(id);

		// all dependencies
		this.sources = [];
		this.dependencies = [];
		this.resolvedIds = resolvedIds || blank();
		this.resolvedExternalIds = resolvedExternalIds || blank();

		// imports and exports, indexed by local name
		this.imports = blank();
		this.exports = blank();
		this.exportsAll = blank();
		this.reexports = blank();

		this.exportAllSources = [];
		this.exportAllModules = null;

		// By default, `id` is the filename. Custom resolvers and loaders
		// can change that, but it makes sense to use it for the source filename
		this.magicString = new MagicString__default(code, {
			filename: this.excludeFromSourcemap ? null : id, // don't include plugin helpers in sourcemap
			indentExclusionRanges: []
		});

		// remove existing sourceMappingURL comments
		this.comments = this.comments.filter(function (comment) {
			//only one line comment can contain source maps
			var isSourceMapComment = !comment.block && SOURCEMAPPING_URL_RE.test(comment.text);
			if (isSourceMapComment) {
				_this.magicString.remove(comment.start, comment.end);
			}
			return !isSourceMapComment;
		});

		this.declarations = blank();
		this.type = 'Module'; // TODO only necessary so that Scope knows this should be treated as a function scope... messy
		this.scope = new ModuleScope(this);

		timeStart('analyse');

		this.analyse();

		timeEnd('analyse');

		this.strongDependencies = [];
	}

	_createClass$1(Module, [{
		key: 'addExport',
		value: function addExport(node) {
			var _this2 = this;

			var source = node.source && node.source.value;

			// export { name } from './other.js'
			if (source) {
				if (!~this.sources.indexOf(source)) this.sources.push(source);

				if (node.type === 'ExportAllDeclaration') {
					// Store `export * from '...'` statements in an array of delegates.
					// When an unknown import is encountered, we see if one of them can satisfy it.
					this.exportAllSources.push(source);
				} else {
					node.specifiers.forEach(function (specifier) {
						var name = specifier.exported.name;

						if (_this2.exports[name] || _this2.reexports[name]) {
							_this2.error({
								code: 'DUPLICATE_EXPORT',
								message: 'A module cannot have multiple exports with the same name (\'' + name + '\')'
							}, specifier.start);
						}

						_this2.reexports[name] = {
							start: specifier.start,
							source: source,
							localName: specifier.local.name,
							module: null // filled in later
						};
					});
				}
			}

			// export default function foo () {}
			// export default foo;
			// export default 42;
			else if (node.type === 'ExportDefaultDeclaration') {
					var identifier = node.declaration.id && node.declaration.id.name || node.declaration.name;

					if (this.exports.default) {
						this.error({
							code: 'DUPLICATE_EXPORT',
							message: 'A module can only have one default export'
						}, node.start);
					}

					this.exports.default = {
						localName: 'default',
						identifier: identifier
					};
				}

				// export var { foo, bar } = ...
				// export var foo = 42;
				// export var a = 1, b = 2, c = 3;
				// export function foo () {}
				else if (node.declaration) {
						var declaration = node.declaration;

						if (declaration.type === 'VariableDeclaration') {
							declaration.declarations.forEach(function (decl) {
								extractNames(decl.id).forEach(function (localName) {
									_this2.exports[localName] = { localName: localName };
								});
							});
						} else {
							// export function foo () {}
							var localName = declaration.id.name;
							this.exports[localName] = { localName: localName };
						}
					}

					// export { foo, bar, baz }
					else {
							node.specifiers.forEach(function (specifier) {
								var localName = specifier.local.name;
								var exportedName = specifier.exported.name;

								if (_this2.exports[exportedName] || _this2.reexports[exportedName]) {
									_this2.error({
										code: 'DUPLICATE_EXPORT',
										message: 'A module cannot have multiple exports with the same name (\'' + exportedName + '\')'
									}, specifier.start);
								}

								_this2.exports[exportedName] = { localName: localName };
							});
						}
		}
	}, {
		key: 'addImport',
		value: function addImport(node) {
			var _this3 = this;

			var source = node.source.value;

			if (!~this.sources.indexOf(source)) this.sources.push(source);

			node.specifiers.forEach(function (specifier) {
				var localName = specifier.local.name;

				if (_this3.imports[localName]) {
					_this3.error({
						code: 'DUPLICATE_IMPORT',
						message: 'Duplicated import \'' + localName + '\''
					}, specifier.start);
				}

				var isDefault = specifier.type === 'ImportDefaultSpecifier';
				var isNamespace = specifier.type === 'ImportNamespaceSpecifier';

				var name = isDefault ? 'default' : isNamespace ? '*' : specifier.imported.name;
				_this3.imports[localName] = { source: source, specifier: specifier, name: name, module: null };
			});
		}
	}, {
		key: 'analyse',
		value: function analyse() {
			enhance(this.ast, this, this.comments);

			// discover this module's imports and exports
			var lastNode = void 0;

			var _iteratorNormalCompletion = true;
			var _didIteratorError = false;
			var _iteratorError = undefined;

			try {
				for (var _iterator = this.ast.body[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
					var node = _step.value;

					if (node.isImportDeclaration) {
						this.addImport(node);
					} else if (node.isExportDeclaration) {
						this.addExport(node);
					}

					if (lastNode) lastNode.next = node.leadingCommentStart || node.start;
					lastNode = node;
				}
			} catch (err) {
				_didIteratorError = true;
				_iteratorError = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion && _iterator.return) {
						_iterator.return();
					}
				} finally {
					if (_didIteratorError) {
						throw _iteratorError;
					}
				}
			}
		}
	}, {
		key: 'basename',
		value: function basename$$1() {
			var base = path.basename(this.id);
			var ext = path.extname(this.id);

			return makeLegal(ext ? base.slice(0, -ext.length) : base);
		}
	}, {
		key: 'bindImportSpecifiers',
		value: function bindImportSpecifiers() {
			var _this4 = this;

			[this.imports, this.reexports].forEach(function (specifiers) {
				keys(specifiers).forEach(function (name) {
					var specifier = specifiers[name];

					var id = _this4.resolvedIds[specifier.source] || _this4.resolvedExternalIds[specifier.source];
					specifier.module = _this4.bundle.moduleById.get(id);
				});
			});

			this.exportAllModules = this.exportAllSources.map(function (source) {
				var id = _this4.resolvedIds[source] || _this4.resolvedExternalIds[source];
				return _this4.bundle.moduleById.get(id);
			});

			this.sources.forEach(function (source) {
				var id = _this4.resolvedIds[source];

				if (id) {
					var module = _this4.bundle.moduleById.get(id);
					_this4.dependencies.push(module);
				}
			});
		}
	}, {
		key: 'bindReferences',
		value: function bindReferences() {
			var _iteratorNormalCompletion2 = true;
			var _didIteratorError2 = false;
			var _iteratorError2 = undefined;

			try {
				for (var _iterator2 = this.ast.body[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
					var node = _step2.value;

					node.bind();
				}

				// if ( this.declarations.default ) {
				// 	if ( this.exports.default.identifier ) {
				// 		const declaration = this.trace( this.exports.default.identifier );
				// 		if ( declaration ) this.declarations.default.bind( declaration );
				// 	}
				// }
			} catch (err) {
				_didIteratorError2 = true;
				_iteratorError2 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion2 && _iterator2.return) {
						_iterator2.return();
					}
				} finally {
					if (_didIteratorError2) {
						throw _iteratorError2;
					}
				}
			}
		}
	}, {
		key: 'error',
		value: function error$$1(props, pos) {
			if (pos !== undefined) {
				props.pos = pos;

				var _locate = locateCharacter.locate(this.code, pos, { offsetLine: 1 }),
				    line = _locate.line,
				    column = _locate.column; // TODO trace sourcemaps

				props.loc = { file: this.id, line: line, column: column };
				props.frame = getCodeFrame(this.code, line, column);
			}

			error(props);
		}
	}, {
		key: 'getExports',
		value: function getExports() {
			return keys(this.exports);
		}
	}, {
		key: 'getReexports',
		value: function getReexports() {
			var reexports = blank();

			keys(this.reexports).forEach(function (name) {
				reexports[name] = true;
			});

			this.exportAllModules.forEach(function (module) {
				if (module.isExternal) {
					reexports['*' + module.id] = true;
					return;
				}

				module.getExports().concat(module.getReexports()).forEach(function (name) {
					if (name !== 'default') reexports[name] = true;
				});
			});

			return keys(reexports);
		}
	}, {
		key: 'includeAllInBundle',
		value: function includeAllInBundle() {
			this.ast.body.forEach(includeFully);
		}
	}, {
		key: 'includeInBundle',
		value: function includeInBundle() {
			var addedNewNodes = false;
			this.ast.body.forEach(function (node) {
				if (node.shouldBeIncluded()) {
					if (node.includeInBundle()) {
						addedNewNodes = true;
					}
				}
			});
			return addedNewNodes;
		}
	}, {
		key: 'namespace',
		value: function namespace() {
			if (!this.declarations['*']) {
				this.declarations['*'] = new NamespaceVariable(this);
			}

			return this.declarations['*'];
		}
	}, {
		key: 'render',
		value: function render(es, legacy) {
			var magicString = this.magicString.clone();

			var _iteratorNormalCompletion3 = true;
			var _didIteratorError3 = false;
			var _iteratorError3 = undefined;

			try {
				for (var _iterator3 = this.ast.body[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
					var node = _step3.value;

					node.render(magicString, es);
				}
			} catch (err) {
				_didIteratorError3 = true;
				_iteratorError3 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion3 && _iterator3.return) {
						_iterator3.return();
					}
				} finally {
					if (_didIteratorError3) {
						throw _iteratorError3;
					}
				}
			}

			if (this.namespace().needsNamespaceBlock) {
				magicString.append('\n\n' + this.namespace().renderBlock(es, legacy, '\t')); // TODO use correct indentation
			}

			return magicString.trim();
		}
	}, {
		key: 'toJSON',
		value: function toJSON() {
			return {
				id: this.id,
				dependencies: this.dependencies.map(function (module) {
					return module.id;
				}),
				code: this.code,
				originalCode: this.originalCode,
				originalSourcemap: this.originalSourcemap,
				ast: this.astClone,
				sourcemapChain: this.sourcemapChain,
				resolvedIds: this.resolvedIds,
				resolvedExternalIds: this.resolvedExternalIds
			};
		}
	}, {
		key: 'trace',
		value: function trace(name) {
			// TODO this is slightly circular
			if (name in this.scope.variables) {
				return this.scope.variables[name];
			}

			if (name in this.imports) {
				var importDeclaration = this.imports[name];
				var otherModule = importDeclaration.module;

				if (importDeclaration.name === '*' && !otherModule.isExternal) {
					return otherModule.namespace();
				}

				var declaration = otherModule.traceExport(importDeclaration.name);

				if (!declaration) {
					this.error({
						code: 'MISSING_EXPORT',
						message: '\'' + importDeclaration.name + '\' is not exported by ' + relativeId(otherModule.id),
						url: 'https://github.com/rollup/rollup/wiki/Troubleshooting#name-is-not-exported-by-module'
					}, importDeclaration.specifier.start);
				}

				return declaration;
			}

			return null;
		}
	}, {
		key: 'traceExport',
		value: function traceExport(name) {
			// export * from 'external'
			if (name[0] === '*') {
				var module = this.bundle.moduleById.get(name.slice(1));
				return module.traceExport('*');
			}

			// export { foo } from './other.js'
			var reexportDeclaration = this.reexports[name];
			if (reexportDeclaration) {
				var declaration = reexportDeclaration.module.traceExport(reexportDeclaration.localName);

				if (!declaration) {
					this.error({
						code: 'MISSING_EXPORT',
						message: '\'' + reexportDeclaration.localName + '\' is not exported by ' + relativeId(reexportDeclaration.module.id),
						url: 'https://github.com/rollup/rollup/wiki/Troubleshooting#name-is-not-exported-by-module'
					}, reexportDeclaration.start);
				}

				return declaration;
			}

			var exportDeclaration = this.exports[name];
			if (exportDeclaration) {
				var _name = exportDeclaration.localName;
				var _declaration = this.trace(_name);

				return _declaration || this.bundle.scope.findVariable(_name);
			}

			if (name === 'default') return;

			for (var i = 0; i < this.exportAllModules.length; i += 1) {
				var _module = this.exportAllModules[i];
				var _declaration2 = _module.traceExport(name);

				if (_declaration2) return _declaration2;
			}
		}
	}, {
		key: 'warn',
		value: function warn(warning, pos) {
			if (pos !== undefined) {
				warning.pos = pos;

				var _locate2 = locateCharacter.locate(this.code, pos, { offsetLine: 1 }),
				    line = _locate2.line,
				    column = _locate2.column; // TODO trace sourcemaps

				warning.loc = { file: this.id, line: line, column: column };
				warning.frame = getCodeFrame(this.code, line, column);
			}

			warning.id = this.id;
			this.bundle.warn(warning);
		}
	}]);

	return Module;
}();

var _createClass$69 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$69(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$62(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$62(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ExternalVariable = function (_Variable) {
	_inherits$62(ExternalVariable, _Variable);

	function ExternalVariable(module, name) {
		_classCallCheck$69(this, ExternalVariable);

		var _this = _possibleConstructorReturn$62(this, (ExternalVariable.__proto__ || Object.getPrototypeOf(ExternalVariable)).call(this, name));

		_this.module = module;
		_this.safeName = null;
		_this.isExternal = true;
		_this.isNamespace = name === '*';
		return _this;
	}

	_createClass$69(ExternalVariable, [{
		key: 'addReference',
		value: function addReference(reference) {
			if (this.name === 'default' || this.name === '*') {
				this.module.suggestName(reference.name);
			}
		}
	}, {
		key: 'getName',
		value: function getName(es) {
			if (this.name === '*') {
				return this.module.name;
			}

			if (this.name === 'default') {
				return this.module.exportsNamespace || !es && this.module.exportsNames ? this.module.name + '__default' : this.module.name;
			}

			return es ? this.safeName : this.module.name + '.' + this.name;
		}
	}, {
		key: 'includeDeclaration',
		value: function includeDeclaration() {
			if (this.included) {
				return false;
			}
			this.included = true;
			this.module.used = true;
			return true;
		}
	}, {
		key: 'setSafeName',
		value: function setSafeName(name) {
			this.safeName = name;
		}
	}]);

	return ExternalVariable;
}(Variable);

var _createClass$68 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$68(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ExternalModule = function () {
	function ExternalModule(id) {
		_classCallCheck$68(this, ExternalModule);

		this.id = id;

		var parts = id.split(/[\\/]/);
		this.name = makeLegal(parts.pop());

		this.nameSuggestions = blank();
		this.mostCommonSuggestion = 0;

		this.isExternal = true;
		this.used = false;
		this.declarations = blank();

		this.exportsNames = false;
	}

	_createClass$68(ExternalModule, [{
		key: 'suggestName',
		value: function suggestName(name) {
			if (!this.nameSuggestions[name]) this.nameSuggestions[name] = 0;
			this.nameSuggestions[name] += 1;

			if (this.nameSuggestions[name] > this.mostCommonSuggestion) {
				this.mostCommonSuggestion = this.nameSuggestions[name];
				this.name = name;
			}
		}
	}, {
		key: 'traceExport',
		value: function traceExport(name) {
			if (name !== 'default' && name !== '*') this.exportsNames = true;
			if (name === '*') this.exportsNamespace = true;

			return this.declarations[name] || (this.declarations[name] = new ExternalVariable(this, name));
		}
	}]);

	return ExternalModule;
}();

function getInteropBlock(bundle, options) {
	return bundle.externalModules.map(function (module) {
		if (!module.declarations.default || options.interop === false) return null;

		if (module.exportsNamespace) {
			return bundle.varOrConst + ' ' + module.name + '__default = ' + module.name + '[\'default\'];';
		}

		if (module.exportsNames) {
			return bundle.varOrConst + ' ' + module.name + '__default = \'default\' in ' + module.name + ' ? ' + module.name + '[\'default\'] : ' + module.name + ';';
		}

		return module.name + ' = ' + module.name + ' && ' + module.name + '.hasOwnProperty(\'default\') ? ' + module.name + '[\'default\'] : ' + module.name + ';';
	}).filter(Boolean).join('\n');
}

function getExportBlock(bundle, exportMode) {
	var mechanism = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'return';

	var entryModule = bundle.entryModule;

	if (exportMode === 'default') {
		return mechanism + ' ' + entryModule.traceExport('default').getName(false) + ';';
	}

	var exports = entryModule.getExports().concat(entryModule.getReexports()).map(function (name) {
		if (name[0] === '*') {
			// export all from external
			var id = name.slice(1);
			var module = bundle.moduleById.get(id);

			return 'Object.keys(' + module.name + ').forEach(function (key) { exports[key] = ' + module.name + '[key]; });';
		}

		var prop = name === 'default' ? '[\'default\']' : '.' + name;
		var declaration = entryModule.traceExport(name);

		var lhs = 'exports' + prop;
		var rhs = declaration ? declaration.getName(false) : name; // exporting a global

		// prevent `exports.count = exports.count`
		if (lhs === rhs) return null;

		return lhs + ' = ' + rhs + ';';
	});

	return exports.filter(Boolean).join('\n');
}

var esModuleExport = "Object.defineProperty(exports, '__esModule', { value: true });";

var builtins$1 = {
	process: true,
	events: true,
	stream: true,
	util: true,
	path: true,
	buffer: true,
	querystring: true,
	url: true,
	string_decoder: true,
	punycode: true,
	http: true,
	https: true,
	os: true,
	assert: true,
	constants: true,
	timers: true,
	console: true,
	vm: true,
	zlib: true,
	tty: true,
	domain: true
};

// Creating a browser bundle that depends on Node.js built-in modules ('util'). You might need to include https://www.npmjs.com/package/rollup-plugin-node-builtins

function warnOnBuiltins(bundle) {
	var externalBuiltins = bundle.externalModules.filter(function (mod) {
		return mod.id in builtins$1;
	}).map(function (mod) {
		return mod.id;
	});

	if (!externalBuiltins.length) return;

	var detail = externalBuiltins.length === 1 ? 'module (\'' + externalBuiltins[0] + '\')' : 'modules (' + externalBuiltins.slice(0, -1).map(function (name) {
		return '\'' + name + '\'';
	}).join(', ') + ' and \'' + externalBuiltins.slice(-1) + '\')';

	bundle.warn({
		code: 'MISSING_NODE_BUILTINS',
		modules: externalBuiltins,
		message: 'Creating a browser bundle that depends on Node.js built-in ' + detail + '. You might need to include https://www.npmjs.com/package/rollup-plugin-node-builtins'
	});
}

function amd(bundle, magicString, _ref, options) {
	var exportMode = _ref.exportMode,
	    getPath = _ref.getPath,
	    indentString = _ref.indentString,
	    intro = _ref.intro,
	    outro = _ref.outro;

	warnOnBuiltins(bundle);
	var deps = bundle.externalModules.map(function (m) {
		return '\'' + getPath(m.id) + '\'';
	});
	var args = bundle.externalModules.map(function (m) {
		return m.name;
	});

	if (exportMode === 'named') {
		args.unshift('exports');
		deps.unshift('\'exports\'');
	}

	var amdOptions = options.amd || {};

	var params = (amdOptions.id ? '\'' + amdOptions.id + '\', ' : '') + (deps.length ? '[' + deps.join(', ') + '], ' : '');

	var useStrict = options.strict !== false ? ' \'use strict\';' : '';
	var define = amdOptions.define || 'define';
	var wrapperStart = define + '(' + params + 'function (' + args.join(', ') + ') {' + useStrict + '\n\n';

	// var foo__default = 'default' in foo ? foo['default'] : foo;
	var interopBlock = getInteropBlock(bundle, options);
	if (interopBlock) magicString.prepend(interopBlock + '\n\n');

	if (intro) magicString.prepend(intro);

	var exportBlock = getExportBlock(bundle, exportMode);
	if (exportBlock) magicString.append('\n\n' + exportBlock);
	if (exportMode === 'named' && options.legacy !== true) magicString.append('\n\n' + esModuleExport);
	if (outro) magicString.append(outro);

	return magicString.indent(indentString).append('\n\n});').prepend(wrapperStart);
}

function cjs(bundle, magicString, _ref, options) {
	var exportMode = _ref.exportMode,
	    getPath = _ref.getPath,
	    intro = _ref.intro,
	    outro = _ref.outro;

	intro = (options.strict === false ? intro : '\'use strict\';\n\n' + intro) + (exportMode === 'named' && options.legacy !== true ? esModuleExport + '\n\n' : '');

	var needsInterop = false;

	var varOrConst = bundle.varOrConst;
	var interop = options.interop !== false;

	// TODO handle empty imports, once they're supported
	var importBlock = bundle.externalModules.map(function (module) {
		if (interop && module.declarations.default) {
			if (module.exportsNamespace) {
				return varOrConst + ' ' + module.name + ' = require(\'' + getPath(module.id) + '\');' + ('\n' + varOrConst + ' ' + module.name + '__default = ' + module.name + '[\'default\'];');
			}

			needsInterop = true;

			if (module.exportsNames) {
				return varOrConst + ' ' + module.name + ' = require(\'' + getPath(module.id) + '\');' + ('\n' + varOrConst + ' ' + module.name + '__default = _interopDefault(' + module.name + ');');
			}

			return varOrConst + ' ' + module.name + ' = _interopDefault(require(\'' + getPath(module.id) + '\'));';
		} else {
			var includedDeclarations = Object.keys(module.declarations).filter(function (name) {
				return module.declarations[name].included;
			});

			var needsVar = includedDeclarations.length || module.reexported;

			return needsVar ? varOrConst + ' ' + module.name + ' = require(\'' + getPath(module.id) + '\');' : 'require(\'' + getPath(module.id) + '\');';
		}
	}).join('\n');

	if (needsInterop) {
		intro += 'function _interopDefault (ex) { return (ex && (typeof ex === \'object\') && \'default\' in ex) ? ex[\'default\'] : ex; }\n\n';
	}

	if (importBlock) {
		intro += importBlock + '\n\n';
	}

	magicString.prepend(intro);

	var exportBlock = getExportBlock(bundle, exportMode, 'module.exports =');
	if (exportBlock) magicString.append('\n\n' + exportBlock);
	if (outro) magicString.append(outro);

	return magicString;
}

function notDefault(name) {
	return name !== 'default';
}

function es(bundle, magicString, _ref) {
	var getPath = _ref.getPath,
	    intro = _ref.intro,
	    outro = _ref.outro;

	var importBlock = bundle.externalModules.map(function (module) {
		var specifiers = [];
		var specifiersList = [specifiers];
		var importedNames = keys(module.declarations).filter(function (name) {
			return name !== '*' && name !== 'default';
		}).filter(function (name) {
			return module.declarations[name].included;
		}).map(function (name) {
			if (name[0] === '*') {
				return '* as ' + module.name;
			}

			var declaration = module.declarations[name];

			if (declaration.name === declaration.safeName) return declaration.name;
			return declaration.name + ' as ' + declaration.safeName;
		}).filter(Boolean);

		if (module.declarations.default) {
			if (module.exportsNamespace) {
				specifiersList.push([module.name + '__default']);
			} else {
				specifiers.push(module.name);
			}
		}

		var namespaceSpecifier = module.declarations['*'] && module.declarations['*'].included ? '* as ' + module.name : null; // TODO prevent unnecessary namespace import, e.g form/external-imports
		var namedSpecifier = importedNames.length ? '{ ' + importedNames.sort().join(', ') + ' }' : null;

		if (namespaceSpecifier && namedSpecifier) {
			// Namespace and named specifiers cannot be combined.
			specifiersList.push([namespaceSpecifier]);
			specifiers.push(namedSpecifier);
		} else if (namedSpecifier) {
			specifiers.push(namedSpecifier);
		} else if (namespaceSpecifier) {
			specifiers.push(namespaceSpecifier);
		}

		return specifiersList.map(function (specifiers) {
			if (specifiers.length) {
				return 'import ' + specifiers.join(', ') + ' from \'' + getPath(module.id) + '\';';
			}

			return module.reexported ? null : 'import \'' + getPath(module.id) + '\';';
		}).filter(Boolean).join('\n');
	}).join('\n');

	if (importBlock) intro += importBlock + '\n\n';
	if (intro) magicString.prepend(intro);

	var module = bundle.entryModule;

	var exportInternalSpecifiers = [];
	var exportExternalSpecifiers = new Map();
	var exportAllDeclarations = [];

	module.getExports().filter(notDefault).forEach(function (name) {
		var declaration = module.traceExport(name);
		var rendered = declaration.getName(true);
		exportInternalSpecifiers.push(rendered === name ? name : rendered + ' as ' + name);
	});

	module.getReexports().filter(notDefault).forEach(function (name) {
		var declaration = module.traceExport(name);

		if (declaration.isExternal) {
			if (name[0] === '*') {
				// export * from 'external'
				exportAllDeclarations.push('export * from \'' + name.slice(1) + '\';');
			} else {
				if (!exportExternalSpecifiers.has(declaration.module.id)) exportExternalSpecifiers.set(declaration.module.id, []);
				exportExternalSpecifiers.get(declaration.module.id).push(name);
			}

			return;
		}

		var rendered = declaration.getName(true);
		exportInternalSpecifiers.push(rendered === name ? name : rendered + ' as ' + name);
	});

	var exportBlock = [];
	if (exportInternalSpecifiers.length) exportBlock.push('export { ' + exportInternalSpecifiers.join(', ') + ' };');
	if (module.exports.default || module.reexports.default) exportBlock.push('export default ' + module.traceExport('default').getName(true) + ';');
	if (exportAllDeclarations.length) exportBlock.push(exportAllDeclarations.join('\n'));
	if (exportExternalSpecifiers.size) {
		exportExternalSpecifiers.forEach(function (specifiers, id) {
			exportBlock.push('export { ' + specifiers.join(', ') + ' } from \'' + id + '\';');
		});
	}

	if (exportBlock.length) magicString.append('\n\n' + exportBlock.join('\n').trim());

	if (outro) magicString.append(outro);

	return magicString.trim();
}

function getGlobalNameMaker(globals, bundle) {
	var fallback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

	var fn = typeof globals === 'function' ? globals : function (id) {
		return globals[id];
	};

	return function (module) {
		var name = fn(module.id);
		if (name) return name;

		if (Object.keys(module.declarations).length > 0) {
			bundle.warn({
				code: 'MISSING_GLOBAL_NAME',
				source: module.id,
				guess: module.name,
				message: 'No name was provided for external module \'' + module.id + '\' in options.globals \u2013 guessing \'' + module.name + '\''
			});

			return module.name;
		}

		return fallback;
	};
}

// Generate strings which dereference dotted properties, but use array notation `['prop-deref']`
// if the property name isn't trivial
var shouldUseDot = /^[a-zA-Z$_][a-zA-Z0-9$_]*$/;

function property(prop) {
	return shouldUseDot.test(prop) ? '.' + prop : '[\'' + prop + '\']';
}

function keypath(keypath) {
	return keypath.split('.').map(property).join('');
}

function trimEmptyImports(modules) {
	var i = modules.length;

	while (i--) {
		var module = modules[i];
		if (Object.keys(module.declarations).length > 0) {
			return modules.slice(0, i + 1);
		}
	}

	return [];
}

function setupNamespace(keypath$$1) {
	var parts = keypath$$1.split('.');

	parts.pop();

	var acc = 'this';

	return parts.map(function (part) {
		return acc += property(part), acc + ' = ' + acc + ' || {};';
	}).join('\n') + '\n';
}

var thisProp = function thisProp(name) {
	return 'this' + keypath(name);
};

function iife(bundle, magicString, _ref, options) {
	var exportMode = _ref.exportMode,
	    indentString = _ref.indentString,
	    intro = _ref.intro,
	    outro = _ref.outro;

	var globalNameMaker = getGlobalNameMaker(options.globals || blank(), bundle, 'null');

	var extend = options.extend,
	    name = options.name;

	var isNamespaced = name && name.indexOf('.') !== -1;
	var possibleVariableAssignment = !extend && !isNamespaced;

	if (name && possibleVariableAssignment && !isLegal(name)) {
		error({
			code: 'ILLEGAL_IDENTIFIER_AS_NAME',
			message: 'Given name (' + name + ') is not legal JS identifier. If you need this you can try --extend option'
		});
	}

	warnOnBuiltins(bundle);

	var external = trimEmptyImports(bundle.externalModules);
	var dependencies = external.map(globalNameMaker);
	var args = external.map(function (m) {
		return m.name;
	});

	if (exportMode !== 'none' && !name) {
		error({
			code: 'INVALID_OPTION',
			message: 'You must supply options.name for IIFE bundles'
		});
	}

	if (extend) {
		dependencies.unshift('(' + thisProp(name) + ' = ' + thisProp(name) + ' || {})');
		args.unshift('exports');
	} else if (exportMode === 'named') {
		dependencies.unshift('{}');
		args.unshift('exports');
	}

	var useStrict = options.strict !== false ? indentString + '\'use strict\';\n\n' : '';

	var wrapperIntro = '(function (' + args + ') {\n' + useStrict;

	if (exportMode !== 'none' && !extend) {
		wrapperIntro = (isNamespaced ? thisProp(name) : bundle.varOrConst + ' ' + name) + (' = ' + wrapperIntro);
	}

	if (isNamespaced) {
		wrapperIntro = setupNamespace(name) + wrapperIntro;
	}

	var wrapperOutro = '\n\n}(' + dependencies + '));';

	if (!extend && exportMode === 'named') {
		wrapperOutro = '\n\n' + indentString + 'return exports;' + wrapperOutro;
	}

	// var foo__default = 'default' in foo ? foo['default'] : foo;
	var interopBlock = getInteropBlock(bundle, options);
	if (interopBlock) magicString.prepend(interopBlock + '\n\n');

	if (intro) magicString.prepend(intro);

	var exportBlock = getExportBlock(bundle, exportMode);
	if (exportBlock) magicString.append('\n\n' + exportBlock);
	if (outro) magicString.append(outro);

	return magicString.indent(indentString).prepend(wrapperIntro).append(wrapperOutro);
}

function globalProp(name) {
	if (!name) return 'null';
	return 'global' + keypath(name);
}

function setupNamespace$1(name) {
	var parts = name.split('.');
	var last = property(parts.pop());

	var acc = 'global';
	return parts.map(function (part) {
		return acc += property(part), acc + ' = ' + acc + ' || {}';
	}).concat('' + acc + last).join(', ');
}

function safeAccess(name) {
	var parts = name.split('.');

	var acc = 'global';
	return parts.map(function (part) {
		return acc += property(part), acc;
	}).join(' && ');
}

var wrapperOutro = '\n\n})));';

function umd(bundle, magicString, _ref, options) {
	var exportMode = _ref.exportMode,
	    getPath = _ref.getPath,
	    indentString = _ref.indentString,
	    intro = _ref.intro,
	    outro = _ref.outro;

	if (exportMode !== 'none' && !options.name) {
		error({
			code: 'INVALID_OPTION',
			message: 'You must supply options.name for UMD bundles'
		});
	}

	warnOnBuiltins(bundle);

	var globalNameMaker = getGlobalNameMaker(options.globals || blank(), bundle);

	var amdDeps = bundle.externalModules.map(function (m) {
		return '\'' + getPath(m.id) + '\'';
	});
	var cjsDeps = bundle.externalModules.map(function (m) {
		return 'require(\'' + getPath(m.id) + '\')';
	});

	var trimmed = trimEmptyImports(bundle.externalModules);
	var globalDeps = trimmed.map(function (module) {
		return globalProp(globalNameMaker(module));
	});
	var args = trimmed.map(function (m) {
		return m.name;
	});

	if (exportMode === 'named') {
		amdDeps.unshift('\'exports\'');
		cjsDeps.unshift('exports');
		globalDeps.unshift('(' + setupNamespace$1(options.name) + ' = ' + (options.extend ? globalProp(options.name) + ' || ' : '') + '{})');

		args.unshift('exports');
	}

	var amdOptions = options.amd || {};

	var amdParams = (amdOptions.id ? '\'' + amdOptions.id + '\', ' : '') + (amdDeps.length ? '[' + amdDeps.join(', ') + '], ' : '');

	var define = amdOptions.define || 'define';

	var cjsExport = exportMode === 'default' ? 'module.exports = ' : '';
	var defaultExport = exportMode === 'default' ? setupNamespace$1(options.name) + ' = ' : '';

	var useStrict = options.strict !== false ? ' \'use strict\';' : '';

	var globalExport = void 0;

	if (options.noConflict === true) {
		var factory = void 0;

		if (exportMode === 'default') {
			factory = 'var exports = factory(' + globalDeps + ');';
		} else if (exportMode === 'named') {
			var module = globalDeps.shift();
			factory = 'var exports = ' + module + ';\n\t\t\t\tfactory(' + ['exports'].concat(globalDeps) + ');';
		}
		globalExport = '(function() {\n\t\t\t\tvar current = ' + safeAccess(options.name) + ';\n\t\t\t\t' + factory + '\n\t\t\t\t' + globalProp(options.name) + ' = exports;\n\t\t\t\texports.noConflict = function() { ' + globalProp(options.name) + ' = current; return exports; };\n\t\t\t})()';
	} else {
		globalExport = '(' + defaultExport + 'factory(' + globalDeps + '))';
	}

	var wrapperIntro = ('(function (global, factory) {\n\t\t\ttypeof exports === \'object\' && typeof module !== \'undefined\' ? ' + cjsExport + 'factory(' + cjsDeps.join(', ') + ') :\n\t\t\ttypeof ' + define + ' === \'function\' && ' + define + '.amd ? ' + define + '(' + amdParams + 'factory) :\n\t\t\t' + globalExport + ';\n\t\t}(this, (function (' + args + ') {' + useStrict + '\n\n\t\t').replace(/^\t\t/gm, '').replace(/^\t/gm, indentString || '\t');

	// var foo__default = 'default' in foo ? foo['default'] : foo;
	var interopBlock = getInteropBlock(bundle, options);
	if (interopBlock) magicString.prepend(interopBlock + '\n\n');

	if (intro) magicString.prepend(intro);

	var exportBlock = getExportBlock(bundle, exportMode);
	if (exportBlock) magicString.append('\n\n' + exportBlock);
	if (exportMode === 'named' && options.legacy !== true) magicString.append('\n\n' + esModuleExport);
	if (outro) magicString.append(outro);

	return magicString.trim().indent(indentString).append(wrapperOutro).prepend(wrapperIntro);
}

var finalisers = { amd: amd, cjs: cjs, es: es, iife: iife, umd: umd };

function ensureArray(thing) {
	if (Array.isArray(thing)) return thing;
	if (thing == undefined) return [];
	return [thing];
}

function load(id) {
	return fs.readFileSync(id, 'utf-8');
}

function findFile(file) {
	try {
		var stats = fs.lstatSync(file);
		if (stats.isSymbolicLink()) return findFile(fs.realpathSync(file));
		if (stats.isFile()) {
			// check case
			var name = path.basename(file);
			var files = fs.readdirSync(path.dirname(file));

			if (~files.indexOf(name)) return file;
		}
	} catch (err) {
		// suppress
	}
}

function addJsExtensionIfNecessary(file) {
	return findFile(file) || findFile(file + '.js');
}

function resolveId(importee, importer) {
	if (typeof process === 'undefined') {
		error({
			code: 'MISSING_PROCESS',
			message: 'It looks like you\'re using Rollup in a non-Node.js environment. This means you must supply a plugin with custom resolveId and load functions',
			url: 'https://github.com/rollup/rollup/wiki/Plugins'
		});
	}

	// external modules (non-entry modules that start with neither '.' or '/')
	// are skipped at this stage.
	if (importer !== undefined && !isAbsolute(importee) && importee[0] !== '.') return null;

	// `resolve` processes paths from right to left, prepending them until an
	// absolute path is created. Absolute importees therefore shortcircuit the
	// resolve call and require no special handing on our part.
	// See https://nodejs.org/api/path.html#path_path_resolve_paths
	return addJsExtensionIfNecessary(path.resolve(importer ? path.dirname(importer) : path.resolve(), importee));
}

function makeOnwarn() {
	var warned = blank();

	return function (warning) {
		var str = warning.toString();
		if (str in warned) return;
		console.error(str); //eslint-disable-line no-console
		warned[str] = true;
	};
}

function badExports(option, keys$$1) {
	error({
		code: 'INVALID_EXPORT_OPTION',
		message: '\'' + option + '\' was specified for options.exports, but entry module has following exports: ' + keys$$1.join(', ')
	});
}

function getExportMode(bundle, _ref) {
	var exportMode = _ref.exports,
	    name = _ref.name,
	    format = _ref.format;

	var exportKeys = keys(bundle.entryModule.exports).concat(keys(bundle.entryModule.reexports)).concat(bundle.entryModule.exportAllSources); // not keys, but makes our job easier this way

	if (exportMode === 'default') {
		if (exportKeys.length !== 1 || exportKeys[0] !== 'default') {
			badExports('default', exportKeys);
		}
	} else if (exportMode === 'none' && exportKeys.length) {
		badExports('none', exportKeys);
	}

	if (!exportMode || exportMode === 'auto') {
		if (exportKeys.length === 0) {
			exportMode = 'none';
		} else if (exportKeys.length === 1 && exportKeys[0] === 'default') {
			exportMode = 'default';
		} else {
			if (bundle.entryModule.exports.default && format !== 'es') {
				bundle.warn({
					code: 'MIXED_EXPORTS',
					message: 'Using named and default exports together. Consumers of your bundle will have to use ' + (name || 'bundle') + '[\'default\'] to access the default export, which may not be what you want. Use `exports: \'named\'` to disable this warning',
					url: 'https://github.com/rollup/rollup/wiki/JavaScript-API#exports'
				});
			}
			exportMode = 'named';
		}
	}

	if (!/(?:default|named|none)/.test(exportMode)) {
		error({
			code: 'INVALID_EXPORT_OPTION',
			message: 'options.exports must be \'default\', \'named\', \'none\', \'auto\', or left unspecified (defaults to \'auto\')'
		});
	}

	return exportMode;
}

function getIndentString(magicString, options) {
	if (options.indent === true) {
		return magicString.getIndentString();
	}

	return options.indent || '';
}

function transform(bundle, source, id, plugins) {
	var sourcemapChain = [];

	var originalSourcemap = typeof source.map === 'string' ? JSON.parse(source.map) : source.map;

	if (originalSourcemap && typeof originalSourcemap.mappings === 'string') {
		originalSourcemap.mappings = sourcemapCodec.decode(originalSourcemap.mappings);
	}

	var originalCode = source.code;
	var ast = source.ast;

	var promise = Promise.resolve(source.code);

	plugins.forEach(function (plugin) {
		if (!plugin.transform) return;

		promise = promise.then(function (previous) {
			function augment(object, pos, code) {
				if (typeof object === 'string') {
					object = { message: object };
				}

				if (object.code) object.pluginCode = object.code;
				object.code = code;

				if (pos !== undefined) {
					if (pos.line !== undefined && pos.column !== undefined) {
						var line = pos.line,
						    column = pos.column;

						object.loc = { file: id, line: line, column: column };
						object.frame = getCodeFrame(previous, line, column);
					} else {
						object.pos = pos;

						var _locate = locateCharacter.locate(previous, pos, { offsetLine: 1 }),
						    _line = _locate.line,
						    _column = _locate.column;

						object.loc = { file: id, line: _line, column: _column };
						object.frame = getCodeFrame(previous, _line, _column);
					}
				}

				object.plugin = plugin.name;
				object.id = id;

				return object;
			}

			var throwing = void 0;

			var context = {
				warn: function warn(warning, pos) {
					warning = augment(warning, pos, 'PLUGIN_WARNING');
					bundle.warn(warning);
				},

				error: function error$$1(err, pos) {
					err = augment(err, pos, 'PLUGIN_ERROR');
					throwing = true;
					error(err);
				}
			};

			var transformed = void 0;

			try {
				transformed = plugin.transform.call(context, previous, id);
			} catch (err) {
				if (!throwing) context.error(err);
				error(err);
			}

			return Promise.resolve(transformed).then(function (result) {
				if (result == null) return previous;

				if (typeof result === 'string') {
					result = {
						code: result,
						ast: null,
						map: null
					};
				}

				// `result.map` can only be a string if `result` isn't
				else if (typeof result.map === 'string') {
						result.map = JSON.parse(result.map);
					}

				if (result.map && typeof result.map.mappings === 'string') {
					result.map.mappings = sourcemapCodec.decode(result.map.mappings);
				}

				sourcemapChain.push(result.map || { missing: true, plugin: plugin.name }); // lil' bit hacky but it works
				ast = result.ast;

				return result.code;
			}).catch(function (err) {
				err = augment(err, undefined, 'PLUGIN_ERROR');
				error(err);
			});
		});
	});

	return promise.then(function (code) {
		return { code: code, originalCode: originalCode, originalSourcemap: originalSourcemap, ast: ast, sourcemapChain: sourcemapChain };
	});
}

function transformBundle(code, plugins, sourcemapChain, options) {
	return plugins.reduce(function (promise, plugin) {
		if (!plugin.transformBundle) return promise;

		return promise.then(function (code) {
			return Promise.resolve().then(function () {
				return plugin.transformBundle(code, { format: options.format });
			}).then(function (result) {
				if (result == null) return code;

				if (typeof result === 'string') {
					result = {
						code: result,
						map: null
					};
				}

				var map = typeof result.map === 'string' ? JSON.parse(result.map) : result.map;
				if (map && typeof map.mappings === 'string') {
					map.mappings = sourcemapCodec.decode(map.mappings);
				}

				sourcemapChain.push(map);

				return result.code;
			}).catch(function (err) {
				error({
					code: 'BAD_BUNDLE_TRANSFORMER',
					message: 'Error transforming bundle' + (plugin.name ? ' with \'' + plugin.name + '\' plugin' : '') + ': ' + err.message,
					plugin: plugin.name
				});
			});
		});
	}, Promise.resolve(code));
}

var _createClass$70 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$70(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Source = function () {
	function Source(filename, content) {
		_classCallCheck$70(this, Source);

		this.isOriginal = true;
		this.filename = filename;
		this.content = content;
	}

	_createClass$70(Source, [{
		key: 'traceSegment',
		value: function traceSegment(line, column, name) {
			return { line: line, column: column, name: name, source: this };
		}
	}]);

	return Source;
}();

var Link = function () {
	function Link(map, sources) {
		_classCallCheck$70(this, Link);

		this.sources = sources;
		this.names = map.names;
		this.mappings = map.mappings;
	}

	_createClass$70(Link, [{
		key: 'traceMappings',
		value: function traceMappings() {
			var _this = this;

			var sources = [];
			var sourcesContent = [];
			var names = [];

			var mappings = this.mappings.map(function (line) {
				var tracedLine = [];

				line.forEach(function (segment) {
					var source = _this.sources[segment[1]];

					if (!source) return;

					var traced = source.traceSegment(segment[2], segment[3], _this.names[segment[4]]);

					if (traced) {
						var sourceIndex = null;
						var nameIndex = null;
						segment = [segment[0], null, traced.line, traced.column];

						// newer sources are more likely to be used, so search backwards.
						sourceIndex = sources.lastIndexOf(traced.source.filename);
						if (sourceIndex === -1) {
							sourceIndex = sources.length;
							sources.push(traced.source.filename);
							sourcesContent[sourceIndex] = traced.source.content;
						} else if (sourcesContent[sourceIndex] == null) {
							sourcesContent[sourceIndex] = traced.source.content;
						} else if (traced.source.content != null && sourcesContent[sourceIndex] !== traced.source.content) {
							error({
								message: 'Multiple conflicting contents for sourcemap source ' + source.filename
							});
						}

						segment[1] = sourceIndex;

						if (traced.name) {
							nameIndex = names.indexOf(traced.name);
							if (nameIndex === -1) {
								nameIndex = names.length;
								names.push(traced.name);
							}

							segment[4] = nameIndex;
						}

						tracedLine.push(segment);
					}
				});

				return tracedLine;
			});

			return { sources: sources, sourcesContent: sourcesContent, names: names, mappings: mappings };
		}
	}, {
		key: 'traceSegment',
		value: function traceSegment(line, column, name) {
			var segments = this.mappings[line];

			if (!segments) return null;

			for (var i = 0; i < segments.length; i += 1) {
				var segment = segments[i];

				if (segment[0] > column) return null;

				if (segment[0] === column) {
					var source = this.sources[segment[1]];
					if (!source) return null;

					return source.traceSegment(segment[2], segment[3], this.names[segment[4]] || name);
				}
			}

			return null;
		}
	}]);

	return Link;
}();

function collapseSourcemaps(bundle, file, map, modules, bundleSourcemapChain) {
	var moduleSources = modules.filter(function (module) {
		return !module.excludeFromSourcemap;
	}).map(function (module) {
		var sourcemapChain = module.sourcemapChain;

		var source = void 0;
		if (module.originalSourcemap == null) {
			source = new Source(module.id, module.originalCode);
		} else {
			var _sources = module.originalSourcemap.sources;
			var _sourcesContent = module.originalSourcemap.sourcesContent || [];

			if (_sources == null || _sources.length <= 1 && _sources[0] == null) {
				source = new Source(module.id, _sourcesContent[0]);
				sourcemapChain = [module.originalSourcemap].concat(sourcemapChain);
			} else {
				// TODO indiscriminately treating IDs and sources as normal paths is probably bad.
				var directory = path.dirname(module.id) || '.';
				var sourceRoot = module.originalSourcemap.sourceRoot || '.';

				var baseSources = _sources.map(function (source, i) {
					return new Source(path.resolve(directory, sourceRoot, source), _sourcesContent[i]);
				});

				source = new Link(module.originalSourcemap, baseSources);
			}
		}

		sourcemapChain.forEach(function (map) {
			if (map.missing) {
				bundle.warn({
					code: 'SOURCEMAP_BROKEN',
					plugin: map.plugin,
					message: 'Sourcemap is likely to be incorrect: a plugin' + (map.plugin ? ' (\'' + map.plugin + '\')' : '') + ' was used to transform files, but didn\'t generate a sourcemap for the transformation. Consult the plugin documentation for help',
					url: 'https://github.com/rollup/rollup/wiki/Troubleshooting#sourcemap-is-likely-to-be-incorrect'
				});

				map = {
					names: [],
					mappings: ''
				};
			}

			source = new Link(map, [source]);
		});

		return source;
	});

	var source = new Link(map, moduleSources);

	bundleSourcemapChain.forEach(function (map) {
		source = new Link(map, [source]);
	});

	var _source$traceMappings = source.traceMappings(),
	    sources = _source$traceMappings.sources,
	    sourcesContent = _source$traceMappings.sourcesContent,
	    names = _source$traceMappings.names,
	    mappings = _source$traceMappings.mappings;

	if (file) {
		var directory = path.dirname(file);
		sources = sources.map(function (source) {
			return path.relative(directory, source);
		});

		map.file = path.basename(file);
	}

	// we re-use the `map` object because it has convenient toString/toURL methods
	map.sources = sources;
	map.sourcesContent = sourcesContent;
	map.names = names;
	map.mappings = sourcemapCodec.encode(mappings);

	return map;
}

function callIfFunction(thing) {
	return typeof thing === 'function' ? thing() : thing;
}

var _createClass$72 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$72(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$64(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$64(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var GlobalVariable = function (_Variable) {
	_inherits$64(GlobalVariable, _Variable);

	function GlobalVariable(name) {
		_classCallCheck$72(this, GlobalVariable);

		var _this = _possibleConstructorReturn$64(this, (GlobalVariable.__proto__ || Object.getPrototypeOf(GlobalVariable)).call(this, name));

		_this.isExternal = true;
		_this.isGlobal = true;
		_this.isReassigned = false;
		_this.included = true;
		return _this;
	}

	_createClass$72(GlobalVariable, [{
		key: 'addReference',
		value: function addReference(reference) {
			if (reference.isReassignment) this.isReassigned = true;
		}
	}, {
		key: 'assignExpression',
		value: function assignExpression() {}
	}, {
		key: 'hasEffectsWhenCalled',
		value: function hasEffectsWhenCalled() {
			return !pureFunctions[this.name];
		}
	}]);

	return GlobalVariable;
}(Variable);

var _createClass$71 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$71(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$63(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$63(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BundleScope = function (_Scope) {
	_inherits$63(BundleScope, _Scope);

	function BundleScope() {
		_classCallCheck$71(this, BundleScope);

		return _possibleConstructorReturn$63(this, (BundleScope.__proto__ || Object.getPrototypeOf(BundleScope)).apply(this, arguments));
	}

	_createClass$71(BundleScope, [{
		key: 'findVariable',
		value: function findVariable(name) {
			if (!this.variables[name]) {
				this.variables[name] = new GlobalVariable(name);
			}

			return this.variables[name];
		}
	}]);

	return BundleScope;
}(Scope);

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Bundle$1 = function () {
	function Bundle$$1(options) {
		var _this = this;

		_classCallCheck(this, Bundle$$1);

		this.cachedModules = new Map();
		if (options.cache) {
			options.cache.modules.forEach(function (module) {
				_this.cachedModules.set(module.id, module);
			});
		}

		this.plugins = ensureArray(options.plugins);

		options = this.plugins.reduce(function (acc, plugin) {
			if (plugin.options) return plugin.options(acc) || acc;
			return acc;
		}, options);

		if (!options.input) {
			throw new Error('You must supply options.input to rollup');
		}

		this.entry = options.input;
		this.entryId = null;
		this.entryModule = null;

		this.treeshake = options.treeshake !== false;

		if (options.pureExternalModules === true) {
			this.isPureExternalModule = function () {
				return true;
			};
		} else if (typeof options.pureExternalModules === 'function') {
			this.isPureExternalModule = options.pureExternalModules;
		} else if (Array.isArray(options.pureExternalModules)) {
			var pureExternalModules = new Set(options.pureExternalModules);
			this.isPureExternalModule = function (id) {
				return pureExternalModules.has(id);
			};
		} else {
			this.isPureExternalModule = function () {
				return false;
			};
		}

		this.resolveId = first([function (id) {
			return _this.isExternal(id) ? false : null;
		}].concat(this.plugins.map(function (plugin) {
			return plugin.resolveId;
		}).filter(Boolean)).concat(resolveId));

		var loaders = this.plugins.map(function (plugin) {
			return plugin.load;
		}).filter(Boolean);
		this.hasLoaders = loaders.length !== 0;
		this.load = first(loaders.concat(load));

		this.scope = new BundleScope();
		// TODO strictly speaking, this only applies with non-ES6, non-default-only bundles
		['module', 'exports', '_interopDefault'].forEach(function (name) {
			_this.scope.findVariable(name); // creates global variable as side-effect
		});

		this.moduleById = new Map();
		this.modules = [];
		this.externalModules = [];

		this.context = String(options.context);

		var optionsModuleContext = options.moduleContext;
		if (typeof optionsModuleContext === 'function') {
			this.getModuleContext = function (id) {
				return optionsModuleContext(id) || _this.context;
			};
		} else if ((typeof optionsModuleContext === 'undefined' ? 'undefined' : _typeof(optionsModuleContext)) === 'object') {
			var moduleContext = new Map();
			Object.keys(optionsModuleContext).forEach(function (key) {
				moduleContext.set(path.resolve(key), optionsModuleContext[key]);
			});
			this.getModuleContext = function (id) {
				return moduleContext.get(id) || _this.context;
			};
		} else {
			this.getModuleContext = function () {
				return _this.context;
			};
		}

		if (typeof options.external === 'function') {
			this.isExternal = options.external;
		} else {
			var ids = ensureArray(options.external);
			this.isExternal = function (id) {
				return ids.indexOf(id) !== -1;
			};
		}

		this.onwarn = options.onwarn || makeOnwarn();

		this.varOrConst = options.preferConst ? 'const' : 'var';
		this.legacy = options.legacy;
		this.acornOptions = options.acorn || {};
	}

	_createClass(Bundle$$1, [{
		key: 'build',
		value: function build() {
			var _this2 = this;

			// Phase 1 – discovery. We load the entry module and find which
			// modules it imports, and import those, until we have all
			// of the entry module's dependencies
			return this.resolveId(this.entry, undefined).then(function (id) {
				if (id === false) {
					error({
						code: 'UNRESOLVED_ENTRY',
						message: 'Entry module cannot be external'
					});
				}

				if (id == null) {
					error({
						code: 'UNRESOLVED_ENTRY',
						message: 'Could not resolve entry (' + _this2.entry + ')'
					});
				}

				_this2.entryId = id;
				return _this2.fetchModule(id, undefined);
			}).then(function (entryModule) {
				_this2.entryModule = entryModule;

				// Phase 2 – binding. We link references to their variables
				// to generate a complete picture of the bundle

				timeStart('phase 2');

				_this2.modules.forEach(function (module) {
					return module.bindImportSpecifiers();
				});
				_this2.modules.forEach(function (module) {
					return module.bindReferences();
				});

				timeEnd('phase 2');

				// Phase 3 – marking. We include all statements that should be included

				timeStart('phase 3');

				// mark all export statements
				entryModule.getExports().forEach(function (name) {
					var variable = entryModule.traceExport(name);

					variable.exportName = name;
					variable.includeVariable();

					if (variable.isNamespace) {
						variable.needsNamespaceBlock = true;
					}
				});

				entryModule.getReexports().forEach(function (name) {
					var variable = entryModule.traceExport(name);

					if (variable.isExternal) {
						variable.reexported = variable.module.reexported = true;
					} else {
						variable.exportName = name;
						variable.includeVariable();
					}
				});

				// mark statements that should appear in the bundle
				if (_this2.treeshake) {
					var addedNewNodes = void 0;
					do {
						addedNewNodes = false;
						_this2.modules.forEach(function (module) {
							if (module.includeInBundle()) {
								addedNewNodes = true;
							}
						});
					} while (addedNewNodes);
				} else {
					// Necessary to properly replace namespace imports
					_this2.modules.forEach(function (module) {
						return module.includeAllInBundle();
					});
				}

				timeEnd('phase 3');

				// Phase 4 – final preparation. We order the modules with an
				// enhanced topological sort that accounts for cycles, then
				// ensure that names are deconflicted throughout the bundle

				timeStart('phase 4');

				// while we're here, check for unused external imports
				_this2.externalModules.forEach(function (module) {
					var unused = Object.keys(module.declarations).filter(function (name) {
						return name !== '*';
					}).filter(function (name) {
						return !module.declarations[name].included && !module.declarations[name].reexported;
					});

					if (unused.length === 0) return;

					var names = unused.length === 1 ? '\'' + unused[0] + '\' is' : unused.slice(0, -1).map(function (name) {
						return '\'' + name + '\'';
					}).join(', ') + ' and \'' + unused.slice(-1) + '\' are';

					_this2.warn({
						code: 'UNUSED_EXTERNAL_IMPORT',
						source: module.id,
						names: unused,
						message: names + ' imported from external module \'' + module.id + '\' but never used'
					});
				});

				// prune unused external imports
				_this2.externalModules = _this2.externalModules.filter(function (module) {
					return module.used || !_this2.isPureExternalModule(module.id);
				});

				_this2.orderedModules = _this2.sort();
				_this2.deconflict();

				timeEnd('phase 4');
			});
		}
	}, {
		key: 'deconflict',
		value: function deconflict() {
			var used = blank();

			// ensure no conflicts with globals
			keys(this.scope.variables).forEach(function (name) {
				return used[name] = 1;
			});

			function getSafeName(name) {
				while (used[name]) {
					name += '$' + used[name]++;
				}

				used[name] = 1;
				return name;
			}

			var toDeshadow = new Set();

			this.externalModules.forEach(function (module) {
				var safeName = getSafeName(module.name);
				toDeshadow.add(safeName);
				module.name = safeName;

				// ensure we don't shadow named external imports, if
				// we're creating an ES6 bundle
				forOwn(module.declarations, function (declaration, name) {
					var safeName = getSafeName(name);
					toDeshadow.add(safeName);
					declaration.setSafeName(safeName);
				});
			});

			this.modules.forEach(function (module) {
				forOwn(module.scope.variables, function (variable) {
					if (variable.isDefault && variable.declaration.id) {
						return;
					}

					variable.name = getSafeName(variable.name);
				});

				// deconflict reified namespaces
				var namespace = module.namespace();
				if (namespace.needsNamespaceBlock) {
					namespace.name = getSafeName(namespace.name);
				}
			});

			this.scope.deshadow(toDeshadow);
		}
	}, {
		key: 'fetchModule',
		value: function fetchModule(id, importer) {
			var _this3 = this;

			// short-circuit cycles
			if (this.moduleById.has(id)) return null;
			this.moduleById.set(id, null);

			return this.load(id).catch(function (err) {
				var msg = 'Could not load ' + id;
				if (importer) msg += ' (imported by ' + importer + ')';

				msg += ': ' + err.message;
				throw new Error(msg);
			}).then(function (source) {
				if (typeof source === 'string') return source;
				if (source && (typeof source === 'undefined' ? 'undefined' : _typeof(source)) === 'object' && source.code) return source;

				// TODO report which plugin failed
				error({
					code: 'BAD_LOADER',
					message: 'Error loading ' + relativeId(id) + ': plugin load hook should return a string, a { code, map } object, or nothing/null'
				});
			}).then(function (source) {
				if (typeof source === 'string') {
					source = {
						code: source,
						ast: null
					};
				}

				if (_this3.cachedModules.has(id) && _this3.cachedModules.get(id).originalCode === source.code) {
					return _this3.cachedModules.get(id);
				}

				return transform(_this3, source, id, _this3.plugins);
			}).then(function (source) {
				var code = source.code,
				    originalCode = source.originalCode,
				    originalSourcemap = source.originalSourcemap,
				    ast = source.ast,
				    sourcemapChain = source.sourcemapChain,
				    resolvedIds = source.resolvedIds;


				var module = new Module({
					id: id,
					code: code,
					originalCode: originalCode,
					originalSourcemap: originalSourcemap,
					ast: ast,
					sourcemapChain: sourcemapChain,
					resolvedIds: resolvedIds,
					bundle: _this3
				});

				_this3.modules.push(module);
				_this3.moduleById.set(id, module);

				return _this3.fetchAllDependencies(module).then(function () {
					keys(module.exports).forEach(function (name) {
						if (name !== 'default') {
							module.exportsAll[name] = module.id;
						}
					});
					module.exportAllSources.forEach(function (source) {
						var id = module.resolvedIds[source] || module.resolvedExternalIds[source];
						var exportAllModule = _this3.moduleById.get(id);
						if (exportAllModule.isExternal) return;

						keys(exportAllModule.exportsAll).forEach(function (name) {
							if (name in module.exportsAll) {
								_this3.warn({
									code: 'NAMESPACE_CONFLICT',
									reexporter: module.id,
									name: name,
									sources: [module.exportsAll[name], exportAllModule.exportsAll[name]],
									message: 'Conflicting namespaces: ' + relativeId(module.id) + ' re-exports \'' + name + '\' from both ' + relativeId(module.exportsAll[name]) + ' and ' + relativeId(exportAllModule.exportsAll[name]) + ' (will be ignored)'
								});
							} else {
								module.exportsAll[name] = exportAllModule.exportsAll[name];
							}
						});
					});
					return module;
				});
			});
		}
	}, {
		key: 'fetchAllDependencies',
		value: function fetchAllDependencies(module) {
			var _this4 = this;

			return mapSequence(module.sources, function (source) {
				var resolvedId = module.resolvedIds[source];
				return (resolvedId ? Promise.resolve(resolvedId) : _this4.resolveId(source, module.id)).then(function (resolvedId) {
					var externalId = resolvedId || (isRelative(source) ? path.resolve(module.id, '..', source) : source);
					var isExternal = _this4.isExternal(externalId);

					if (!resolvedId && !isExternal) {
						if (isRelative(source)) {
							error({
								code: 'UNRESOLVED_IMPORT',
								message: 'Could not resolve \'' + source + '\' from ' + relativeId(module.id)
							});
						}

						_this4.warn({
							code: 'UNRESOLVED_IMPORT',
							source: source,
							importer: relativeId(module.id),
							message: '\'' + source + '\' is imported by ' + relativeId(module.id) + ', but could not be resolved \u2013 treating it as an external dependency',
							url: 'https://github.com/rollup/rollup/wiki/Troubleshooting#treating-module-as-external-dependency'
						});
						isExternal = true;
					}

					if (isExternal) {
						module.resolvedExternalIds[source] = externalId;

						if (!_this4.moduleById.has(externalId)) {
							var _module = new ExternalModule(externalId);
							_this4.externalModules.push(_module);
							_this4.moduleById.set(externalId, _module);
						}

						var externalModule = _this4.moduleById.get(externalId);

						// add external declarations so we can detect which are never used
						Object.keys(module.imports).forEach(function (name) {
							var importDeclaration = module.imports[name];
							if (importDeclaration.source !== source) return;

							externalModule.traceExport(importDeclaration.name);
						});
					} else {
						if (resolvedId === module.id) {
							// need to find the actual import declaration, so we can provide
							// a useful error message. Bit hoop-jumpy but what can you do
							var declaration = module.ast.body.find(function (node) {
								return (node.isImportDeclaration || node.isExportDeclaration) && node.source.value === source;
							});
							var declarationType = /Export/.test(declaration.type) ? 'export' : 'import';
							module.error({
								code: 'CANNOT_IMPORT_SELF',
								message: 'A module cannot ' + declarationType + ' itself'
							}, declaration.start);
						}

						module.resolvedIds[source] = resolvedId;
						return _this4.fetchModule(resolvedId, module.id);
					}
				});
			});
		}
	}, {
		key: 'getPathRelativeToEntryDirname',
		value: function getPathRelativeToEntryDirname(resolvedId) {
			if (isRelative(resolvedId) || isAbsolute(resolvedId)) {
				var entryDirname = path.dirname(this.entryId);
				var relativeToEntry = normalize(path.relative(entryDirname, resolvedId));

				return isRelative(relativeToEntry) ? relativeToEntry : './' + relativeToEntry;
			}

			return resolvedId;
		}
	}, {
		key: 'render',
		value: function render() {
			var _this5 = this;

			var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

			return Promise.resolve().then(function () {
				// Determine export mode - 'default', 'named', 'none'
				var exportMode = getExportMode(_this5, options);

				var magicString = new MagicString.Bundle({ separator: '\n\n' });
				var usedModules = [];

				timeStart('render modules');

				_this5.orderedModules.forEach(function (module) {
					var source = module.render(options.format === 'es', _this5.legacy);
					if (source.toString().length) {
						magicString.addSource(source);
						usedModules.push(module);
					}
				});

				if (!magicString.toString().trim() && _this5.entryModule.getExports().length === 0 && _this5.entryModule.getReexports().length === 0) {
					_this5.warn({
						code: 'EMPTY_BUNDLE',
						message: 'Generated an empty bundle'
					});
				}

				timeEnd('render modules');

				var intro = [options.intro].concat(_this5.plugins.map(function (plugin) {
					return plugin.intro && plugin.intro();
				})).filter(Boolean).join('\n\n');

				if (intro) intro += '\n\n';

				var outro = [options.outro].concat(_this5.plugins.map(function (plugin) {
					return plugin.outro && plugin.outro();
				})).filter(Boolean).join('\n\n');

				if (outro) outro = '\n\n' + outro;

				var indentString = getIndentString(magicString, options);

				var finalise = finalisers[options.format];
				if (!finalise) {
					error({
						code: 'INVALID_OPTION',
						message: 'Invalid format: ' + options.format + ' - valid options are ' + keys(finalisers).join(', ')
					});
				}

				timeStart('render format');

				var optionsPaths = options.paths;
				var getPath = typeof optionsPaths === 'function' ? function (id) {
					return optionsPaths(id) || _this5.getPathRelativeToEntryDirname(id);
				} : optionsPaths ? function (id) {
					return optionsPaths.hasOwnProperty(id) ? optionsPaths[id] : _this5.getPathRelativeToEntryDirname(id);
				} : function (id) {
					return _this5.getPathRelativeToEntryDirname(id);
				};

				magicString = finalise(_this5, magicString.trim(), { exportMode: exportMode, getPath: getPath, indentString: indentString, intro: intro, outro: outro }, options);

				timeEnd('render format');

				var banner = [options.banner].concat(_this5.plugins.map(function (plugin) {
					return plugin.banner;
				})).map(callIfFunction).filter(Boolean).join('\n');

				var footer = [options.footer].concat(_this5.plugins.map(function (plugin) {
					return plugin.footer;
				})).map(callIfFunction).filter(Boolean).join('\n');

				if (banner) magicString.prepend(banner + '\n');
				if (footer) magicString.append('\n' + footer);

				var prevCode = magicString.toString();
				var map = null;
				var bundleSourcemapChain = [];

				return transformBundle(prevCode, _this5.plugins, bundleSourcemapChain, options).then(function (code) {
					if (options.sourcemap) {
						timeStart('sourcemap');

						var file = options.sourcemapFile || options.file;
						if (file) file = path.resolve(typeof process !== 'undefined' ? process.cwd() : '', file);

						if (_this5.hasLoaders || find(_this5.plugins, function (plugin) {
							return plugin.transform || plugin.transformBundle;
						})) {
							map = magicString.generateMap({});
							if (typeof map.mappings === 'string') {
								map.mappings = sourcemapCodec.decode(map.mappings);
							}
							map = collapseSourcemaps(_this5, file, map, usedModules, bundleSourcemapChain);
						} else {
							map = magicString.generateMap({ file: file, includeContent: true });
						}

						map.sources = map.sources.map(normalize);

						timeEnd('sourcemap');
					}

					if (code[code.length - 1] !== '\n') code += '\n';
					return { code: code, map: map };
				});
			});
		}
	}, {
		key: 'sort',
		value: function sort() {
			var _this6 = this;

			var hasCycles = void 0;
			var seen = {};
			var ordered = [];

			var stronglyDependsOn = blank();
			var dependsOn = blank();

			this.modules.forEach(function (module) {
				stronglyDependsOn[module.id] = blank();
				dependsOn[module.id] = blank();
			});

			this.modules.forEach(function (module) {
				function processStrongDependency(dependency) {
					if (dependency === module || stronglyDependsOn[module.id][dependency.id]) return;

					stronglyDependsOn[module.id][dependency.id] = true;
					dependency.strongDependencies.forEach(processStrongDependency);
				}

				function processDependency(dependency) {
					if (dependency === module || dependsOn[module.id][dependency.id]) return;

					dependsOn[module.id][dependency.id] = true;
					dependency.dependencies.forEach(processDependency);
				}

				module.strongDependencies.forEach(processStrongDependency);
				module.dependencies.forEach(processDependency);
			});

			var visit = function visit(module) {
				if (seen[module.id]) {
					hasCycles = true;
					return;
				}

				seen[module.id] = true;

				module.dependencies.forEach(visit);
				ordered.push(module);
			};

			visit(this.entryModule);

			if (hasCycles) {
				ordered.forEach(function (a, i) {
					var _loop = function _loop() {
						var b = ordered[i];

						// TODO reinstate this! it no longer works
						if (stronglyDependsOn[a.id][b.id]) {
							// somewhere, there is a module that imports b before a. Because
							// b imports a, a is placed before b. We need to find the module
							// in question, so we can provide a useful error message
							var parent = '[[unknown]]';
							var visited = {};

							var findParent = function findParent(module) {
								if (dependsOn[module.id][a.id] && dependsOn[module.id][b.id]) {
									parent = module.id;
									return true;
								}
								visited[module.id] = true;
								for (var _i = 0; _i < module.dependencies.length; _i += 1) {
									var dependency = module.dependencies[_i];
									if (!visited[dependency.id] && findParent(dependency)) return true;
								}
							};

							findParent(_this6.entryModule);

							_this6.onwarn('Module ' + a.id + ' may be unable to evaluate without ' + b.id + ', but is included first due to a cyclical dependency. Consider swapping the import statements in ' + parent + ' to ensure correct ordering');
						}
					};

					for (i += 1; i < ordered.length; i += 1) {
						_loop();
					}
				});
			}

			return ordered;
		}
	}, {
		key: 'warn',
		value: function warn(warning) {
			warning.toString = function () {
				var str = '';

				if (warning.plugin) str += '(' + warning.plugin + ' plugin) ';
				if (warning.loc) str += relativeId(warning.loc.file) + ' (' + warning.loc.line + ':' + warning.loc.column + ') ';
				str += warning.message;

				return str;
			};

			this.onwarn(warning);
		}
	}]);

	return Bundle$$1;
}();

var ALLOWED_KEYS = ['acorn', 'amd', 'banner', 'cache', 'context', 'entry', 'exports', 'extend', 'external', 'file', 'footer', 'format', 'globals', 'indent', 'input', 'interop', 'intro', 'legacy', 'moduleContext', 'name', 'noConflict', 'onwarn', 'output', 'outro', 'paths', 'plugins', 'preferConst', 'pureExternalModules', 'sourcemap', 'sourcemapFile', 'strict', 'targets', 'treeshake', 'watch'];

function checkAmd(options) {
	if (options.moduleId) {
		if (options.amd) throw new Error('Cannot have both options.amd and options.moduleId');

		options.amd = { id: options.moduleId };
		delete options.moduleId;

		var message = 'options.moduleId is deprecated in favour of options.amd = { id: moduleId }';
		if (options.onwarn) {
			options.onwarn({ message: message });
		} else {
			console.warn(message); // eslint-disable-line no-console
		}
	}
}

function checkInputOptions(options, warn) {
	if (options.transform || options.load || options.resolveId || options.resolveExternal) {
		throw new Error('The `transform`, `load`, `resolveId` and `resolveExternal` options are deprecated in favour of a unified plugin API. See https://github.com/rollup/rollup/wiki/Plugins for details');
	}

	if (options.entry && !options.input) {
		options.input = options.entry;
		warn({
			message: 'options.entry is deprecated, use options.input'
		});
	}

	var err = validateKeys(keys(options), ALLOWED_KEYS);
	if (err) throw err;
}

var deprecated = {
	dest: 'file',
	moduleName: 'name',
	sourceMap: 'sourcemap',
	sourceMapFile: 'sourcemapFile',
	useStrict: 'strict'
};

function checkOutputOptions(options, warn) {
	if (options.format === 'es6') {
		error({
			message: 'The `es6` output format is deprecated – use `es` instead',
			url: 'https://github.com/rollup/rollup/wiki/JavaScript-API#format'
		});
	}

	if (!options.format) {
		error({
			message: 'You must specify options.format, which can be one of \'amd\', \'cjs\', \'es\', \'iife\' or \'umd\'',
			url: 'https://github.com/rollup/rollup/wiki/JavaScript-API#format'
		});
	}

	if (options.moduleId) {
		if (options.amd) throw new Error('Cannot have both options.amd and options.moduleId');

		options.amd = { id: options.moduleId };
		delete options.moduleId;

		warn({
			message: 'options.moduleId is deprecated in favour of options.amd = { id: moduleId }'
		});
	}

	var deprecations = [];
	Object.keys(deprecated).forEach(function (old) {
		if (old in options) {
			deprecations.push({ old: old, new: deprecated[old] });
			options[deprecated[old]] = options[old];
			delete options[old];
		}
	});

	if (deprecations.length) {
		var message = 'The following options have been renamed \u2014 please update your config: ' + deprecations.map(function (option) {
			return option.old + ' -> ' + option.new;
		}).join(', ');
		warn({
			code: 'DEPRECATED_OPTIONS',
			message: message,
			deprecations: deprecations
		});
	}
}

var throwAsyncGenerateError = {
	get: function get() {
		throw new Error('bundle.generate(...) now returns a Promise instead of a { code, map } object');
	}
};

function rollup(options) {
	try {
		if (!options) {
			throw new Error('You must supply an options object to rollup');
		}

		var warn = options.onwarn || function (warning) {
			return console.warn(warning.message);
		}; // eslint-disable-line no-console

		checkInputOptions(options, warn);
		var bundle = new Bundle$1(options);

		timeStart('--BUILD--');

		return bundle.build().then(function () {
			timeEnd('--BUILD--');

			function generate(options) {
				if (!options) {
					throw new Error('You must supply an options object');
				}
				checkOutputOptions(options, warn);
				checkAmd(options);

				timeStart('--GENERATE--');

				var promise = Promise.resolve().then(function () {
					return bundle.render(options);
				}).then(function (rendered) {
					timeEnd('--GENERATE--');

					bundle.plugins.forEach(function (plugin) {
						if (plugin.ongenerate) {
							plugin.ongenerate(assign({
								bundle: result
							}, options), rendered);
						}
					});

					flushTime();

					return rendered;
				});

				Object.defineProperty(promise, 'code', throwAsyncGenerateError);
				Object.defineProperty(promise, 'map', throwAsyncGenerateError);

				return promise;
			}

			var result = {
				imports: bundle.externalModules.map(function (module) {
					return module.id;
				}),
				exports: keys(bundle.entryModule.exports),
				modules: bundle.orderedModules.map(function (module) {
					return module.toJSON();
				}),

				generate: generate,
				write: function write(options) {
					if (!options || !options.file && !options.dest) {
						error({
							code: 'MISSING_OPTION',
							message: 'You must specify options.file'
						});
					}

					return generate(options).then(function (result) {
						var file = options.file;
						var code = result.code,
						    map = result.map;


						var promises = [];

						if (options.sourcemap) {
							var url = void 0;

							if (options.sourcemap === 'inline') {
								url = map.toUrl();
							} else {
								url = path.basename(file) + '.map';
								promises.push(writeFile$1(file + '.map', map.toString()));
							}

							code += '//# ' + SOURCEMAPPING_URL + '=' + url + '\n';
						}

						promises.push(writeFile$1(file, code));
						return Promise.all(promises).then(function () {
							return mapSequence(bundle.plugins.filter(function (plugin) {
								return plugin.onwrite;
							}), function (plugin) {
								return Promise.resolve(plugin.onwrite(assign({
									bundle: result
								}, options), result));
							});
						});
					});
				}
			};

			return result;
		});
	} catch (err) {
		return Promise.reject(err);
	}
}

var chokidar = void 0;

try {
	chokidar = relative$1('chokidar', process.cwd());
} catch (err) {
	chokidar = null;
}

var chokidar$1 = chokidar;

var _createClass$74 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$74(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var opts = { encoding: 'utf-8', persistent: true };

var watchers = new Map();

function addTask(id, task, chokidarOptions, chokidarOptionsHash) {
	if (!watchers.has(chokidarOptionsHash)) watchers.set(chokidarOptionsHash, new Map());
	var group = watchers.get(chokidarOptionsHash);

	if (!group.has(id)) {
		var watcher = new FileWatcher(id, chokidarOptions, function () {
			group.delete(id);
		});

		if (watcher.fileExists) {
			group.set(id, watcher);
		} else {
			return;
		}
	}

	group.get(id).tasks.add(task);
}

function deleteTask(id, target, chokidarOptionsHash) {
	var group = watchers.get(chokidarOptionsHash);

	var watcher = group.get(id);
	if (watcher) {
		watcher.tasks.delete(target);

		if (watcher.tasks.size === 0) {
			watcher.close();
			group.delete(id);
		}
	}
}

var FileWatcher = function () {
	function FileWatcher(id, chokidarOptions, dispose) {
		var _this = this;

		_classCallCheck$74(this, FileWatcher);

		this.tasks = new Set();

		var data = void 0;

		try {
			fs.statSync(id);
			this.fileExists = true;
		} catch (err) {
			if (err.code === 'ENOENT') {
				// can't watch files that don't exist (e.g. injected
				// by plugins somehow)
				this.fileExists = false;
				return;
			} else {
				throw err;
			}
		}

		var handleWatchEvent = function handleWatchEvent(event) {
			if (event === 'rename' || event === 'unlink') {
				_this.fsWatcher.close();
				_this.trigger();
				dispose();
			} else {
				// this is necessary because we get duplicate events...
				var contents = fs.readFileSync(id, 'utf-8');
				if (contents !== data) {
					data = contents;
					_this.trigger();
				}
			}
		};

		if (chokidarOptions) {
			this.fsWatcher = chokidar$1.watch(id, chokidarOptions).on('all', handleWatchEvent);
		} else {
			this.fsWatcher = fs.watch(id, opts, handleWatchEvent);
		}
	}

	_createClass$74(FileWatcher, [{
		key: 'close',
		value: function close() {
			this.fsWatcher.close();
		}
	}, {
		key: 'trigger',
		value: function trigger() {
			this.tasks.forEach(function (task) {
				task.makeDirty();
			});
		}
	}]);

	return FileWatcher;
}();

var _createClass$73 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$73(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$65(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$65(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var DELAY = 100;

var Watcher = function (_EventEmitter) {
	_inherits$65(Watcher, _EventEmitter);

	function Watcher(configs) {
		_classCallCheck$73(this, Watcher);

		var _this = _possibleConstructorReturn$65(this, (Watcher.__proto__ || Object.getPrototypeOf(Watcher)).call(this));

		_this.dirty = true;
		_this.running = false;
		_this.tasks = ensureArray(configs).map(function (config) {
			return new Task(_this, config);
		});
		_this.succeeded = false;

		process.nextTick(function () {
			_this._run();
		});
		return _this;
	}

	_createClass$73(Watcher, [{
		key: 'close',
		value: function close() {
			this.tasks.forEach(function (task) {
				task.close();
			});

			this.removeAllListeners();
		}
	}, {
		key: '_makeDirty',
		value: function _makeDirty() {
			var _this2 = this;

			if (this.dirty) return;
			this.dirty = true;

			if (!this.running) {
				setTimeout(function () {
					_this2._run();
				}, DELAY);
			}
		}
	}, {
		key: '_run',
		value: function _run() {
			var _this3 = this;

			this.running = true;
			this.dirty = false;

			this.emit('event', {
				code: 'START'
			});

			mapSequence(this.tasks, function (task) {
				return task.run();
			}).then(function () {
				_this3.succeeded = true;

				_this3.emit('event', {
					code: 'END'
				});
			}).catch(function (error) {
				_this3.emit('event', {
					code: _this3.succeeded ? 'ERROR' : 'FATAL',
					error: error
				});
			}).then(function () {
				_this3.running = false;

				if (_this3.dirty) {
					_this3._run();
				}
			});
		}
	}]);

	return Watcher;
}(EventEmitter);

var Task = function () {
	function Task(watcher, config) {
		_classCallCheck$73(this, Task);

		this.cache = null;
		this.watcher = watcher;

		this.dirty = true;
		this.closed = false;
		this.watched = new Set();

		this.inputOptions = {
			input: config.input,
			entry: config.input, // legacy, for e.g. commonjs plugin
			legacy: config.legacy,
			treeshake: config.treeshake,
			plugins: config.plugins,
			external: config.external,
			onwarn: config.onwarn || function (warning) {
				return console.warn(warning.message);
			}, // eslint-disable-line no-console
			acorn: config.acorn,
			context: config.context,
			moduleContext: config.moduleContext
		};

		var baseOutputOptions = {
			extend: config.extend,
			exports: config.exports,
			amd: config.amd,
			banner: config.banner,
			footer: config.footer,
			intro: config.intro,
			outro: config.outro,
			sourcemap: config.sourcemap,
			sourcemapFile: config.sourcemapFile,
			name: config.name,
			globals: config.globals,
			interop: config.interop,
			legacy: config.legacy,
			indent: config.indent,
			strict: config.strict,
			noConflict: config.noConflict,
			paths: config.paths,
			preferConst: config.preferConst
		};

		this.outputs = ensureArray(config.output).map(function (output) {
			return Object.assign({}, baseOutputOptions, output);
		});
		this.outputFiles = this.outputs.map(function (output) {
			return path__default.resolve(output.file);
		});

		var watchOptions = config.watch || {};
		if ('useChokidar' in watchOptions) watchOptions.chokidar = watchOptions.useChokidar;
		var chokidarOptions = 'chokidar' in watchOptions ? watchOptions.chokidar : !!chokidar$1;
		if (chokidarOptions) {
			chokidarOptions = Object.assign(chokidarOptions === true ? {} : chokidarOptions, {
				ignoreInitial: true
			});
		}

		if (chokidarOptions && !chokidar$1) {
			throw new Error('options.watch.chokidar was provided, but chokidar could not be found. Have you installed it?');
		}

		this.chokidarOptions = chokidarOptions;
		this.chokidarOptionsHash = JSON.stringify(chokidarOptions);

		this.filter = createFilter(watchOptions.include, watchOptions.exclude);
		this.deprecations = watchOptions._deprecations;
	}

	_createClass$73(Task, [{
		key: 'close',
		value: function close() {
			var _this4 = this;

			this.closed = true;
			this.watched.forEach(function (id) {
				deleteTask(id, _this4, _this4.chokidarOptionsHash);
			});
		}
	}, {
		key: 'makeDirty',
		value: function makeDirty() {
			if (!this.dirty) {
				this.dirty = true;
				this.watcher._makeDirty();
			}
		}
	}, {
		key: 'run',
		value: function run() {
			var _this5 = this;

			if (!this.dirty) return;
			this.dirty = false;

			var options = Object.assign(this.inputOptions, {
				cache: this.cache
			});

			var start = Date.now();

			this.watcher.emit('event', {
				code: 'BUNDLE_START',
				input: this.inputOptions.input,
				output: this.outputFiles
			});

			if (this.deprecations) {
				this.inputOptions.onwarn({
					code: 'DEPRECATED_OPTIONS',
					deprecations: this.deprecations
				});
			}

			return rollup(options).then(function (bundle) {
				if (_this5.closed) return;

				_this5.cache = bundle;

				var watched = new Set();

				bundle.modules.forEach(function (module) {
					watched.add(module.id);
					_this5.watchFile(module.id);
				});

				_this5.watched.forEach(function (id) {
					if (!watched.has(id)) deleteTask(id, _this5, _this5.chokidarOptionsHash);
				});

				_this5.watched = watched;

				return Promise.all(_this5.outputs.map(function (output) {
					return bundle.write(output);
				}));
			}).then(function () {
				_this5.watcher.emit('event', {
					code: 'BUNDLE_END',
					input: _this5.inputOptions.input,
					output: _this5.outputFiles,
					duration: Date.now() - start
				});
			}).catch(function (error) {
				if (_this5.closed) return;

				if (_this5.cache) {
					_this5.cache.modules.forEach(function (module) {
						// this is necessary to ensure that any 'renamed' files
						// continue to be watched following an error
						_this5.watchFile(module.id);
					});
				}
				throw error;
			});
		}
	}, {
		key: 'watchFile',
		value: function watchFile(id) {
			if (!this.filter(id)) return;

			if (this.outputFiles.some(function (file) {
				return file === id;
			})) {
				throw new Error('Cannot import the generated bundle');
			}

			// this is necessary to ensure that any 'renamed' files
			// continue to be watched following an error
			addTask(id, this, this.chokidarOptions, this.chokidarOptionsHash);
		}
	}]);

	return Task;
}();

function watch$1(configs) {
	return new Watcher(configs);
}

var version = "0.50.0";

exports.rollup = rollup;
exports.watch = watch$1;
exports.VERSION = version;
//# sourceMappingURL=rollup.js.map
